@@ -21,15 +21,19 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __CMD_HELP_H__
 #define __CMD_HELP_H__
 
+class gpsimObject;
+
 class cmd_help : public command
 {
 public:
 
-  cmd_help(void);
+  cmd_help();
 
-  void help(void);
-  void help(char *cmd);
-  virtual int is_repeatable(void) { return 1; };
+  void help();
+  void help(const char *cmd);
+  void help(gpsimObject *);
+
+  virtual int is_repeatable() { return 1; };
 
 };
 
