@@ -28,6 +28,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../src/pic-processor.h"
 #include "../src/modules.h"
+#include "../src/symbol.h"
 #include "../src/cmd_manager.h"
 
 
@@ -53,10 +54,9 @@ static cmd_options cmd_module_options[] =
 };
 
 
-cmd_module::cmd_module(void)
+cmd_module::cmd_module()
+  : command("module","mod")
 { 
-  name = "module";
-  abbreviation = "mod";
 
   brief_doc = string("Select & Display modules");
 
@@ -100,13 +100,21 @@ cmd_module::cmd_module(void)
   op = cmd_module_options; 
 }
 
+static void dumpModules(const SymbolTableEntry_t &st)
+{
+  cout << " Module: " << st.first << endl;
+}
 
 void cmd_module::module(void)
 {
 
   if(verbose)
     cout << "cmd_module: display modules\n";
+#ifdef OLD_MODULE_LIBRARY
   cout << ModuleLibrary::DisplayModuleList();
+#else
+  globalSymbolTable().ForEachModule(dumpModules);
+#endif
 
 }
 
@@ -119,7 +127,11 @@ void cmd_module::module(cmd_options *opt)
     {
 
     case CMD_MOD_LIST:
+#ifdef OLD_MODULE_LIBRARY
       cout << ModuleLibrary::DisplayFileList();
+#else
+      ModuleLibrary::ListLoadableModules();
+#endif
       break;
 
     default:
@@ -190,21 +202,31 @@ void cmd_module::module(cmd_options_str *cos)
     case CMD_MOD_LIB:
       if(verbose)
         cout << "module command got the library " << cos->str << endl;
+
       try {
-        ModuleLibrary::LoadFile(cos->str);
+        string fname(cos->str);
+        ModuleLibrary::LoadFile(fname);
       }
       catch(Error *pError) {
         cout << pError->get_errMsg();
       }
+
       break;
     case CMD_MOD_LOAD:
       // Load a module from (an already loaded) library and let
       // gpsim assign the name.
       if(verbose)
         cout << "module command got the module " << cos->str << '\n';
+#ifdef OLD_MODULE_LIBRARY
       if(ModuleLibrary::NewObject(cos->str) == NULL) {
         GetUserInterface().DisplayMessage("module type %s not created\n", cos->str);
       }
+#else
+      {
+        cout << "Fixme -- module NewObject\n";
+      }
+#endif
+
       break;
 
     case CMD_MOD_DUMP:
@@ -212,15 +234,14 @@ void cmd_module::module(cmd_options_str *cos)
       break;
 
     case CMD_MOD_PINS:
-      ModuleLibrary::DisplayModulePins(cos->str);
+      cout << "Fixme: display module pins is not supported...\n";
+      //ModuleLibrary::DisplayModulePins(cos->str);
       break;
 
     default:
       cout << "cmd_module error\n";
     }
 
-  delete cos;
-
 }
 
 
@@ -232,9 +253,19 @@ void  cmd_module::module(cmd_options_str *cos, const char *op1)
 
     case CMD_MOD_LOAD:
       // Load a module from (an already loaded) library 
+#ifdef OLD_MODULE_LIBRARY
       if(ModuleLibrary::NewObject(cos->str,  op1) == NULL) {
         GetUserInterface().DisplayMessage("module type %s not created\n", cos->str);
       }
+#else
+      {
+        string mName(cos->str);
+        string refDes(op1);
+        if(!ModuleLibrary::InstantiateObject(mName,refDes))
+          GetUserInterface().DisplayMessage("module type %s not created\n", cos->str);
+      }        
+#endif
+
       break;
 
     default:
