@@ -37,6 +37,7 @@ cmd_trace c_trace;
 #define TRACE_SAVE_CMD          3
 #define TRACE_LOGON_CMD		4
 #define TRACE_LOGOFF_CMD	5
+#define TRACE_INFO_CMD  	6
 
 static cmd_options cmd_trace_options[] =
 {
@@ -45,26 +46,26 @@ static cmd_options cmd_trace_options[] =
   {"mask",	 TRACE_MASK_CMD,	OPT_TT_NUMERIC},
   {"log",        TRACE_LOGON_CMD,	OPT_TT_STRING},
   {"save",       TRACE_SAVE_CMD,	OPT_TT_STRING},
+  {"info",       TRACE_INFO_CMD,	OPT_TT_BITFLAG},
   {"disable_log",TRACE_LOGOFF_CMD,	OPT_TT_BITFLAG},
  {0,0,0}
 };
 
-cmd_trace::cmd_trace(void)
+cmd_trace::cmd_trace()
+  : command("trace", "tr")
 { 
-  name = "trace";
-  abbreviation = "tr";
 
-    brief_doc = string("Dump the trace history");
+  brief_doc = string("Dump the trace history");
 
-    long_doc = string ("\ntrace [dump_amount | raw | log fname | disable_log]\n"
-		       "\ttrace will print out the most recent \"dump_amount\" traces.\n"
-		       "\tIf no dump_amount is specified, then only the lat few trace\n"
-		       "\tevents will be displayed.\n\n"
-		       "\ttrace raw expr -- display the trace contents in a minimally decoded manner\n"
-		       "\ttrace log fname -- log all raw trace events to a file\n"
-		       "\ttrace save fname -- save the decode trace buffer to a file\n"
-		       "\ttrace disable_log -- stop all file logging\n"
-);
+  long_doc = string ("\ntrace [dump_amount | raw | log fname | disable_log]\n"
+                     "\ttrace will print out the most recent \"dump_amount\" traces.\n"
+                     "\tIf no dump_amount is specified, then only the lat few trace\n"
+                     "\tevents will be displayed.\n\n"
+                     "\ttrace raw expr -- display the trace contents in a minimally decoded manner\n"
+                     "\ttrace log fname -- log all raw trace events to a file\n"
+                     "\ttrace save fname -- save the decode trace buffer to a file\n"
+                     "\ttrace disable_log -- stop all file logging\n"
+                     );
 
   op = cmd_trace_options; 
 }
@@ -89,6 +90,9 @@ void cmd_trace::trace(cmd_options *opt)
     get_trace().disableLogging();
     cout << "Logging to file disabled" << endl;
     break;
+  case TRACE_INFO_CMD:
+    get_trace().showInfo();
+    break;
   default:
     cout << " Invalid set option\n";
   }
