@@ -21,6 +21,8 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __CMD_X_H__
 #define __CMD_X_H__
 
+class Expression;
+
 class cmd_x : public command
 {
 public:
@@ -28,10 +30,10 @@ public:
   cmd_x(void);
   void x(void);
 
-  void x(int reg);
-  void x(int reg, int val);
+  void x(int reg, Expression *pExpr = NULL);
   void x(char *reg_name, int val);
   void x(char *reg_name);
+  void x(Expression *);
 
   virtual int is_repeatable(void) { return 1; };
 };
