@@ -24,6 +24,13 @@ Boston, MA 02111-1307, USA.  */
 #include <string>
 #include <vector>
 
+#include "../config.h"
+
+#include "../src/expr.h"
+#include "../src/pic-processor.h"
+#include "../src/sim_context.h"
+#include "../src/trace.h"
+
 #include "command.h"
 
 #include "cmd_attach.h"
@@ -55,12 +62,6 @@ Boston, MA 02111-1307, USA.  */
 #include "cmd_x.h"
 #include "cmd_icd.h"
 
-#include "../src/expr.h"
-
-#include "../src/pic-processor.h"
-#include "../src/sim_context.h"
-#include "../src/trace.h"
-
 int quit_gpsim = 0;
 
 command *command_list[] =
@@ -96,21 +97,20 @@ command *command_list[] =
 };
 
 
-command::command(void)
+command::command(const char *_name, const char *_abbr)
+  : m_pName(_name), m_pAbbreviation(_abbr)
 {
   op = 0;
-  name = 0;
-  abbreviation = 0;
   token_value = 0;
 }
-
-command::command(struct cmd_options *options,int tv) 
-{ 
+/*
+command::command(struct cmd_options *options,int tv)
+{
   abbreviation = 0;
-  op = options; 
+  op = options;
   token_value = tv;
 }
-
+*/
 
 int number_of_commands = sizeof(command_list) / sizeof(command  *);
 
@@ -122,8 +122,8 @@ command *search_commands(const string &s)
 
   while(i<number_of_commands) {
     command * cmd = command_list[i];
-    if(strcmp(cmd->name, s.c_str()) == 0 ||
-      (cmd->abbreviation != 0 && strcmp(cmd->abbreviation, s.c_str()) == 0 )) {
+    if(strcmp(cmd->name(), s.c_str()) == 0 ||
+       (cmd->abbreviation() != 0 && strcmp(cmd->abbreviation(), s.c_str()) == 0 )) {
       return cmd;
     }
 
@@ -140,22 +140,12 @@ void execute_line(char *cmd)
     cout << "Executing a line:\n  " << cmd;
 }
 
-Processor *command::GetActiveCPU() {
-  return CSimulationContext::GetContext()->GetActiveCPU();
-}
-
-bool command::have_cpu(bool display_warning)
+Processor *command::GetActiveCPU(bool bDisplayWarnings)
 {
-
-  if(!GetActiveCPU())
-    {
-      if(display_warning)
-        cout << "No cpu has been selected\n";
-      return(0);
-    }
-
-  return 1;
-
+  Processor *pCpu = CSimulationContext::GetContext()->GetActiveCPU();
+  if (bDisplayWarnings && !pCpu)
+    cout << "No cpu has been selected\n";
+  return pCpu;
 }
 
 Value *command::toValue(Expression *expr)
@@ -173,7 +163,7 @@ double command::evaluate(Expression *expr)
     if(expr) {
 
       Value *v = toValue(expr);
-      
+
       v->get(value);
 
       delete v;
@@ -200,7 +190,7 @@ static gint64 evaluateToInt(Expression *expr)
     if(expr) {
 
       Value *v = expr->evaluate();
-      
+
       v->get(value);
 
       delete v;
@@ -218,8 +208,8 @@ static gint64 evaluateToInt(Expression *expr)
   return value;
 }
 void command::evaluate(ExprList_t *eList,
-		       guint64 *parameters, 
-		       int *nParameters)
+                       guint64 *parameters,
+                       int *nParameters)
 {
   ExprList_itor ei;
 
@@ -233,8 +223,8 @@ void command::evaluate(ExprList_t *eList,
     return;
 
   int n = 0;
-  for(ei = eList->begin(); 
-      (ei != eList->end()) && (n < *nParameters); 
+  for(ei = eList->begin();
+      (ei != eList->end()) && (n < *nParameters);
       ++ei, n++)
     {
       parameters[n] = evaluateToInt(*ei);
