@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.6.4.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
 /* Bison interface for Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
-   
+
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,13 +26,13 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
 #ifndef YY_YY_PARSE_H_INCLUDED
 # define YY_YY_PARSE_H_INCLUDED
-/* Enabling traces.  */
+/* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 1
 #endif
@@ -40,105 +40,104 @@
 extern int yydebug;
 #endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     ABORT = 258,
-     ATTACH = 259,
-     BREAK = 260,
-     BUS = 261,
-     CLEAR = 262,
-     DISASSEMBLE = 263,
-     DUMP = 264,
-     ENDM = 265,
-     FREQUENCY = 266,
-     HELP = 267,
-     LOAD = 268,
-     LOG = 269,
-     LIST = 270,
-     NODE = 271,
-     MACRO = 272,
-     MODULE = 273,
-     PROCESSOR = 274,
-     QUIT = 275,
-     RESET = 276,
-     RUN = 277,
-     SET = 278,
-     SHELL = 279,
-     STEP = 280,
-     STIMULUS = 281,
-     SYMBOL = 282,
-     TRACE = 283,
-     gpsim_VERSION = 284,
-     X = 285,
-     ICD = 286,
-     END_OF_COMMAND = 287,
-     MACROBODY_T = 288,
-     MACROINVOCATION_T = 289,
-     INDIRECT = 290,
-     END_OF_INPUT = 291,
-     BIT_FLAG = 292,
-     EXPRESSION_OPTION = 293,
-     NUMERIC_OPTION = 294,
-     STRING_OPTION = 295,
-     CMD_SUBTYPE = 296,
-     SYMBOL_OPTION = 297,
-     LITERAL_INT_T = 298,
-     LITERAL_BOOL_T = 299,
-     LITERAL_FLOAT_T = 300,
-     LITERAL_STRING_T = 301,
-     LITERAL_ARRAY_T = 302,
-     SYMBOL_T = 303,
-     GPSIMOBJECT_T = 304,
-     PORT_T = 305,
-     EQU_T = 306,
-     AND_T = 307,
-     COLON_T = 308,
-     COMMENT_T = 309,
-     DIV_T = 310,
-     EOLN_T = 311,
-     MINUS_T = 312,
-     MPY_T = 313,
-     OR_T = 314,
-     PLUS_T = 315,
-     SHL_T = 316,
-     SHR_T = 317,
-     XOR_T = 318,
-     INDEXERLEFT_T = 319,
-     INDEXERRIGHT_T = 320,
-     DECLARE_TYPE = 321,
-     DECLARE_INT_T = 322,
-     DECLARE_FLOAT_T = 323,
-     DECLARE_BOOL_T = 324,
-     DECLARE_CHAR_T = 325,
-     LOR_T = 326,
-     LAND_T = 327,
-     NE_T = 328,
-     EQ_T = 329,
-     ABS_T = 330,
-     MAX_T = 331,
-     MIN_T = 332,
-     GE_T = 333,
-     GT_T = 334,
-     LE_T = 335,
-     LT_T = 336,
-     IND_T = 337,
-     BITS_T = 338,
-     BIT_T = 339,
-     WORD_T = 340,
-     LADDR_T = 341,
-     HIGH_T = 342,
-     LOW_T = 343,
-     INDEXED_T = 344,
-     UNARYOP_PREC = 345,
-     ONESCOMP_T = 346,
-     LNOT_T = 347,
-     POW_T = 348,
-     REG_T = 349
-   };
+  enum yytokentype
+  {
+    ABORT = 258,
+    ATTACH = 259,
+    BREAK = 260,
+    BUS = 261,
+    CLEAR = 262,
+    DISASSEMBLE = 263,
+    DUMP = 264,
+    ENDM = 265,
+    FREQUENCY = 266,
+    HELP = 267,
+    LOAD = 268,
+    LOG = 269,
+    LIST = 270,
+    NODE = 271,
+    MACRO = 272,
+    MODULE = 273,
+    PROCESSOR = 274,
+    QUIT = 275,
+    RESET = 276,
+    RUN = 277,
+    SET = 278,
+    SHELL = 279,
+    STEP = 280,
+    STIMULUS = 281,
+    SYMBOL = 282,
+    TRACE = 283,
+    gpsim_VERSION = 284,
+    X = 285,
+    ICD = 286,
+    END_OF_COMMAND = 287,
+    MACROBODY_T = 288,
+    MACROINVOCATION_T = 289,
+    INDIRECT = 290,
+    END_OF_INPUT = 291,
+    BIT_FLAG = 292,
+    EXPRESSION_OPTION = 293,
+    NUMERIC_OPTION = 294,
+    STRING_OPTION = 295,
+    CMD_SUBTYPE = 296,
+    SYMBOL_OPTION = 297,
+    LITERAL_INT_T = 298,
+    LITERAL_BOOL_T = 299,
+    LITERAL_FLOAT_T = 300,
+    LITERAL_STRING_T = 301,
+    LITERAL_ARRAY_T = 302,
+    SYMBOL_T = 303,
+    GPSIMOBJECT_T = 304,
+    PORT_T = 305,
+    EQU_T = 306,
+    AND_T = 307,
+    COLON_T = 308,
+    COMMENT_T = 309,
+    DIV_T = 310,
+    EOLN_T = 311,
+    MINUS_T = 312,
+    MPY_T = 313,
+    OR_T = 314,
+    PLUS_T = 315,
+    SHL_T = 316,
+    SHR_T = 317,
+    XOR_T = 318,
+    INDEXERLEFT_T = 319,
+    INDEXERRIGHT_T = 320,
+    DECLARE_TYPE = 321,
+    DECLARE_INT_T = 322,
+    DECLARE_FLOAT_T = 323,
+    DECLARE_BOOL_T = 324,
+    DECLARE_CHAR_T = 325,
+    LOR_T = 326,
+    LAND_T = 327,
+    EQ_T = 328,
+    NE_T = 329,
+    LT_T = 330,
+    LE_T = 331,
+    GT_T = 332,
+    GE_T = 333,
+    MIN_T = 334,
+    MAX_T = 335,
+    ABS_T = 336,
+    IND_T = 337,
+    BIT_T = 338,
+    BITS_T = 339,
+    LOW_T = 340,
+    HIGH_T = 341,
+    LADDR_T = 342,
+    WORD_T = 343,
+    INDEXED_T = 344,
+    LNOT_T = 345,
+    ONESCOMP_T = 346,
+    UNARYOP_PREC = 347,
+    POW_T = 348,
+    REG_T = 349
+  };
 #endif
 /* Tokens.  */
 #define ABORT 258
@@ -211,36 +210,35 @@ extern int yydebug;
 #define DECLARE_CHAR_T 325
 #define LOR_T 326
 #define LAND_T 327
-#define NE_T 328
-#define EQ_T 329
-#define ABS_T 330
-#define MAX_T 331
-#define MIN_T 332
+#define EQ_T 328
+#define NE_T 329
+#define LT_T 330
+#define LE_T 331
+#define GT_T 332
 #define GE_T 333
-#define GT_T 334
-#define LE_T 335
-#define LT_T 336
+#define MIN_T 334
+#define MAX_T 335
+#define ABS_T 336
 #define IND_T 337
-#define BITS_T 338
-#define BIT_T 339
-#define WORD_T 340
-#define LADDR_T 341
-#define HIGH_T 342
-#define LOW_T 343
+#define BIT_T 338
+#define BITS_T 339
+#define LOW_T 340
+#define HIGH_T 341
+#define LADDR_T 342
+#define WORD_T 343
 #define INDEXED_T 344
-#define UNARYOP_PREC 345
+#define LNOT_T 345
 #define ONESCOMP_T 346
-#define LNOT_T 347
+#define UNARYOP_PREC 347
 #define POW_T 348
 #define REG_T 349
 
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
-/* Line 2077 of yacc.c  */
-#line 140 "parse.yy"
+#line 140 "parse.yy" /* yacc.c:1909  */
 
   guint32              i;
   guint64             li;
@@ -266,28 +264,14 @@ typedef union YYSTYPE
 
   Macro                    *Macro_P;
 
-
-/* Line 2077 of yacc.c  */
-#line 272 "parse.h"
-} YYSTYPE;
+#line 268 "parse.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
 
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int yyparse (void *YYPARSE_PARAM);
-#else
-int yyparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
+
 int yyparse (void);
-#else
-int yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
 #endif /* !YY_YY_PARSE_H_INCLUDED  */
