@@ -8,10 +8,10 @@
 /*****
 * eXdbm.c : eXdbm main source file
 *
-* This file Version     $Revision: 1967 $
+* This file Version     $Revision: 2216 $
 *
-* Last modification:    $Date: 2007-10-15 06:19:53 +1000 (Mon, 15 Oct 2007) $
-* By:                                   $Author: borutr $
+* Last modification:    $Date: 2013-06-30 10:56:58 +1000 (Sun, 30 Jun 2013) $
+* By:                                   $Author: roy_r_rankin $
 * Current State:                $State$
 *
 * Copyright (C) 1997 Fred Pesch
@@ -35,11 +35,7 @@
 *
 *****/
 
-/*      $Id: eXdbm.c 1967 2007-10-14 20:19:53Z borutr $         */
-
-#ifndef lint
-static char vcid[] = "$Id: eXdbm.c 1967 2007-10-14 20:19:53Z borutr $";
-#endif /* lint */
+/*      $Id: eXdbm.c 2216 2013-06-30 00:56:58Z roy_r_rankin $         */
 
 /* includes */
 
