@@ -23,7 +23,15 @@ Boston, MA 02111-1307, USA.  */
 #define __GUI_BREADBOARD_H__
 
 #include "../src/packages.h"
+#include "../src/stimuli.h"
+#include "gui_object.h"
 
+#include <string>
+#include <vector>
+
+class CrossReferenceToGUI;
+class GUI_Processor;
+class Module;
 
 //
 // The Breadboard window data
@@ -37,17 +45,18 @@ typedef enum {PIN_DIGITAL, PIN_ANALOG, PIN_OTHER} pintype;
 
 // Routing types
 typedef enum {R_NONE,R_LEFT, R_RIGHT, R_UP, R_DOWN} route_direction;
-typedef struct
-{
-    int x;
-    int y;
-} point;
-typedef struct _path
-{
-    point p;
-    route_direction dir;
-    struct _path *next;
-} path;
+class point {
+public:
+  int x;
+  int y;
+};
+
+class path {
+public:
+  point p;
+  route_direction dir;
+  path *next;
+};
 // End routing types
 
 
@@ -111,8 +120,8 @@ public:
   void SetLabelPosition(int x, int y);
   int label_x() { return m_label_x; }
   int label_y() { return m_label_y; }
-  int DrawGUIlabel(GdkPixmap *pix_map,  int pinnameWidths[]);
-  void DrawLabel(GdkPixmap *pix_map);
+  bool DrawGUIlabel();
+  void DrawLabel(cairo_t *cr);
 
   void SetOrientation(eOrientation o) { orientation = o; }
 
@@ -122,8 +131,6 @@ public:
   const char *pinName();
 
   GtkWidget *m_pinDrawingArea;
-  GdkPixmap *pixmap;
-  GdkGC *gc;
 
   eDirection direction;
   eOrientation orientation;
@@ -143,11 +150,26 @@ protected:
   int    m_label_y;
 
   int    m_pkgPinNumber;    // 
+
+private:
+  static gboolean expose_pin(GtkWidget *widget, GdkEventExpose *event,
+    GuiPin *p);
 };
 
 
 enum module_type {PIC_MODULE, EXTERNAL_MODULE};
 
+class BB_ModuleLabel {
+public:
+  BB_ModuleLabel(const std::string &text, PangoFontDescription *font);
+  ~BB_ModuleLabel();
+
+  GtkWidget *gobj() {return m_label;}
+
+private:
+  GtkWidget *m_label;
+};
+
 //------------------------------------------------------------------------
 // GuiModule
 // 
@@ -172,22 +194,17 @@ public:
   virtual void Build();
   virtual void Draw();
   virtual void Destroy();
-  virtual void DrawCaseOutline(GtkWidget *);
+  virtual void DrawCaseOutline(cairo_t *);
 
   virtual void AddPin(unsigned int);
   virtual void AddPinGeometry(GuiPin *);
 
-  void BuildReferenceDesignator();
-
   int pin_count() { return m_pin_count; }
-  GList * pins() { return m_pins; }
+  std::vector<GuiPin *> * pins() { return &m_pins; }
   Module *module() { return m_module; }
   GtkWidget *module_widget() { return m_module_widget; }
   GtkWidget *pinLabel_widget() { return m_pinLabel_widget; }
-  GtkWidget *name_widget() { return m_name_widget; }
-  GtkWidget *tree_item() { return m_tree_item; }
-  GdkPixmap *module_pixmap() { return m_module_pixmap; }
-  GdkPixmap *name_pixmap() { return m_name_pixmap; }
+  GtkWidget *name_widget() { return m_name_widget->gobj(); }
 
 protected:
 
@@ -198,26 +215,22 @@ protected:
   int        m_module_y;       /* GuiBreadBoardObject. Their purpose is to allow clients to 
 				* reposition a customly created module_widget */
 
-  GtkWidget *m_name_widget;    // Name of widget, positioned above module_widget.
+  BB_ModuleLabel *m_name_widget;    // Name of widget, positioned above module_widget.
 
   int pinnameWidths[4];
   int m_pin_count;
 
-  GdkPixmap *m_module_pixmap;
-  GdkPixmap *m_name_pixmap;
-
-  GtkWidget *m_tree_item;
-
-  GList *m_pins;
-
+  std::vector<GuiPin *> m_pins;
 
+private:
+  static gboolean module_expose(GtkWidget *widget, GdkEventExpose *event, GuiModule *p);
 };
 
 class GuiDipModule : public GuiModule
 {
 public:
   GuiDipModule(Module *, Breadboard_Window *);
-  virtual void DrawCaseOutline(GtkWidget *);
+  virtual void DrawCaseOutline(cairo_t *);
 };
 
 struct gui_node
@@ -225,9 +238,6 @@ struct gui_node
     Breadboard_Window *bbw;
     Stimulus_Node *node;
     GtkWidget *tree_item;
-    int selected_row;
-
-    GList *pins;
 };
 
 
@@ -236,23 +246,15 @@ class Breadboard_Window : public GUI_Object
 {
 public:
 
-#if GTK_MAJOR_VERSION >= 2
   PangoFontDescription *pinstatefont;
   PangoFontDescription *pinnamefont;
-#else
-  GdkFont *pinstatefont;
-  GdkFont *pinnamefont;
-#endif
+
   int pinnameheight;
 
   GtkWidget *layout;
 
-  GdkGC *pinname_gc;
-  GdkGC *pinline_gc;
-  GdkGC *case_gc;
-
-  GList *modules;
-  GList *nodes;
+  std::vector<GuiModule *> modules;
+  std::vector<Stimulus_Node *> nodes;
 
   GtkWidget *tree;
 
@@ -261,7 +263,6 @@ public:
   GtkWidget *module_frame;
   GtkWidget *stimulus_frame;
 
-  GtkWidget *pic_settings_clist;
   GtkWidget *attribute_clist;
   GtkWidget *attribute_entry;
   GtkWidget *attribute_button;
@@ -272,8 +273,6 @@ public:
 
   GtkWidget *stimulus_add_node_button;
 
-  GdkPixmap *layout_pixmap;
-
   GtkAdjustment *hadj, *vadj;
   
   GtkTreeIter *node_iter;
@@ -282,19 +281,29 @@ public:
   struct gui_node *selected_node;
   GuiModule *selected_module;
 
-
   Breadboard_Window(GUI_Processor *gp);
+
   virtual void Build(void);
   virtual void NewProcessor(GUI_Processor *gp);
   virtual void Update(void);
   virtual void NewModule(Module *module);
   virtual void NodeConfigurationChanged(Stimulus_Node *node);
 
+  void draw_nodes();
+  void update_board_matrix();
+  void clear_nodes();
+
   GtkWidget *add_button(const char *label, const char *name,
-			GtkSignalFunc f, GtkWidget *box);
+			GCallback f, GtkWidget *box);
+
+protected:
+  virtual const char *name();
 
 private:
   GuiModule *m_MainCpuModule;
+
+  static gboolean layout_expose(GtkWidget *widget,
+    GdkEventExpose *event, Breadboard_Window *bbw);
 };
 
 
