@@ -3,20 +3,19 @@
 
 
 #ifdef __cplusplus
-extern "C" {
+//extern "C" {
 #endif /* __cplusplus */
 
 
 void gui_new_processor (unsigned int pic_id);
 void gui_new_source (unsigned int pic_id);
 
-void gui_simulation_has_stopped(gpointer _gp);
 void update_register(unsigned int reg_number);
 void update_program_memory(GUI_Processor *gp, unsigned int reg_number);
 
 
 #ifdef __cplusplus
-}
+//}
 #endif /* __cplusplus */
 
 
