@@ -22,21 +22,12 @@ Boston, MA 02111-1307, USA.  */
 #include "../config.h"
 #ifdef HAVE_GUI
 
-#ifdef DOING_GNOME
-#include <gnome.h>
-#endif
-
-#include <unistd.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
 #include <glib.h>
-#include <string.h>
-
-#include <gtkextra/gtkbordercombo.h>
-#include <gtkextra/gtkcolorcombo.h>
-#include <gtkextra/gtksheet.h>
 
+#include <cstring>
+#include <string>
 
 #include "gui.h"
 #include "preferences.h"
@@ -57,13 +48,8 @@ Boston, MA 02111-1307, USA.  */
 #include "../cli/input.h"  // for gpsim_open()
 #include "../src/gpsim_interface.h"
 
-typedef struct _note_book_item
-{
-  gchar        *name;
-  GtkSignalFunc func;
-} NotebookItem;
 
-GtkItemFactory *item_factory=0;
+GtkUIManager *ui;
 
 extern GUI_Processor *gpGuiProcessor;
 
@@ -74,65 +60,29 @@ do_quit_app(GtkWidget *widget)
 }
 
 
-static void
-show_message (const char *title, const char *message)
-{
-  GtkWidget *window;
-  GtkWidget *label;
-  GtkWidget *button;
-
-  window = gtk_dialog_new ();
-
-  gtk_window_set_title (GTK_WINDOW (window), title);
-  gtk_container_set_border_width (GTK_CONTAINER (window), 0);
-
-
-  button = gtk_button_new_with_label ("close");
-  gtk_container_set_border_width (GTK_CONTAINER (button), 10);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                             GTK_OBJECT (window));
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area), button, TRUE, TRUE, 0);
-  gtk_widget_grab_default(button);
-  gtk_widget_show(button);
-
-
-  label = gtk_label_new (message);
-  gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), label, TRUE, TRUE, 0);
-
-  gtk_widget_show (label);
-  gtk_widget_show (window);
-
-  gtk_grab_add (window);
-
-}
 //========================================================================
 static void
-about_cb (gpointer             callback_data,
-          guint                callback_action,
-          GtkWidget           *widget)
+about_cb(GtkAction *action, gpointer user_data)
 {
+  static const gchar *authors[] = {
+    "Scott Dattalo - <scott@dattalo.com>",
+    "Ralf Forsberg - <rfg@home.se>",
+    "Borut Ra" "\xc5\xbe" "em - <borut.razem@gmail.com>",
+    NULL
+  };
 
-  show_message(  "The GNUPIC Simulator - " VERSION, "A simulator for Microchip PIC microcontrollers.\n"
-                 "by T. Scott Dattalo - mailto:scott@dattalo.com\n"
-                 "   Ralf Forsberg - mailto:rfg@home.se\n"
-                 "   Borut Ra" "\xc5\xbe" "em - mailto:borut.razem@gmail.com\n\n"
-                 "gpsim homepage: http://gpsim.sourceforge.net/gpsim.html\n"
-                 "gpsimWin32: http://gpsim.sourceforge.net/gpsimWin32/gpsimWin32.html\n");
-
+  gtk_show_about_dialog(NULL,
+    "authors", authors,
+    "comments", "A simulator for Microchip PIC microcontrollers.",
+    "version", VERSION,
+    "website", "http://gpsim.sourceforge.net/gpsim.html",
+    "program-name", "The GNUPIC Simulator",
+    NULL);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-#if defined(NEW_SOURCE_BROWSER)
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-
 //========================================================================
 //
 // class ColorButton
@@ -281,45 +231,35 @@ public:
   gpsimGuiPreferences();
   ~gpsimGuiPreferences();
 
-  static void setup (gpointer             callback_data,
-                     guint                callback_action,
-                     GtkWidget           *widget);
+  static void setup (GtkAction *action, gpointer user_data);
 
 
 private:
   SourceBrowserPreferences *m_SourceBrowser;
 
-  static gint cancel_cb (gpsimGuiPreferences *Self);
-  static gint apply_cb  (gpsimGuiPreferences *Self);
+  static void response_cb(GtkDialog *dialog, gint response_id,
+    gpsimGuiPreferences *Self);
   void apply() { m_SourceBrowser->apply();}
   void cancel() { m_SourceBrowser->cancel();}
   GtkWidget *window;
 };
-static GtkWidget *LocalWindow=0;
 
 
-void gpsimGuiPreferences::setup (gpointer             callback_data,
-                                 guint                callback_action,
-                                 GtkWidget           *widget)
+void gpsimGuiPreferences::setup (GtkAction *action, gpointer user_data)
 {
   new gpsimGuiPreferences();
 }
 
-
-gint gpsimGuiPreferences::cancel_cb (gpsimGuiPreferences  *Self)
+void gpsimGuiPreferences::response_cb(GtkDialog *dialog, gint response_id,
+  gpsimGuiPreferences *Self)
 {
-
-  Self->cancel();
+  if (response_id == gint(GTK_RESPONSE_CANCEL))
+    Self->cancel();
+  if (response_id == gint(GTK_RESPONSE_APPLY))
+    Self->apply();
   delete Self;
-  return TRUE;
 }
-gint gpsimGuiPreferences::apply_cb  (gpsimGuiPreferences *Self)
-{
 
-  Self->apply();
-  delete Self;
-  return TRUE;
-}
 
 //------------------------------------------------------------------------
 // ColorButton Constructor
@@ -335,13 +275,13 @@ ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
   gtk_box_pack_start (GTK_BOX(hbox),colorButton,FALSE, FALSE, 0);
   gtk_widget_show(colorButton);
 
-  gtk_signal_connect (GTK_OBJECT(colorButton),
+  g_signal_connect (colorButton,
                       "color-set",
-                      GTK_SIGNAL_FUNC(setColor_cb),
+                      G_CALLBACK(setColor_cb),
                       this);
-  const int cBORDER = 10; // pixels
+
   GtkWidget *label       = gtk_label_new(colorName);
-  gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE, cBORDER);
+  gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE, 10);
   gtk_widget_show (label);
 
   gtk_widget_show (hbox);
@@ -388,16 +328,15 @@ MarginButton::MarginButton(GtkWidget *pParent, const char *pName,
                                 bState);
   gtk_box_pack_start (GTK_BOX (pParent), m_button, FALSE, TRUE, 10);
 
-  gtk_signal_connect (GTK_OBJECT(m_button),
+  g_signal_connect (m_button,
                       "toggled",
-                      GTK_SIGNAL_FUNC(toggle_cb),
+                      G_CALLBACK(toggle_cb),
                       this);
 }
 //------------------------------------------------------------------------
 void MarginButton::toggle_cb(GtkToggleButton *widget,
                              MarginButton    *This)
 {
-  //  This->m_pStyle->setFG(&newColor);
   This->set_active();
 }
 void MarginButton::set_active()
@@ -425,16 +364,15 @@ TabButton::TabButton(GtkWidget *pParent, GtkWidget *pButton,
 {
   gtk_box_pack_start (GTK_BOX (pParent), m_button, FALSE, TRUE, 5);
 
-  gtk_signal_connect (GTK_OBJECT(m_button),
+  g_signal_connect (m_button,
                       "toggled",
-                      GTK_SIGNAL_FUNC(toggle_cb),
+                      G_CALLBACK(toggle_cb),
                       this);
 }
 //------------------------------------------------------------------------
 void TabButton::toggle_cb(GtkToggleButton *widget,
                           TabButton    *This)
 {
-  //  This->m_pStyle->setFG(&newColor);
   This->set_active();
 }
 void TabButton::set_active()
@@ -460,15 +398,13 @@ FontSelection::FontSelection (GtkWidget *pParent,
   gtk_box_pack_start (GTK_BOX(hbox),m_fontButton,FALSE, FALSE, 0);
   gtk_widget_show(m_fontButton);
 
-  gtk_signal_connect (GTK_OBJECT(m_fontButton),
+  g_signal_connect (m_fontButton,
                       "font-set",
-                      GTK_SIGNAL_FUNC(setFont_cb),
+                      G_CALLBACK(setFont_cb),
                       this);
 
-  const char *fontName = "font";
-  const int cBORDER = 10; // pixels
-  GtkWidget *label       = gtk_label_new(fontName);
-  gtk_box_pack_start (GTK_BOX(hbox),label,TRUE, TRUE, cBORDER);
+  GtkWidget *label = gtk_label_new("font");
+  gtk_box_pack_start (GTK_BOX(hbox),label,TRUE, TRUE, 10);
   gtk_widget_show (label);
 
   gtk_widget_show (hbox);
@@ -485,45 +421,6 @@ void FontSelection::setFont()
 }
 
 //------------------------------------------------------------------------
-#if 0 // defined but not used
-static bool isButtonEvent (GdkEventType type)
-{
-  return
-    type == GDK_BUTTON_PRESS ||
-    type == GDK_2BUTTON_PRESS ||
-    type == GDK_3BUTTON_PRESS ||
-    type == GDK_BUTTON_RELEASE;
-
-}
-
-static gboolean    TagEvent  (GtkTextTag *texttag,
-                       GObject *arg1,
-                       GdkEvent *event,
-                       GtkTextIter *arg2,
-                       gpointer user_data)
-{
-  printf("Received tag event signal Tag:%p arg1:%p Event:%p iter:%p user:%p %08X Line:%d\n",
-         texttag, arg1,event,arg2, user_data, event->type, gtk_text_iter_get_line(arg2));
-
-  if (isButtonEvent(event->type)) {
-    GdkEventButton *evtButton = (GdkEventButton *) event;
-
-    printf("Button Event: button:%d  modifier:%d coords(%g,%g)\n",
-           evtButton->button, evtButton->state, evtButton->x,evtButton->y);
-    // If the right mouse button is pressed then suppress the GTK pop up menu.
-    if (evtButton->button == 3)
-      return TRUE;
-  }
-  return FALSE;
-}
-//------------------------------------------------------------------------
-
-static void preferences_AddFontSelect(GtkWidget *pParent, const char *fontDescription,
-                                      const char *fontName  )
-{
-}
-#endif
-
 
 void SourceBrowserPreferences::toggleBreak(int line)
 {
@@ -657,15 +554,11 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
     pages[id]->m_pBuffer->parseLine( "Label:  ADDWF  Variable,F  ; Comment",1);
 
     gtk_widget_show_all(frame);
-#if 1
+
     label = gtk_label_new("file2.asm");
     GtkWidget *emptyBox = gtk_hbox_new(0,0);
     gtk_notebook_append_page(GTK_NOTEBOOK(m_Notebook),emptyBox,label);
 
-#endif
-
-
-
   }
 
   gtk_widget_show_all(notebook);
@@ -738,50 +631,21 @@ int SourceBrowserPreferences::getOpcode(int address)
 //========================================================================
 gpsimGuiPreferences::gpsimGuiPreferences()
 {
-  GtkWidget *button;
+  window = gtk_dialog_new_with_buttons("Source Browser configuration",
+    NULL,
+    GTK_DIALOG_MODAL,
+    GTK_STOCK_CANCEL, gint(GTK_RESPONSE_CANCEL),
+    GTK_STOCK_APPLY, gint(GTK_RESPONSE_APPLY),
+    NULL);
 
-  LocalWindow = gtk_dialog_new ();
-  window = LocalWindow;
-  gtk_widget_show (window);
+  g_signal_connect(window, "response",
+    G_CALLBACK(gpsimGuiPreferences::response_cb), this);
 
-  gtk_window_set_title (GTK_WINDOW (window), "Source Browser configuration");
-  gtk_container_set_border_width (GTK_CONTAINER (window), 0);
+  GtkWidget *box = gtk_dialog_get_content_area(GTK_DIALOG(window));
 
-  GtkWidget *vbox = GTK_DIALOG (window)->vbox;
-
-  m_SourceBrowser = new SourceBrowserPreferences(vbox);
-  gtk_widget_show_all(vbox);
-
-
-
-  // Cancel and Apply buttons
-
-  GtkWidget *buttonBox = gtk_hbutton_box_new();
-  gtk_box_pack_start (GTK_BOX (vbox), buttonBox, TRUE, TRUE, 0);
-
-  button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-  gtk_container_set_border_width (GTK_CONTAINER (button), 10);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-                             GTK_SIGNAL_FUNC(gpsimGuiPreferences::cancel_cb),
-                             this);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-
-  gtk_box_pack_start (GTK_BOX (buttonBox), button, TRUE, TRUE, 0);
-  gtk_widget_grab_default(button);
-
-  button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
-  gtk_container_set_border_width (GTK_CONTAINER (button), 10);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-                             GTK_SIGNAL_FUNC(gpsimGuiPreferences::apply_cb),
-                             this);
-
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (buttonBox), button, TRUE, TRUE, 0);
-  gtk_widget_show_all(buttonBox);
-
-
-  gtk_widget_show (window);
+  m_SourceBrowser = new SourceBrowserPreferences(box);
 
+  gtk_widget_show_all(window);
 }
 
 gpsimGuiPreferences::~gpsimGuiPreferences()
@@ -796,148 +660,71 @@ gpsimGuiPreferences::~gpsimGuiPreferences()
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-#endif //if defined(NEW_SOURCE_BROWSER)
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
 //========================================================================
-void
-file_selection_hide_fileops (GtkWidget *widget,
-                             GtkFileSelection *fs)
-{
-  gtk_file_selection_hide_fileop_buttons (fs);
-}
 
 extern int gui_message(const char *message);
 
-void
-file_selection_ok (GtkWidget        *w,
-                   GtkFileSelection *fs)
+static void
+fileopen_dialog(GtkAction *action, gpointer user_data)
 {
+  GtkWidget *dialog = gtk_file_chooser_dialog_new("Open file",
+    NULL,
+    GTK_FILE_CHOOSER_ACTION_OPEN,
+    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+    NULL);
 
-  const char *file;
-  char msg[200];
-
-  if(gpGuiProcessor)
-  {
-    file=gtk_file_selection_get_filename (fs);
-    if(!gpsim_open(gpGuiProcessor->cpu, file, 0, 0))
-    {
-      sprintf(msg, "Open failedCould not open \"%s\"", (char *)file);
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
+    char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+    if (!gpsim_open(gpGuiProcessor->cpu, filename, 0, 0)) {
+      gchar *msg = g_strdup_printf(
+        "Open failed. Could not open \"%s\"", filename);
       gui_message(msg);
+      g_free(msg);
     }
+    g_free(filename);
   }
-  gtk_widget_hide (GTK_WIDGET (fs));
-}
 
-extern int gui_question(char *question, char *a, char *b);
-
-static GtkItemFactoryCallback
-fileopen_dialog(gpointer             callback_data,
-              guint                callback_action,
-              GtkWidget           *widget)
-{
-  static GtkWidget *window = 0;
-  GtkWidget *button;
-
-  if (!window)
-  {
-
-    window = gtk_file_selection_new ("file selection dialog");
-
-    gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (window));
-
-    gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-
-    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
-                        "clicked", GTK_SIGNAL_FUNC(file_selection_ok),
-                        window);
-    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
-                               "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide),
-                               GTK_OBJECT (window));
-
-    button = gtk_button_new_with_label ("Hide Fileops");
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                        (GtkSignalFunc) file_selection_hide_fileops,
-                        (gpointer) window);
-    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area),
-                        button, FALSE, FALSE, 0);
-    gtk_widget_show (button);
-
-    button = gtk_button_new_with_label ("Show Fileops");
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                        (GtkSignalFunc) gtk_file_selection_show_fileop_buttons,
-                        (gpointer) window);
-    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area),
-                        button, FALSE, FALSE, 0);
-    gtk_widget_show (button);
-  }
-    gtk_widget_show (window);
-    return 0;
+  gtk_widget_destroy(dialog);
 }
 
 
 
 
 // Menuhandler for Windows menu buttons
-static GtkItemFactoryCallback
-toggle_window (gpointer             callback_data,
-              guint                callback_action,
-              GtkWidget           *widget)
+static void
+toggle_window(GtkToggleAction *action, gpointer user_data)
 {
-  GtkWidget *menu_item = 0;
+  if (gpGuiProcessor) {
+    std::string item = gtk_action_get_name(GTK_ACTION(action));
+    gboolean view_state = gtk_toggle_action_get_active(action);
 
-  menu_item = gtk_item_factory_get_item (item_factory,
-                                         gtk_item_factory_path_from_widget (widget));
-  if(gpGuiProcessor && menu_item) {
-
-    int view_state =  GTK_CHECK_MENU_ITEM(menu_item)->active ? VIEW_SHOW : VIEW_HIDE;
-
-    switch(callback_action) {
-    case WT_opcode_source_window:
+    if (item == "Program memory") {
       gpGuiProcessor->program_memory->ChangeView(view_state);
-      break;
-    case WT_asm_source_window:
+    } else if (item == "Source") {
       gpGuiProcessor->source_browser->ChangeView(view_state);
-      break;
-    case WT_register_window:
+    } else if (item == "Ram") {
       gpGuiProcessor->regwin_ram->ChangeView(view_state);
-      break;
-    case WT_eeprom_window:
+    } else if (item == "EEPROM") {
       gpGuiProcessor->regwin_eeprom->ChangeView(view_state);
-      break;
-    case WT_watch_window:
+    } else if (item == "Watch") {
       gpGuiProcessor->watch_window->ChangeView(view_state);
-      break;
-    case WT_symbol_window:
+    } else if (item == "Symbols") {
       gpGuiProcessor->symbol_window->ChangeView(view_state);
-      break;
-    case WT_breadboard_window:
+    } else if (item == "Breadboard") {
       gpGuiProcessor->breadboard_window->ChangeView(view_state);
-      break;
-    case WT_stack_window:
+    } else if (item == "Stack") {
       gpGuiProcessor->stack_window->ChangeView(view_state);
-      break;
-    case WT_trace_window:
+    } else if (item == "Trace") {
       gpGuiProcessor->trace_window->ChangeView(view_state);
-      break;
-    case WT_profile_window:
+    } else if (item == "Profile") {
       gpGuiProcessor->profile_window->ChangeView(view_state);
-      break;
-    case WT_stopwatch_window:
+    } else if (item == "Stopwatch") {
       gpGuiProcessor->stopwatch_window->ChangeView(view_state);
-      break;
-    case WT_scope_window:
+    } else if (item == "Scope") {
       gpGuiProcessor->scope_window->ChangeView(view_state);
-      //cout << " The Scope is disabled right now\n";
-      break;
-    default:
-      puts("unknown menu action");
     }
-
   }
-  return 0;
 }
 
 //========================================================================
@@ -1031,10 +818,7 @@ gui_update_cb(GtkWidget *widget, gpointer data)
   gint index = combo_box ? gtk_combo_box_get_active(combo_box) : 0;
 
   UpdateRateMenuItem *umi = UpdateRateMenuItemIndexed[index];
-  if (umi)
-    umi->Select();
-  else
-    cout << "Error UpdateRateMenuItem bad index:" << index << endl;
+  umi->Select();
 }
 
 UpdateRateMenuItem::UpdateRateMenuItem(GtkWidget *parent,
@@ -1055,7 +839,7 @@ UpdateRateMenuItem::UpdateRateMenuItem(GtkWidget *parent,
   if(!menu)
     menu = gtk_menu_new();
 
-  gtk_combo_box_append_text (GTK_COMBO_BOX(parent), label);
+  gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(parent), label);
 
   menu_index = seq_no;
   seq_no++;
@@ -1082,11 +866,6 @@ void UpdateRateMenuItem::Select()
     gpGuiProcessor->cpu->pma->stop();
 
   config_set_variable("dispatcher", "SimulationMode", id);
-
-  if (0)
-    cout << "Update gui refresh: " << hex << update_rate
-         << " ID:" << id << "Seq no:" << menu_index
-         << endl;
 }
 
 
@@ -1149,7 +928,7 @@ public:
     double time_db = 0.;
     if(gpGuiProcessor && gpGuiProcessor->cpu)
         time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e6;
-    snprintf(buf,size, "%19.2f us",time_db);
+    g_snprintf(buf,size, "%19.2f us",time_db);
   }
 };
 
@@ -1168,7 +947,7 @@ public:
     double time_db = 0.;
   if(gpGuiProcessor && gpGuiProcessor->cpu)
         time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e3;
-    snprintf(buf,size, "%19.3f ms",time_db);
+    g_snprintf(buf,size, "%19.3f ms",time_db);
   }
 };
 
@@ -1187,7 +966,7 @@ public:
     double time_db = 0.;
   if(gpGuiProcessor && gpGuiProcessor->cpu)
        time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
-    snprintf(buf,size, "%19.3f Sec",time_db);
+    g_snprintf(buf,size, "%19.3f Sec",time_db);
   }
 };
 
@@ -1213,7 +992,7 @@ public:
     v-=mm*60.0;
     ss=(int)v;
     cc=(int)((v-ss)*100.0);
-    snprintf(buf,size,"    %02d:%02d:%02d.%02d",hh,mm,ss,cc);
+    g_snprintf(buf,size,"    %02d:%02d:%02d.%02d",hh,mm,ss,cc);
   }
 };
 
@@ -1229,7 +1008,7 @@ public:
 
   void Format(char *buf, int size)
   {
-    snprintf(buf,size,"0x%016" PRINTF_GINT64_MODIFIER "x",get_cycles().get());
+    g_snprintf(buf,size,"0x%016" PRINTF_GINT64_MODIFIER "x",get_cycles().get());
   }
 };
 
@@ -1245,7 +1024,7 @@ public:
 
   void Format(char *buf, int size)
   {
-    snprintf(buf,size,"%016" PRINTF_GINT64_MODIFIER "d",get_cycles().get());
+    g_snprintf(buf,size,"%016" PRINTF_GINT64_MODIFIER "d",get_cycles().get());
   }
 };
 
@@ -1273,7 +1052,7 @@ cbTimeFormatPopup(GtkWidget *widget, GdkEventButton *event, TimeWidget *tw)
                    3, event->time);
     // It looks like we need it to avoid a selection in the entry.
     // For this we tell the entry to stop reporting this event.
-    gtk_signal_emit_stop_by_name(GTK_OBJECT(tw->entry),"button_press_event");
+    g_signal_stop_emission_by_name(tw->entry, "button_press_event");
   }
   return FALSE;
 }
@@ -1289,11 +1068,11 @@ void TimeFormatter::AddToMenu(GtkWidget *menu,
                               const char*menu_text)
 {
   GtkWidget *item = gtk_menu_item_new_with_label(menu_text);
-  gtk_signal_connect(GTK_OBJECT(item),"activate",
-                     (GtkSignalFunc) cbTimeFormatActivated,
+  g_signal_connect(item, "activate",
+                     G_CALLBACK(cbTimeFormatActivated),
                      this);
   gtk_widget_show(item);
-  gtk_menu_append(GTK_MENU(menu),item);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 }
 
 void TimeWidget::Create(GtkWidget *container)
@@ -1305,10 +1084,6 @@ void TimeWidget::Create(GtkWidget *container)
   SetEntryWidth(18);
 
   menu = gtk_menu_new();
-  GtkWidget *item = gtk_tearoff_menu_item_new ();
-  gtk_menu_append (GTK_MENU (menu), item);
-  gtk_widget_show (item);
-
 
   // Create an entry for each item in the formatter pop up window.
 
@@ -1320,9 +1095,9 @@ void TimeWidget::Create(GtkWidget *container)
   new TimeCyclesDec(this,menu);
 
   // Associate a callback with the user button-click actions
-  gtk_signal_connect(GTK_OBJECT(entry),
+  g_signal_connect(entry,
                      "button_press_event",
-                     (GtkSignalFunc) cbTimeFormatPopup,
+                     G_CALLBACK(cbTimeFormatPopup),
                      this);
 }
 
@@ -1364,67 +1139,65 @@ static int dispatcher_delete_event(GtkWidget *widget,
     return 0;
 }
 
-static GtkItemFactoryCallback
-gtk_ifactory_cb (gpointer             callback_data,
-                 guint                callback_action,
-                 GtkWidget           *widget)
-{
-  g_message ("\"%s\" is not supported yet.", gtk_item_factory_path_from_widget (widget));
-    return 0;
-}
-
-#define TOGGLE_WINDOW (GtkItemFactoryCallback)toggle_window
-
-static GtkItemFactoryEntry menu_items[] =
-{
-  { (gchar *)"/_File",            0,         0,        0, (gchar *)"<Branch>" },
-  { (gchar *)"/File/tearoff1",    0,         (GtkItemFactoryCallback)gtk_ifactory_cb,       0, (gchar *)"<Tearoff>" },
-  //{ (gchar *)"/File/_New",        "<control>N", (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-  { (gchar *)"/File/_Open",       (gchar *)"<control>O", (GtkItemFactoryCallback)fileopen_dialog,       0 },
-  //{ (gchar *)"/File/_Save",       "<control>S", (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-  //{ (gchar *)"/File/Save _As...", 0,         (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-  { (gchar *)"/File/sep1",        0,         (GtkItemFactoryCallback)gtk_ifactory_cb,       0, (gchar *)"<Separator>" },
-  { (gchar *)"/File/_Quit",       (gchar *)"<control>Q", (GtkItemFactoryCallback)do_quit_app,         0 },
-
-  //  { (gchar *)"/_Processor",           0,       0,               0, "<Branch>" },
-  //  { (gchar *)"/_Processor/New",   0,       (GtkItemFactoryCallback)new_processor_dialog,       0 },
-  //  { (gchar *)"/_Processor/Delete",0,       (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-  //  { (gchar *)"/_Processor/Switch",0,       (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-
-
-  //  { (gchar *)"/_Break",       0, 0,               0, "<Branch>" },
-  //  { (gchar *)"/_Break/Set",       0, (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-  //  { (gchar *)"/_Break/Clear",     0, (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
-
-  { (gchar *)"/_Windows",     0, 0,       0, (gchar *)"<Branch>" },
-  { (gchar *)"/Windows/Program _memory", 0, TOGGLE_WINDOW,WT_opcode_source_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/_Source",         0, TOGGLE_WINDOW,WT_asm_source_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/sep1",            0, (GtkItemFactoryCallback)gtk_ifactory_cb,0,(gchar *)"<Separator>"  },
-  { (gchar *)"/Windows/_Ram",            0, TOGGLE_WINDOW,WT_register_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/_EEPROM",         0, TOGGLE_WINDOW,WT_eeprom_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/_Watch",          0, TOGGLE_WINDOW,WT_watch_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/Sta_ck",          0, TOGGLE_WINDOW,WT_stack_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/sep2",            0, (GtkItemFactoryCallback)gtk_ifactory_cb,0,(gchar *)"<Separator>"  },
-  { (gchar *)"/Windows/Symbo_ls",        0, TOGGLE_WINDOW,WT_symbol_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/_Breadboard",     0, TOGGLE_WINDOW,WT_breadboard_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/sep3",            0, (GtkItemFactoryCallback)gtk_ifactory_cb,0,(gchar *)"<Separator>"  },
-  { (gchar *)"/Windows/_Trace",          0, TOGGLE_WINDOW,WT_trace_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/Pro_file",        0, TOGGLE_WINDOW,WT_profile_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/St_opwatch",      0, TOGGLE_WINDOW,WT_stopwatch_window,(gchar *)"<ToggleItem>" },
-  { (gchar *)"/Windows/Sco_pe",          0, TOGGLE_WINDOW,WT_scope_window,(gchar *)"<ToggleItem>" },
-
-#if defined(NEW_SOURCE_BROWSER)
-  { (gchar *)"/_Edit",     0, 0,       0, (gchar *)"<Branch>" },
-  { (gchar *)"/Edit/Preferences",        0, (GtkItemFactoryCallback)gpsimGuiPreferences::setup, 0 },
-#endif
-
-  { (gchar *)"/_Help",            0,         0,                     0, (gchar *)"<LastBranch>" },
-  { (gchar *)"/Help/_About",      0,         (GtkItemFactoryCallback)about_cb,       0 },
-
+static const GtkActionEntry entries[] = {
+  {"FileMenu", NULL, "_File"},
+  {"Open", GTK_STOCK_OPEN, "_Open", "<control>O", NULL, G_CALLBACK(fileopen_dialog)},
+  {"Quit", GTK_STOCK_QUIT, "_Quit", "<control>Q", "Quit", G_CALLBACK(do_quit_app)},
+  {"Windows", NULL, "_Windows"},
+  {"Edit", NULL, "_Edit"},
+  {"Preferences", GTK_STOCK_PREFERENCES, "Preferences", NULL, NULL, G_CALLBACK(gpsimGuiPreferences::setup)},
+  {"Help", NULL, "_Help"},
+  {"About", GTK_STOCK_ABOUT, "_About", NULL, NULL, G_CALLBACK(about_cb)}
 };
 
-static int nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
+static const GtkToggleActionEntry toggle_entries[] = {
+  {"Program memory", NULL, "Program _memory", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Source", NULL, "_Source", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Ram", NULL, "_Ram", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"EEPROM", NULL, "_EEPROM", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Watch", NULL, "_Watch", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Stack", NULL, "Sta_ck", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Symbols", NULL, "Symbo_ls", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Breadboard", NULL, "_Breadboard", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Trace", NULL, "_Trace", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Profile", NULL, "Pro_file", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Stopwatch", NULL, "St_opwatch", NULL, NULL, G_CALLBACK(toggle_window)},
+  {"Scope", NULL, "Sco_pe", NULL, NULL, G_CALLBACK(toggle_window)}
+};
 
+static const gchar *ui_info =
+"<ui>"
+"  <menubar name='menu'>"
+"    <menu action='FileMenu'>"
+"      <menuitem action='Open'/>"
+"      <separator/>"
+"      <menuitem action='Quit'/>"
+"    </menu>"
+"    <menu action='Windows'>"
+"      <menuitem action='Program memory'/>"
+"      <menuitem action='Source'/>"
+"      <separator/>"
+"      <menuitem action='Ram'/>"
+"      <menuitem action='EEPROM'/>"
+"      <menuitem action='Watch'/>"
+"      <menuitem action='Stack'/>"
+"      <separator/>"
+"      <menuitem action='Symbols'/>"
+"      <menuitem action='Breadboard'/>"
+"      <separator/>"
+"      <menuitem action='Trace'/>"
+"      <menuitem action='Profile'/>"
+"      <menuitem action='Stopwatch'/>"
+"      <menuitem action='Scope'/>"
+"    </menu>"
+"    <menu action='Edit'>"
+"      <menuitem action='Preferences'/>"
+"    </menu>"
+"    <menu action='Help'>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
 
 GtkWidget *dispatcher_window = 0;
 //========================================================================
@@ -1445,8 +1218,8 @@ public:
 MainWindow TheWindow;
 
 MainWindow::MainWindow()
+  : timeW(0)
 {
-  timeW=0;
 }
 
 void MainWindow::Update()
@@ -1483,8 +1256,6 @@ void MainWindow::Create ()
   GtkWidget *separator;
   GtkWidget *button;
   GtkWidget *frame;
-  GtkAccelGroup *accel_group;
-
   int x,y,width,height;
 
   GtkWidget *update_rate_menu;
@@ -1501,26 +1272,29 @@ void MainWindow::Create ()
     width=1;
   if(!config_get_variable("dispatcher", "height", &height))
     height=1;
-  gtk_window_set_default_size(GTK_WINDOW(dispatcher_window), width,height);
-  gtk_widget_set_uposition(GTK_WIDGET(dispatcher_window),x,y);
+  gtk_window_resize(GTK_WINDOW(dispatcher_window), width, height);
+  gtk_window_move(GTK_WINDOW(dispatcher_window), x, y);
 
 
-  gtk_signal_connect (GTK_OBJECT (dispatcher_window), "delete-event",
-                      GTK_SIGNAL_FUNC (dispatcher_delete_event),
+  g_signal_connect (dispatcher_window, "delete-event",
+                      G_CALLBACK(dispatcher_delete_event),
                       0);
 
-#if GTK_MAJOR_VERSION >= 2
-  accel_group = gtk_accel_group_new();
-#else
-  accel_group = gtk_accel_group_get_default ();
-#endif
-  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
-  gtk_object_set_data_full (GTK_OBJECT (dispatcher_window),
-                            "<main>",
-                            item_factory,
-                            (GtkDestroyNotify) gtk_object_unref);
-  //      gtk_accel_group_attach (accel_group, GTK_OBJECT (dispatcher_window));
-  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, 0);
+  GtkActionGroup *actions = gtk_action_group_new("Actions");
+  gtk_action_group_add_actions(actions, entries, G_N_ELEMENTS(entries), NULL);
+  gtk_action_group_add_toggle_actions(actions, toggle_entries,
+    G_N_ELEMENTS(toggle_entries), NULL);
+
+  ui = gtk_ui_manager_new();
+  gtk_ui_manager_insert_action_group(ui, actions, 0);
+  g_object_unref(actions);
+  gtk_window_add_accel_group(GTK_WINDOW(dispatcher_window),
+    gtk_ui_manager_get_accel_group(ui));
+
+  if (!gtk_ui_manager_add_ui_from_string(ui, ui_info, -1, NULL)) {
+    g_error("building menus failed");
+  }
+
   gtk_window_set_title (GTK_WINDOW (dispatcher_window),
                         VERSION);
   gtk_container_set_border_width (GTK_CONTAINER (dispatcher_window), 0);
@@ -1529,7 +1303,7 @@ void MainWindow::Create ()
   gtk_container_add (GTK_CONTAINER (dispatcher_window), box1);
 
   gtk_box_pack_start (GTK_BOX (box1),
-                      gtk_item_factory_get_widget (item_factory, "<main>"),
+                      gtk_ui_manager_get_widget(ui, "/menu"),
                       FALSE, FALSE, 0);
 
 
@@ -1542,33 +1316,33 @@ void MainWindow::Create ()
 
   // Buttons
   button = gtk_button_new_with_label ("step");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) stepbutton_cb, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(stepbutton_cb), 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("over");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) overbutton_cb, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(overbutton_cb), 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("finish");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) finishbutton_cb, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(finishbutton_cb), 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("run");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) runbutton_cb, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(runbutton_cb), 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("stop");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) stopbutton_cb, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(stopbutton_cb), 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("reset");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) resetbutton_cb, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(resetbutton_cb), 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
 
@@ -1586,9 +1360,8 @@ void MainWindow::Create ()
   //
   // Gui Update Rate
   //
-  cout << "SimulationMode:"<<SimulationMode<<endl;
 
-  update_rate_menu = gtk_combo_box_new_text();
+  update_rate_menu = gtk_combo_box_text_new();
   gtk_container_add(GTK_CONTAINER(frame),update_rate_menu);
 
   new UpdateRateMenuItem(update_rate_menu,'5',"Without gui (fastest simulation)",0);
@@ -1604,14 +1377,12 @@ void MainWindow::Create ()
 
   UpdateRateMenuItem *umi = UpdateRateMenuItemMap[SimulationMode];
 
-  if(!umi)
-    cout << "error selecting update rate menu\n";
   umi->Select();
 
   gtk_combo_box_set_active(GTK_COMBO_BOX(update_rate_menu), umi->menu_index);
 
-  gtk_signal_connect(GTK_OBJECT(update_rate_menu),"changed",
-                     (GtkSignalFunc) gui_update_cb,
+  g_signal_connect(update_rate_menu, "changed",
+                     G_CALLBACK(gui_update_cb),
                      (gpointer)update_rate_menu);
 
   gtk_box_pack_start (GTK_BOX (buttonbox), frame, FALSE, FALSE, 5);
@@ -1632,8 +1403,8 @@ void MainWindow::Create ()
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
   button = gtk_button_new_with_label ("Quit gpsim");
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                     (GtkSignalFunc) do_quit_app, 0);
+  g_signal_connect(button, "clicked",
+                     G_CALLBACK(do_quit_app), 0);
 
   gtk_box_pack_start (GTK_BOX (box1), button, FALSE, TRUE, 5);
   gtk_widget_show_all (dispatcher_window);
