@@ -28,9 +28,7 @@ Boston, MA 02111-1307, USA.  */
 
 class RAM_RegisterWindow;
 class EEPROM_RegisterWindow;
-#if defined(NEW_SOURCE_BROWSER)
 class SourceWindow;
-#endif
 class SourceBrowser_Window;
 class SourceBrowserParent_Window;
 class Symbol_Window;
@@ -51,14 +49,12 @@ class GUIRegisterList;
 class GUI_Processor {
  public:
 
-  GUI_Processor(void);
+  GUI_Processor();
   void SetCPU(Processor *new_cpu);
 
   RAM_RegisterWindow *regwin_ram;
   EEPROM_RegisterWindow *regwin_eeprom;
-#if defined(NEW_SOURCE_BROWSER)
   SourceWindow *source_window;
-#endif
   SourceBrowser_Window *program_memory;
   SourceBrowserParent_Window *source_browser;
   Symbol_Window *symbol_window;
