@@ -23,20 +23,16 @@ Boston, MA 02111-1307, USA.  */
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <typeinfo>
 
 #include "../config.h"
 #ifdef HAVE_GUI
 
-#include <unistd.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <string.h>
 
-#include <assert.h>
-
 #include "../src/interface.h"
 #include "../src/trace.h"
 #include "../src/breakpoints.h"
@@ -147,10 +143,6 @@ static menu_item menu_items[] = {
     {"Settings...", MENU_SETTINGS}
 };
 
-extern int font_dialog_browse(GtkWidget *w, gpointer user_data);
-static int dlg_x=200, dlg_y=200;
-
-
 // Used only in popup menus
 Register_Window *popup_rw;
 
@@ -347,7 +339,7 @@ class RegisterWindowXREF : public CrossReferenceToGUI
 {
 public:
 
-  void Update(int new_value)
+  virtual void Update(int new_value)
   {
     GUIRegister *reg;
     Register_Window *rw;
@@ -370,7 +362,6 @@ public:
 
     rw->UpdateASCII(reg->row);
   }
-
 };
 
 
@@ -431,348 +422,162 @@ gtk_sheet_REALLY_set_row_height(GtkSheet *sheet, gint row, gint height)
 
 //========================================================================
 // get_value
-static void a_cb(GtkWidget *w, gpointer user_data)
-{
-    *(int*)user_data=TRUE;
-//    gtk_main_quit();
-}
-static void b_cb(GtkWidget *w, gpointer user_data)
-{
-    *(int*)user_data=FALSE;
-//    gtk_main_quit();
-}
 // used for reading a value from user when break on value is requested
 int gui_get_value(const char *prompt)
 {
-    static GtkWidget *dialog=0;
-    static GtkWidget *label;
-    static GtkWidget *entry;
-    GtkWidget *button;
-    GtkWidget *hbox;
+  GtkWidget *dialog;
 
-    int retval=-1;
+  dialog = gtk_dialog_new_with_buttons("enter value",
+    NULL,
+    GTK_DIALOG_MODAL,
+    "_Cancel", GTK_RESPONSE_CANCEL,
+    "_OK", GTK_RESPONSE_OK,
+    NULL);
 
-    int value;
+  GtkWidget *area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+  GtkWidget *label = gtk_label_new("values can be entered in decimal, hexadecimal, and octal.\nFor example: 31 is the same as 0x1f and 037");
 
-    if(dialog==0) {
-      dialog = gtk_dialog_new();
-      gtk_window_set_title(GTK_WINDOW(dialog),"enter value");
-      //        gtk_signal_connect(GTK_OBJECT(dialog),
-      //                           "configure_event",GTK_SIGNAL_FUNC(configure_event),0);
-      gtk_signal_connect_object(GTK_OBJECT(dialog),
-                                      "delete_event",GTK_SIGNAL_FUNC(gtk_widget_hide),GTK_OBJECT(dialog));
+  gtk_box_pack_start(GTK_BOX(area), label, FALSE, FALSE, 18);
 
-      label=gtk_label_new("values can be entered in decimal, hexadecimal, and octal.\nFor example: 31 is the same as 0x1f and 037");
-      gtk_widget_show(label);
-      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label,FALSE,FALSE,20);
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 6);
+  gtk_box_pack_start(GTK_BOX(area), hbox, FALSE, FALSE, 18);
+  label = gtk_label_new(prompt);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  GtkWidget *entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
 
-      hbox = gtk_hbox_new(0,0);
-      gtk_widget_show(hbox);
-      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,FALSE,FALSE,20);
+  gtk_widget_show_all(dialog);
 
-      button = gtk_button_new_with_label("OK");
-      gtk_widget_show(button);
-      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
-                                FALSE,FALSE,10);
-      gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                                GTK_SIGNAL_FUNC(a_cb),(gpointer)&retval);
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK) {
+    gtk_widget_destroy(dialog);
+    return -1;
+  }
 
-      button = gtk_button_new_with_label("Cancel");
-      gtk_widget_show(button);
-      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
-                                FALSE,FALSE,10);
-      gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                                GTK_SIGNAL_FUNC(b_cb),(gpointer)&retval);
+  const gchar *entry_text;
+  char *end;
+  int value;
 
-      label=gtk_label_new(prompt);
-      gtk_widget_show(label);
-      gtk_box_pack_start(GTK_BOX(hbox), label,
-                                FALSE,FALSE, 20);
+  entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
+  value = strtoul(entry_text, &end, 0);
 
-      entry=gtk_entry_new();
-      gtk_widget_show(entry);
-      gtk_box_pack_start(GTK_BOX(hbox), entry,FALSE,FALSE,20);
-    }
-    else {
-      gtk_label_set_text(GTK_LABEL(label),prompt);
-    }
-
-//    gtk_widget_set_uposition(GTK_WIDGET(dialog),dlg_x,dlg_y);
-    gtk_widget_show_now(dialog);
-
-    gtk_grab_add(dialog);
-    while(retval==-1 && GTK_WIDGET_VISIBLE(dialog))
-      gtk_main_iteration();
-    gtk_grab_remove(dialog);
-
-    gtk_widget_hide(dialog);
-
-    if(retval==(int)TRUE)
-    {
-      char *end;
-      const gchar *entry_text;
-      entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
-      value = strtoul(entry_text,&end,0);
-      if(*entry_text!='\0' && *end=='\0')
-        return value;
-      else
-        return -1;
-    }
+  gtk_widget_destroy(dialog);
 
+  if (*entry_text != '\0' && *end == '\0')
+    return value;
+  else
     return -1;
 }
 
 // used for reading a value from user when break on value is requested
 void gui_get_2values(const char *prompt1, int *value1, const char *prompt2, int *value2)
 {
-    static GtkWidget *dialog=0;
-    GtkWidget *button;
-    GtkWidget *hbox1, *hbox2;
-    static GtkWidget *label1, *label2, *label;
-    static GtkWidget *entry1, *entry2;
+  GtkWidget *dialog;
 
-    int value;
+  dialog = gtk_dialog_new_with_buttons("enter values",
+    NULL,
+    GTK_DIALOG_MODAL,
+    "_Cancel", GTK_RESPONSE_CANCEL,
+    "_OK", GTK_RESPONSE_OK,
+    NULL);
 
-    int retval=-1;
+  GtkWidget *area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+  GtkWidget *label = gtk_label_new("values can be entered in decimal, hexadecimal, and octal.\nFor example: 31 is the same as 0x1f and 037");
+  gtk_box_pack_start(GTK_BOX(area), label, FALSE, FALSE, 18);
 
-    if(dialog==0)
-    {
-        dialog = gtk_dialog_new();
-        gtk_window_set_title(GTK_WINDOW(dialog),"enter values");
-//      gtk_signal_connect(GTK_OBJECT(dialog),
-//                         "configure_event",GTK_SIGNAL_FUNC(configure_event),0);
-        gtk_signal_connect_object(GTK_OBJECT(dialog),
-                                  "delete_event",GTK_SIGNAL_FUNC(gtk_widget_hide),GTK_OBJECT(dialog));
+  GtkWidget *hbox, *entry1, *entry2;
 
-        label=gtk_label_new("values can be entered in decimal, hexadecimal, and octal.\nFor example: 31 is the same as 0x1f and 037");
-        gtk_widget_show(label);
-        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label,FALSE,FALSE,20);
+  hbox = gtk_hbox_new(FALSE, 6);
+  gtk_box_pack_start(GTK_BOX(area), hbox, FALSE, FALSE, 18);
+  label = gtk_label_new(prompt1);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  entry1 = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry1, FALSE, FALSE, 0);
 
-        button = gtk_button_new_with_label("OK");
-        gtk_widget_show(button);
-        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
-                           FALSE,FALSE,10);
-        gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                           GTK_SIGNAL_FUNC(a_cb),(gpointer)&retval);
+  hbox = gtk_hbox_new(FALSE, 6);
+  gtk_box_pack_start(GTK_BOX(area), hbox, FALSE, FALSE, 18);
+  label = gtk_label_new(prompt2);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  entry2 = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), entry2, FALSE, FALSE, 0);
 
-        button = gtk_button_new_with_label("Cancel");
-        gtk_widget_show(button);
-        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
-                           FALSE,FALSE,10);
-        gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                           GTK_SIGNAL_FUNC(b_cb),(gpointer)&retval);
+  gtk_widget_show_all(dialog);
 
-        // Value 1
-        hbox1 = gtk_hbox_new(0,0);
-        gtk_widget_show(hbox1);
-        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox1,FALSE,FALSE,20);
-
-        label1=gtk_label_new(prompt1);
-        gtk_widget_show(label1);
-        gtk_box_pack_start(GTK_BOX(hbox1), label1,
-                           FALSE,FALSE, 20);
-
-        entry1=gtk_entry_new();
-        gtk_widget_show(entry1);
-        gtk_box_pack_start(GTK_BOX(hbox1), entry1,FALSE,FALSE,20);
-
-        // Value 2
-        hbox2 = gtk_hbox_new(0,0);
-        gtk_widget_show(hbox2);
-        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox2,FALSE,FALSE,20);
-
-        label2=gtk_label_new(prompt2);
-        gtk_widget_show(label2);
-        gtk_box_pack_start(GTK_BOX(hbox2), label2,
-                           FALSE,FALSE, 20);
-
-        entry2=gtk_entry_new();
-        gtk_widget_show(entry2);
-        gtk_box_pack_start(GTK_BOX(hbox2), entry2,FALSE,FALSE,20);
-
-    }
-    else
-    {
-        gtk_label_set_text(GTK_LABEL(label1),prompt1);
-        gtk_label_set_text(GTK_LABEL(label2),prompt2);
-    }
-
-//    gtk_widget_set_uposition(GTK_WIDGET(dialog),dlg_x,dlg_y);
-    gtk_widget_show_now(dialog);
-
-    gtk_grab_add(dialog);
-    while(retval==-1 && GTK_WIDGET_VISIBLE(dialog))
-      gtk_main_iteration();
-    gtk_grab_remove(dialog);
-
-    gtk_widget_hide(dialog);
-
-    if(retval==(int)TRUE)
-    {
-      // "Ok"
-
-      char *end;
-      const gchar *entry_text;
-
-      entry_text = gtk_entry_get_text(GTK_ENTRY(entry1));
-      value = strtoul(entry_text,&end,0);
-      if(*entry_text=='\0' || *end!='\0')
-      {
-          *value1=-1;
-          *value2=-1;
-          return;
-      }
-      *value1=value;
-
-      entry_text = gtk_entry_get_text(GTK_ENTRY(entry2));
-      value = strtoul(entry_text,&end,0);
-      if(*entry_text=='\0' || *end!='\0')
-      {
-        *value1=-1;
-        *value2=-1;
-        return;
-      }
-      *value2=value;
-      return;
-    }
-
-    // "Cancel"
-
-    *value1=-1;
-    *value2=-1;
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK) {
+    *value1 = -1;
+    *value2 = -1;
+    gtk_widget_destroy(dialog);
     return;
+  }
+
+  char *end;
+  const gchar *entry_text;
+  int value;
+
+  entry_text = gtk_entry_get_text(GTK_ENTRY(entry1));
+  value = strtoul(entry_text, &end, 0);
+  if (*entry_text == '\0' || *end != '\0') {
+    *value1 = -1;
+    *value2 = -1;
+    gtk_widget_destroy(dialog);
+  }
+  *value1 = value;
+
+  entry_text = gtk_entry_get_text(GTK_ENTRY(entry2));
+  value = strtoul(entry_text, &end, 0);
+  if (*entry_text == '\0' || *end != '\0') {
+    *value1 = -1;
+    *value2 = -1;
+    gtk_widget_destroy(dialog);
+  }
+  *value2 = value;
+  gtk_widget_destroy(dialog);
 }
 
-
-static const char *file_selection_name;
-static int filemode;
-static int fs_done;
-
-static void
-file_selection_ok (GtkWidget        *w,
-                   GtkFileSelection *fs)
+// Select logging file.
+// Returns pointer to file name. This needs to be freed with g_free()
+// Can return the optional mode
+static char *gui_get_log_settings(int *mode)
 {
-    file_selection_name=gtk_file_selection_get_filename (fs);
-    fs_done=1;
-}
+  GtkWidget *dialog = gtk_file_chooser_dialog_new("Log settings",
+    NULL,
+    GTK_FILE_CHOOSER_ACTION_SAVE,
+    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+    GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+    NULL);
 
-static void
-file_selection_cancel (GtkWidget        *w,
-                       GtkFileSelection *fs)
-{
-    file_selection_name=0;
-    fs_done=1;
-}
+  gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
 
-static void
-modepopup_activated(GtkWidget *widget, gpointer data)
-{
-  Dprintf((" modepopup_activated\n"));
+  GtkWidget *combo_text;
 
-    char *modestring;
+  if (mode) {
+    combo_text = gtk_combo_box_text_new();
+    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_text), "ASCII");
+    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_text), "LXT");
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_text), 0);
 
-    modestring=(char*)data;
+    GtkWidget *hbox = gtk_hbox_new(FALSE, 12);
+    GtkWidget *label = gtk_label_new("File format:");
+    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), combo_text, FALSE, FALSE, 0);
 
-    if(!strcmp(modestring,"ASCII"))
-        filemode=TRACE_FILE_FORMAT_ASCII;
-    if(!strcmp(modestring,"LXT"))
-        filemode=TRACE_FILE_FORMAT_LXT;
-}
+    gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), hbox);
 
-static char *gui_get_log_settings(const char **filename, int *mode)
-{
-    static GtkWidget *window = 0;
-
-    GtkWidget *hbox, *optionmenu, *label;
-
-    GtkWidget *menu;
-    GtkWidget *item;
-
-    const char *prompt="Log settings";
-
-    if (!window)
-    {
-
-        window = gtk_file_selection_new (prompt);
-
-        gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (window));
-
-        gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-
-        gtk_signal_connect_object(GTK_OBJECT(window),
-                                  "delete_event",GTK_SIGNAL_FUNC(gtk_widget_hide),GTK_OBJECT(window));
-//      gtk_signal_connect_object (GTK_OBJECT (window), "destroy",
-//                                 GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-//                                 GTK_OBJECT(window));
-
-        gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
-                            "clicked", GTK_SIGNAL_FUNC(file_selection_ok),
-                            window);
-        gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
-                            "clicked", GTK_SIGNAL_FUNC(file_selection_cancel),
-                            window);
-
-        hbox = gtk_hbox_new(0,0);
-        gtk_widget_show(hbox);
-
-        gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION (window)->action_area),
-                         hbox,
-                         FALSE,FALSE,20);
-
-        label = gtk_label_new("File format:");
-        gtk_box_pack_start(GTK_BOX(hbox),
-                         label,
-                         FALSE,FALSE,20);
-        gtk_widget_show(label);
-
-        optionmenu = gtk_option_menu_new();
-        gtk_widget_show(optionmenu);
-        gtk_box_pack_end(GTK_BOX(hbox),
-                         optionmenu,
-                         FALSE,FALSE,20);
-
-        menu=gtk_menu_new();
-
-        item=gtk_menu_item_new_with_label("ASCII");
-        gtk_signal_connect(GTK_OBJECT(item),"activate",
-                           (GtkSignalFunc) modepopup_activated,
-                           (gpointer)"ASCII");
-//      GTK_WIDGET_SET_FLAGS (item, GTK_SENSITIVE | GTK_CAN_FOCUS);
-        gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu),item);
-        item=gtk_menu_item_new_with_label("LXT");
-        gtk_signal_connect(GTK_OBJECT(item),"activate",
-                           (GtkSignalFunc) modepopup_activated,
-                           (gpointer)"LXT");
-//      GTK_WIDGET_SET_FLAGS (item, GTK_SENSITIVE | GTK_CAN_FOCUS);
-        gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu),item);
-
-        gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);
+    gtk_widget_show_all(hbox);
+  }
 
+  char *file = NULL;
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
+    file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+    if (mode) {
+      gint position = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_text));
+      if (position == 0) *mode = TRACE_FILE_FORMAT_ASCII;
+      if (position == 1) *mode = TRACE_FILE_FORMAT_LXT;
     }
+  }
+  gtk_widget_destroy(dialog);
 
-    file_selection_name=0;
-    gtk_widget_show_now(window);
-
-    fs_done=0;
-    file_selection_name=0;
-    gtk_grab_add(window);
-    while(!fs_done && GTK_WIDGET_VISIBLE(window))
-        gtk_main_iteration();
-    gtk_grab_remove(window);
-
-    gtk_widget_hide(window);
-
-    if(file_selection_name==0)
-    {
-        *filename=0;
-        return 0;
-    }
-
-    *filename=file_selection_name;
-    *mode=filemode;
-    return 0;
+  return file;
 }
 
 // called when user has selected a menu item
@@ -786,7 +591,7 @@ popup_activated(GtkWidget *widget, gpointer data)
   GtkSheetRange range;
   unsigned int address;
   int value, mask;
-  const char *filename;
+  char *filename;
   int mode;
 
   Dprintf((" popup_activated\n"));
@@ -868,9 +673,10 @@ popup_activated(GtkWidget *widget, gpointer data)
         return;
         break;
       case MENU_LOG_SETTINGS:
-        gui_get_log_settings(&filename, &mode);
-        if(filename!=0)
+        filename = gui_get_log_settings(&mode);
+        if (filename)
           GetTraceLog().enable_logging(filename,mode);
+        g_free(filename);
         return;
         break;
 
@@ -904,29 +710,21 @@ build_menu(Register_Window *rw)
 
   menu=gtk_menu_new();
 
-
-  item = gtk_tearoff_menu_item_new ();
-  gtk_menu_append (GTK_MENU (menu), item);
-  gtk_widget_show (item);
-
-
   for (i=0; i < (int)(sizeof(menu_items)/sizeof(menu_items[0])) ; i++){
       item=gtk_menu_item_new_with_label(menu_items[i].name);
 
-      gtk_signal_connect(GTK_OBJECT(item),"activate",
-                         (GtkSignalFunc) popup_activated,
+      g_signal_connect(item, "activate",
+                         G_CALLBACK (popup_activated),
                          &menu_items[i]);
-      GTK_WIDGET_SET_FLAGS (item, GTK_SENSITIVE | GTK_CAN_FOCUS);
 
       if(rw->type == REGISTER_EEPROM
          && menu_items[i].id!=MENU_ADD_WATCH
          &&menu_items[i].id!=MENU_SETTINGS)
       {
-          GTK_WIDGET_UNSET_FLAGS (item,
-                                  GTK_SENSITIVE | GTK_CAN_FOCUS);
+        gtk_widget_set_sensitive(item, FALSE);
       }
       gtk_widget_show(item);
-      gtk_menu_append(GTK_MENU(menu),item);
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   }
 
   return menu;
@@ -1099,14 +897,14 @@ void Register_Window::UpdateLabel()
 
     if(col > -1 && row > -1) {
       if(col >= REGISTERS_PER_ROW)
-        gtk_label_set(GTK_LABEL(location), "  ascii  ");
+        gtk_label_set_text(GTK_LABEL(location), "  ascii  ");
       else {
 
         GUIRegister *reg = getRegister(row,col);
 
         const char *n = reg ? reg->name() : "INVALID_REGISTER";
 
-        gtk_label_set(GTK_LABEL(location), n);
+        gtk_label_set_text(GTK_LABEL(location), n);
       }
     }
   }
@@ -1152,16 +950,6 @@ void Register_Window::UpdateLabelEntry()
   UpdateEntry();
 }
 
-static gint configure_event(GtkWidget *widget, GdkEventConfigure *e, gpointer data)
-{
-
-  if(widget->window==0)
-    return 0;
-
-  gdk_window_get_root_origin(widget->window,&dlg_x,&dlg_y);
-  return 0; // what should be returned?, FIXME
-}
-
 //------------------------------------------------------------------------
 void Register_Window::UpdateStyle()
 {
@@ -1200,11 +988,7 @@ void Register_Window::UpdateStyle()
 //------------------------------------------------------------------------
 int Register_Window::LoadStyles()
 {
-#if GTK_MAJOR_VERSION >= 2
   normalfont = pango_font_description_from_string(normalfont_string);
-#else
-  normalfont=gdk_fontset_load (normalfont_string);
-#endif
 
   if(!normalfont)
   {
@@ -1213,7 +997,6 @@ int Register_Window::LoadStyles()
     return 0;
   }
 
-#if GTK_MAJOR_VERSION >= 2
   {
     PangoRectangle rect;
     PangoLayout *layout;
@@ -1227,125 +1010,54 @@ int Register_Window::LoadStyles()
     g_object_unref(G_OBJECT(layout));
   }
 
-#else
-  char_width = gdk_string_width (normalfont,"9");
-#endif
-
   return 1;
 }
 
 /********************** Settings dialog ***************************/
-static int settings_active;
-static void settingsok_cb(GtkWidget *w, gpointer user_data)
-{
-  if(settings_active)
-    settings_active=0;
-}
 
 int Register_Window::SettingsDialog()
 {
-  static GtkWidget *dialog=0;    // fixme
-  GtkWidget *button;
-  GtkWidget *hbox;
-  static GtkWidget *normalfontstringentry;   //fixme
-  GtkWidget *label;
-  int fonts_ok=0;
+  GtkWidget *dialog;
 
+  dialog = gtk_dialog_new_with_buttons("Register window settings",
+    NULL,
+    GTK_DIALOG_MODAL,
+    "_Cancel", GTK_RESPONSE_CANCEL,
+    "_OK", GTK_RESPONSE_OK,
+    NULL);
 
-  if(!dialog)
-  {
-    dialog = gtk_dialog_new();
-    gtk_window_set_title (GTK_WINDOW (dialog), "Register window settings");
-    gtk_signal_connect(GTK_OBJECT(dialog),
-                       "configure_event",GTK_SIGNAL_FUNC(configure_event),0);
-    gtk_signal_connect_object(GTK_OBJECT(dialog),
-                              "delete_event",GTK_SIGNAL_FUNC(gtk_widget_hide),GTK_OBJECT(dialog));
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 6);
+  GtkWidget *area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+  gtk_box_pack_start(GTK_BOX(area), hbox, FALSE, FALSE, 18);
 
+  GtkWidget *label = gtk_label_new("Normal font:");
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
-    // Normal font
-    hbox = gtk_hbox_new(0,0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,FALSE,FALSE,20);
-    gtk_widget_show(hbox);
-    label=gtk_label_new("Normal font:");
-    gtk_box_pack_start(GTK_BOX(hbox), label,
-                       FALSE,FALSE, 20);
-    gtk_widget_show(label);
-    normalfontstringentry=gtk_entry_new();
-    gtk_box_pack_start(GTK_BOX(hbox), normalfontstringentry,
-                       TRUE, TRUE, 0);
-    gtk_widget_show(normalfontstringentry);
-    button = gtk_button_new_with_label("Browse...");
-    gtk_widget_show(button);
-    gtk_box_pack_start(GTK_BOX(hbox), button,
-                       FALSE,FALSE,10);
-    gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                       GTK_SIGNAL_FUNC(font_dialog_browse),(gpointer)normalfontstringentry);
+  GtkWidget *font_btn = gtk_font_button_new_with_font(normalfont_string);
+  gtk_box_pack_start(GTK_BOX(hbox), font_btn, FALSE, FALSE, 0);
 
+  gtk_widget_show_all(dialog);
 
-    // OK button
-    button = gtk_button_new_with_label("OK");
-    gtk_widget_show(button);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
-                       FALSE,FALSE,10);
-    gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                       GTK_SIGNAL_FUNC(settingsok_cb),(gpointer)dialog);
-  }
-
-  gtk_entry_set_text(GTK_ENTRY(normalfontstringentry), normalfont_string);
-
-  gtk_widget_set_uposition(GTK_WIDGET(dialog),dlg_x,dlg_y);
-  gtk_widget_show_now(dialog);
-
-
-
-#if GTK_MAJOR_VERSION >= 2
-  PangoFontDescription *font=0;
-#else
-  GdkFont *font=0;
-#endif
-
-  while(fonts_ok!=1)
-  {
-    char fontname[256];
-
-    settings_active=1;
-    while(settings_active)
-      gtk_main_iteration();
-
-    fonts_ok=0;
-
-    strcpy(fontname,gtk_entry_get_text(GTK_ENTRY(normalfontstringentry)));
-#if GTK_MAJOR_VERSION >= 2
-    if((font=pango_font_description_from_string(fontname))==0)
-#else
-      if((font=gdk_fontset_load(fontname))==0)
-#endif
-        {
-          if(gui_question("Font did not load!","Try again","Ignore/Cancel")==FALSE)
-            break;
-        }
-      else
-        {
-#if GTK_MAJOR_VERSION >= 2
-#else
-          gdk_font_unref(font);
-#endif
-          strcpy(normalfont_string,gtk_entry_get_text(GTK_ENTRY(normalfontstringentry)));
-          config_set_string(name(),"normalfont",normalfont_string);
-          fonts_ok++;
-        }
-  }
-
-  if(!LoadStyles())
-  {
-    printf("no font is available\n");
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK) {
+    gtk_widget_destroy(dialog);
     return 0;
   }
 
-  gtk_sheet_freeze(register_sheet);
-  UpdateStyle();
-  gtk_sheet_thaw(register_sheet);
-  gtk_widget_hide(dialog);
+  PangoFontDescription *font;
+  const gchar *font_name = gtk_font_button_get_font_name(GTK_FONT_BUTTON(font_btn));
+  font = pango_font_description_from_string(font_name);
+
+  if (font) {
+    pango_font_description_free(font);
+    strcpy(normalfont_string, font_name);
+    config_set_string(name(), "normalfont", normalfont_string);
+
+    gtk_sheet_freeze(register_sheet);
+    UpdateStyle();
+    gtk_sheet_thaw(register_sheet);
+  }
+
+  gtk_widget_destroy(dialog);
 
   return 0;
 }
@@ -1361,18 +1073,15 @@ clipboard_handler(GtkWidget *widget, GdkEventKey *key)
 
   if(key->state & GDK_CONTROL_MASK || key->keyval==GDK_Control_L ||
      key->keyval==GDK_Control_R){
-    if((key->keyval=='c' || key->keyval == 'C') && sheet->state != GTK_STATE_NORMAL){
-#if GTK_MAJOR_VERSION >= 2
+    if((key->keyval=='c' || key->keyval == 'C') && sheet->state != gint(GTK_STATE_NORMAL)){
+
       /*
         --- tsd - commented out because this function
         is not defined in the official gtkextra-2.0 release.
       if (gtk_sheet_in_clip(sheet))
         gtk_sheet_unclip_range(sheet);
       */
-#else
-      if(GTK_SHEET_IN_CLIP(sheet))
-        gtk_sheet_unclip_range(sheet);
-#endif
+
       gtk_sheet_clip_range(sheet, &sheet->range);
     }
     if(key->keyval=='x' || key->keyval == 'X')
@@ -1466,7 +1175,7 @@ show_sheet_entry(GtkWidget *widget, Register_Window *rw)
       return;
     }
 
-  if(!GTK_WIDGET_HAS_FOCUS(widget)) return;
+  if(!gtk_widget_has_focus(widget)) return;
 
   sheet=GTK_SHEET(rw->register_sheet);
   sheet_entry = GTK_ENTRY(gtk_sheet_get_entry(sheet));
@@ -1531,7 +1240,7 @@ show_entry(GtkWidget *widget, Register_Window *rw)
         return;
     }
 
-    if(!GTK_WIDGET_HAS_FOCUS(widget)) return;
+    if(!gtk_widget_has_focus(widget)) return;
 
     rw->UpdateEntry();
 
@@ -1561,10 +1270,10 @@ activate_sheet_cell(GtkWidget *widget, gint row, gint column, Register_Window *r
 
   if(reg && reg->bIsValid() )
     // enable editing valid cells
-    gtk_entry_set_editable(GTK_ENTRY(gtk_sheet_get_entry(rw->register_sheet)), 1);
+    gtk_editable_set_editable(GTK_EDITABLE(gtk_sheet_get_entry(rw->register_sheet)), TRUE);
   else
     // disable editing invalid cells
-    gtk_entry_set_editable(GTK_ENTRY(gtk_sheet_get_entry(rw->register_sheet)), 0);
+    gtk_editable_set_editable(GTK_EDITABLE(gtk_sheet_get_entry(rw->register_sheet)), FALSE);
 
 
   rw->UpdateLabelEntry();
@@ -1664,9 +1373,9 @@ build_entry_bar(GtkWidget *main_vbox, Register_Window *rw)
 
   rw->location=gtk_label_new("");
   gtk_widget_size_request(rw->location, &request);
-  gtk_widget_set_usize(rw->location, 160, request.height);
+  gtk_widget_set_size_request(rw->location, 160, request.height);
   gtk_box_pack_start(GTK_BOX(status_box), rw->location, FALSE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS(rw->location,GTK_CAN_DEFAULT);
+  gtk_widget_set_can_default(rw->location, TRUE);
   gtk_widget_show(rw->location);
 
   rw->entry=gtk_entry_new();
@@ -1853,7 +1562,7 @@ void Register_Window::Update()
   if(!enabled)
     return;
 
-  if(!GTK_WIDGET_VISIBLE(window))
+  if(!gtk_widget_get_visible(window))
     return;
 
   if(!registers_loaded)
@@ -1900,10 +1609,6 @@ void Register_Window::SetRegisterSize()
 
   chars_per_column = 1 + 2*register_size;
 
-  if(pCellFormat)
-    free(pCellFormat);
-
-  pCellFormat = (char *) malloc(10 * sizeof(char));
   sprintf(pCellFormat,"%%0%dx",register_size*2);
 
   if(register_sheet) {
@@ -2034,7 +1739,6 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
        */
 
       cross_reference = new RegisterWindowXREF();
-      cross_reference->parent_window_type =   WT_register_window;
       cross_reference->parent_window = (gpointer) this;
       cross_reference->data = (gpointer) pGReg;
       pGReg->Assign_xref(cross_reference);
@@ -2165,15 +1869,11 @@ void Register_Window::Build()
   build_entry_bar(main_vbox,this);
 
   gtk_window_set_default_size(GTK_WINDOW(window), width,height);
-  gtk_widget_set_uposition(GTK_WIDGET(window),x,y);
+  gtk_window_move(GTK_WINDOW(window), x, y);
   gtk_window_set_wmclass(GTK_WINDOW(window),name(),"Gpsim");
 
   /**************************** load fonts *********************************/
-#if GTK_MAJOR_VERSION >= 2
 #define DEFAULT_NORMALFONT "Monospace 10"
-#else
-#define DEFAULT_NORMALFONT "-adobe-courier-*-r-*-*-*-140-*-*-*-*-*-*"
-#endif
   strcpy(normalfont_string,DEFAULT_NORMALFONT);
   if(config_get_string(name(),"normalfont",&fontstring))
       strcpy(normalfont_string,fontstring);
@@ -2191,21 +1891,17 @@ void Register_Window::Build()
       }
   }
   UpdateStyle();
-  gtk_signal_connect(GTK_OBJECT (window), "delete_event",
-                     GTK_SIGNAL_FUNC(delete_event), this);
+  g_signal_connect(window, "delete_event",
+                     G_CALLBACK(delete_event), this);
 
-  gtk_signal_connect(GTK_OBJECT (window), "show",
-                  GTK_SIGNAL_FUNC(show_event), this);
+  g_signal_connect(window, "show",
+                  G_CALLBACK(show_event), this);
 
   scrolled_window=gtk_scrolled_window_new(0, 0);
 
   gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(register_sheet));
 
-#if GTK_MAJOR_VERSION >= 2
   GTK_SHEET_CLIP_TEXT(register_sheet);
-#else
-  GTK_SHEET_SET_FLAGS(register_sheet, GTK_SHEET_CLIP_TEXT);
-#endif
 
   gtk_widget_show(GTK_WIDGET(register_sheet));
 
@@ -2213,47 +1909,47 @@ void Register_Window::Build()
 
   gtk_box_pack_start(GTK_BOX(main_vbox), scrolled_window, TRUE, TRUE, 0);
 
-  gtk_signal_connect(GTK_OBJECT(gtk_sheet_get_entry(GTK_SHEET(register_sheet))),
-                     "changed", (GtkSignalFunc)show_entry, this);
+  g_signal_connect(gtk_sheet_get_entry(GTK_SHEET(register_sheet)),
+                     "changed", G_CALLBACK(show_entry), this);
 
-  gtk_signal_connect(GTK_OBJECT(register_sheet),
-                     "activate", (GtkSignalFunc)activate_sheet_cell,
+  g_signal_connect(register_sheet,
+                     "activate", G_CALLBACK(activate_sheet_cell),
                      this);
 
-  gtk_signal_connect(GTK_OBJECT(entry),
-                     "changed", (GtkSignalFunc)show_sheet_entry, this);
+  g_signal_connect(entry,
+                     "changed", G_CALLBACK(show_sheet_entry), this);
 
-  gtk_signal_connect(GTK_OBJECT(entry),
-                     "activate", (GtkSignalFunc)activate_sheet_entry,
+  g_signal_connect(entry,
+                     "activate", G_CALLBACK(activate_sheet_entry),
                      this);
 
-  gtk_signal_connect(GTK_OBJECT(register_sheet),
+  g_signal_connect(register_sheet,
                      "key_press_event",
-                     (GtkSignalFunc) clipboard_handler,
+                     G_CALLBACK(clipboard_handler),
                      0);
 
-  gtk_signal_connect(GTK_OBJECT(register_sheet),
+  g_signal_connect(register_sheet,
                      "resize_range",
-                     (GtkSignalFunc) resize_handler,
+                     G_CALLBACK(resize_handler),
                      this);
 
-  gtk_signal_connect(GTK_OBJECT(register_sheet),
+  g_signal_connect(register_sheet,
                      "move_range",
-                     (GtkSignalFunc) move_handler,
+                     G_CALLBACK(move_handler),
                      this);
 
-  gtk_signal_connect(GTK_OBJECT(register_sheet),
+  g_signal_connect(register_sheet,
                      "button_press_event",
-                     (GtkSignalFunc) do_popup,
+                     G_CALLBACK(do_popup),
                      this);
 
-  gtk_signal_connect(GTK_OBJECT(register_sheet),
+  g_signal_connect(register_sheet,
                      "set_cell",
-                     (GtkSignalFunc) set_cell,
+                     G_CALLBACK(set_cell),
                      this);
 
-  gtk_signal_connect_after(GTK_OBJECT(window), "configure_event",
-                           GTK_SIGNAL_FUNC(gui_object_configure_event),
+  g_signal_connect_after(window, "configure_event",
+                           G_CALLBACK(gui_object_configure_event),
                            this);
 
   SetRegisterSize();
@@ -2285,10 +1981,6 @@ Register_Window::Register_Window(GUI_Processor *_gp)
   int i;
 
   gp = _gp;
-  window = 0;
-  wc = WC_data;
-  wt = WT_register_window;
-  pCellFormat = 0;
   char_width = 0;
   chars_per_column = 3; // assume byte-sized registers
   register_sheet = NULL;
@@ -2301,14 +1993,17 @@ Register_Window::Register_Window(GUI_Processor *_gp)
     row_to_address[i]=-1;
 }
 
+const char *RAM_RegisterWindow::name()
+{
+  return "register_viewer_ram";
+}
 
 RAM_RegisterWindow::RAM_RegisterWindow(GUI_Processor *_gp) :
   Register_Window(_gp)
 {
-  menu = "<main>/Windows/Ram";
+  menu = "/menu/Windows/Ram";
   type = REGISTER_RAM;
 
-  set_name("register_viewer_ram");
   // Add a status bar
   sbw = new StatusBar_Window();
 
@@ -2342,14 +2037,17 @@ void RAM_RegisterWindow::Update()
     sbw->Update();
 }
 
+const char *EEPROM_RegisterWindow::name()
+{
+  return "register_viewer_eeprom";
+}
+
 EEPROM_RegisterWindow::EEPROM_RegisterWindow(GUI_Processor *_gp) :
   Register_Window(_gp)
 {
-  menu = "<main>/Windows/EEPROM";
+  menu = "/menu/Windows/EEPROM";
   type = REGISTER_EEPROM;
 
-  set_name("register_viewer_eeprom");
-
   get_config();
 
   if(enabled)
