@@ -19,38 +19,26 @@ along with gpsim; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
+#include <cstdio>
 #include <typeinfo>
 
 #include "../config.h"
 #ifdef HAVE_GUI
 
-#include <unistd.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
-#include <string.h>
-
-#include <gtkextra/gtkbordercombo.h>
-#include <gtkextra/gtkcolorcombo.h>
-#include <gtkextra/gtksheet.h>
 
 #include "gui.h"
 #include "gui_src.h"
 
-#include <assert.h>
-#include <map>
-
-
 class StepEvent : public KeyEvent
 {
 public:
   void press(gpointer data)
   {
-    SourceBrowser_Window *sbw = (SourceBrowser_Window *) data;
+    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
     if(sbw && sbw->pma)
         sbw->pma->step(1);
   }
@@ -62,7 +50,7 @@ class StepOverEvent : public KeyEvent
 public:
   void press(gpointer data)
   {
-    SourceBrowser_Window *sbw = (SourceBrowser_Window *) data;
+    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
     if(sbw && sbw->pma)
       // Step Over Next instruction, or hll statement
       sbw->pma->step_over();
@@ -90,7 +78,7 @@ class StopEvent : public KeyEvent
 public:
   void press(gpointer data)
   {
-    SourceBrowser_Window *sbw = (SourceBrowser_Window *) data;
+    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
     if(sbw && sbw->pma)
       sbw->pma->stop();
   }
@@ -102,7 +90,7 @@ class FinishEvent : public KeyEvent
 public:
   void press(gpointer data)
   {
-    SourceBrowser_Window *sbw = (SourceBrowser_Window *) data;
+    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
     if(sbw && sbw->pma)
       sbw->pma->finish();
   }
@@ -118,21 +106,12 @@ key_press(GtkWidget *widget,
           gpointer data)
 {
 
-  SourceBrowser_Window *sbw = (SourceBrowser_Window *) data;
+  SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
 
   if(!sbw) return(FALSE);
   if(!sbw->gp) return(FALSE);
   if(!sbw->gp->cpu) return(FALSE);
 
-  // fix this
-  if(sbw->wt == WT_opcode_source_window)
-  {
-      SourceBrowserOpcode_Window *sbow = (SourceBrowserOpcode_Window*)sbw;
-
-      if(gtk_notebook_get_current_page(GTK_NOTEBOOK(sbow->notebook)))
-          return FALSE;
-
-  }
 
   KeyEvent *pKE = KeyMap[key->keyval];
   if(pKE)
@@ -163,7 +142,7 @@ void SourceBrowser_Window::UpdateLine(int address)
   printf("%s shouldn't be called \n",__FUNCTION__);
 
 }
-void SourceBrowser_Window::Update(void)
+void SourceBrowser_Window::Update()
 {
   if(!gp || !gp->cpu)
     return;
@@ -172,18 +151,18 @@ void SourceBrowser_Window::Update(void)
 }
 
 
-void SourceBrowser_Window::Create(void)
+void SourceBrowser_Window::Create()
 {
   last_simulation_mode = eSM_INITIAL;
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   gtk_window_set_default_size(GTK_WINDOW(window), width,height);
-  gtk_widget_set_uposition(GTK_WIDGET(window),x,y);
+  gtk_window_move(GTK_WINDOW(window), x, y);
   gtk_window_set_wmclass(GTK_WINDOW(window),name(),"Gpsim");
 
-  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
-                      GTK_SIGNAL_FUNC(delete_event),
+  g_signal_connect (window, "delete_event",
+                      G_CALLBACK(delete_event),
                       (gpointer) this);
 
   // FIXME - populate the KeyMap map with source browser functions. This should
@@ -191,18 +170,18 @@ void SourceBrowser_Window::Create(void)
 
   KeyMap['s'] = new StepEvent();
   KeyMap['S'] = KeyMap['s'];
-  KeyMap[GDK_F7] = KeyMap['s'];
+  KeyMap[GDK_KEY_F7] = KeyMap['s'];
 
   KeyMap['o'] = new StepOverEvent();
   KeyMap['O'] = KeyMap['o'];
   KeyMap['n'] = KeyMap['o'];
-  KeyMap[GDK_F8] = KeyMap['o'];
+  KeyMap[GDK_KEY_F8] = KeyMap['o'];
 
   KeyMap['r'] = new RunEvent();
   KeyMap['R'] = KeyMap['r'];
-  KeyMap[GDK_F9] = KeyMap['r'];
+  KeyMap[GDK_KEY_F9] = KeyMap['r'];
 
-  KeyMap[GDK_Escape] = new StopEvent();
+  KeyMap[GDK_KEY_Escape] = new StopEvent();
 
   KeyMap['f'] = new FinishEvent();
   KeyMap['F'] = KeyMap['f'];
@@ -210,8 +189,8 @@ void SourceBrowser_Window::Create(void)
   /* Add a signal handler for key press events. This will capture
    * key commands for single stepping, running, etc.
    */
-  gtk_signal_connect(GTK_OBJECT(window),"key_press_event",
-                     (GtkSignalFunc) key_press,
+  g_signal_connect(window, "key_press_event",
+                     G_CALLBACK(key_press),
                      (gpointer) this);
 
 
@@ -225,8 +204,7 @@ void SourceBrowser_Window::Create(void)
 }
 
 void SourceBrowser_Window::SetTitle() {
-  char buffer[256];
-
+  char *buffer = 0;
   if (gp->cpu == NULL || pma == NULL) {
       return;
   }
@@ -239,16 +217,19 @@ void SourceBrowser_Window::SetTitle() {
     sLastPmaName == pma->name()) {
       return;
   }
+
   last_simulation_mode = gp->cpu->simulation_mode;
   const char * sStatus;
   if (gp->cpu->simulation_mode == eSM_RUNNING)
     sStatus = "Run";
   else // if (gp->cpu->simulation_mode == eSM_STOPPED)
     sStatus = "Stopped";
-  sprintf(buffer,"Source Browser: [%s] %s", sStatus, pma != NULL ?
-    pma->name().c_str() : "" );
+
+  buffer = g_strdup_printf(buffer,
+    "Source Browser: [%s] %s", sStatus, pma->name().c_str());
   sLastPmaName = pma->name();
   gtk_window_set_title (GTK_WINDOW (window), buffer);
+  g_free(buffer);
 }
 
 void SourceBrowser_Window::NewProcessor(GUI_Processor *gp)
@@ -273,19 +254,14 @@ void SourceBrowser_Window::SelectAddress(Value *addrSym)
 }
 
 
-gint gui_object_configure_event(GtkWidget *widget, GdkEventConfigure *e, GUI_Object *go)
+gboolean gui_object_configure_event(GtkWidget *widget, GdkEventConfigure *e, GUI_Object *go)
 {
-//    struct gui_config_winattr winattr;
+  gtk_window_get_position(GTK_WINDOW(widget), &go->x, &go->y);
+  gtk_window_get_size(GTK_WINDOW(widget), &go->width, &go->height);
 
-    if(widget->window==0)
-        return 0;
+  go->set_config();
 
-    gdk_window_get_root_origin(widget->window,&go->x,&go->y);
-    gdk_window_get_size(widget->window,&go->width,&go->height);
-
-    go->set_config();
-
-    return 0; // what should be returned?, FIXME
+  return FALSE;
 }
 
 #endif // HAVE_GUI
