@@ -20,8 +20,9 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 #include "settings_exdbm.h"
-#include <stdlib.h>
+#include <cstdlib>
 #include <cstdio>
+#include <string>
 
 SettingsEXdbm::SettingsEXdbm(const char *appl_name)
 {
