@@ -29,18 +29,8 @@ License along with this library; if not, see
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
-#include <time.h>
-#include <stdio.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#include <errno.h>
-#include <stdlib.h>
-#include <string>
-#include <iostream>
-#include <assert.h>
+#include <cstdio>
+#include <cassert>
 
 #include "../config.h"    // get the definition for HAVE_GUI
 #ifdef HAVE_GUI
@@ -48,7 +38,7 @@ License along with this library; if not, see
 #include <gtk/gtk.h>
 #include "../src/gpsim_time.h"
 #include "../src/packages.h"
-
+#include "../src/stimuli.h"
 #include "encoder.h"
 
 #define PIN_A	(0x01)
@@ -118,10 +108,10 @@ void Encoder::create_widget(Encoder *enc)
   gtk_container_set_border_width (GTK_CONTAINER (buttonl), 5);
   gtk_container_set_border_width (GTK_CONTAINER (buttonr), 5);
 
-  gtk_signal_connect (GTK_OBJECT (buttonl), "pressed",			    
-		      GTK_SIGNAL_FUNC (ccw_cb), (gpointer)enc);
-  gtk_signal_connect (GTK_OBJECT (buttonr), "pressed",			    
-		      GTK_SIGNAL_FUNC (cw_cb), (gpointer)enc);
+  g_signal_connect (buttonl, "pressed",			    
+		      G_CALLBACK(ccw_cb), (gpointer)enc);
+  g_signal_connect (buttonr, "pressed",			    
+		      G_CALLBACK(cw_cb), (gpointer)enc);
 	
 		
   gtk_widget_show(buttonl);
@@ -227,8 +217,6 @@ Encoder::callback()
 	    toggle_a();
 	    rs = rot_detent;
 	    break;
-	default:
-	    abort();
     }
 }
 
