@@ -64,20 +64,21 @@ License along with this library; if not, see
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
-#include <errno.h>
-#include <stdlib.h>
-#include <string>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
+#include <string>
 #include <typeinfo>
 
 #include "../config.h"    // get the definition for HAVE_GUI
 
 #ifdef HAVE_GUI
 #include <gtk/gtk.h>
-#include <math.h>
+#include <cmath>
 
+#include "../src/stimuli.h"
+#include "../src/value.h"
 #include "../src/gpsim_interface.h"
-#include "../src/gpsim_time.h"
 
 #include "led.h"
 #include "../src/packages.h"
@@ -94,25 +95,23 @@ namespace Leds {
   {
   private:
     Led_base *led;
-    int lastport;
 
   public:
 
-    virtual void SimulationHasStopped (gpointer object)
+    virtual void SimulationHasStopped(gpointer object)
     {
-      GuiUpdate(object);
+      Update(object);
     }
-    virtual void GuiUpdate  (gpointer object)
+    virtual void Update(gpointer object)
     {
       if(led)
         led->update();
     }
 
 
-    LED_Interface(Led_base *_led) : Interface((gpointer *) _led)
+    LED_Interface(Led_base *_led)
+    : Interface((gpointer *) _led), led(_led)
     {
-      led = _led;
-      lastport = -1;
     }
 
   };
@@ -121,7 +120,7 @@ namespace Leds {
   class Led_Input : public IOPIN
   {
   public:
-    Led_Input(const char *n, Led_base *pParent);
+    Led_Input(const std::string &n, Led_base *pParent);
 
     virtual void setDrivenState(bool);
 
@@ -132,8 +131,8 @@ namespace Leds {
 
   //------------------------------------------------------------------------
   //
-  Led_Input::Led_Input(const char *n, Led_base *pParent)
-    : IOPIN(n), m_pParent(pParent)
+  Led_Input::Led_Input(const std::string &n, Led_base *pParent)
+    : IOPIN(n.c_str()), m_pParent(pParent)
   {
   }
 
@@ -143,140 +142,56 @@ namespace Leds {
   }
 
   //------------------------------------------------------------------------
-  void Led_7Segments::update()
-  {
-    update(darea, w_width,w_height);
-  }
+void Led_7Segments::update()
+{
+  if(get_interface().bUsingGUI())
+    gtk_widget_queue_draw(darea);
+}
 
-  void Led_7Segments::callback()
-  {
+gboolean Led_7Segments::led7_expose_event(GtkWidget *widget, GdkEvent *event,
+  gpointer user_data)
+{
+  Led_7Segments *led = static_cast<Led_7Segments *>(user_data);
 
-    get_cycles().set_break_delta( get_interface().get_update_rate()+1, this);
-    update();
+  g_return_val_if_fail (widget != NULL, TRUE);
+  g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
 
-  }
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+  guint max_width = allocation.width;
+  guint max_height = allocation.height;
 
-  void Led_7Segments::update(  GtkWidget *widget,
-                               guint new_width,
-                               guint new_height)
-  {
+  // not a very O-O way of doing it... but here we go directly
+  // to the I/O port and get the values of the segments
+  int segment_states = led->getPinState();
 
-    guint i;
+  cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
 
-    if(!get_interface().bUsingGUI())
-      return;
+  cairo_rectangle(cr, 0.0, 0.0, max_width, max_height);
+  cairo_fill(cr);
 
-    w_width = new_width;
-    w_height = new_height;
-    GdkDrawable *drawable = widget->window;
-
-    if(!GTK_WIDGET_REALIZED(widget))
-      return;
-
-    if(segment_gc==NULL)
-      {
-        segment_gc = gdk_gc_new(darea->window);
-        gdk_gc_set_line_attributes(segment_gc,
-                                   5,
-                                   GDK_LINE_SOLID,
-                                   GDK_CAP_ROUND,
-                                   GDK_JOIN_ROUND);
-        g_assert(segment_gc!=NULL);
-      }
-
-
-    // not a very O-O way of doing it... but here we go directly
-    // to the I/O port and get the values of the segments
-    int segment_states = getPinState();
-
-    GdkGC *gc = segment_gc;
-
-    gdk_gc_set_foreground(gc,
-                          &led_background_color);
-
-    gdk_draw_rectangle (drawable, gc,
-                        TRUE,
-                        0,
-                        0,
-                        w_width,
-                        w_height);
-
-
-    // cout << "expose led, segment states = " << segment_states << '\n';
-
-    if( (segment_states & 1) == 0) {
-      // common cathode, cathode must be low to turn
-      //digits on.
-
-      gdk_gc_set_foreground(gc,&led_segment_on_color);
-
-      for(i=0; i<7; i++) {
-        if(segment_states & (2<<i))
-          gdk_draw_polygon ( drawable,
-                             gc,
-                             TRUE,
-                             segments[i].p,
-                             6);
-
-
-      }
+  for (int i = 0; i < 7; ++i) {
+    // common cathode, cathode must be low to turn
+    //digits on.
+    if ((segment_states & 1) == 0 && segment_states & (2 << i)) {
+      cairo_set_source_rgb(cr, 0.75, 0.0, 0.0);
+    } else {
+      cairo_set_source_rgb(cr, 0.25, 0.0, 0.0);
     }
 
-    gdk_gc_set_foreground(gc,&led_segment_off_color);
-
-    // turn off the segments that aren't being driven.
-
-    for(i=0; i<7; i++) {
-      if((segment_states & (2<<i)) == 0)
-        gdk_draw_polygon ( drawable,
-                           gc,
-                           TRUE,
-                           segments[i].p,
-                           6);
-
+    XfPoint *pts = &(led->seg_pts[i][0]);
+    cairo_move_to(cr, pts[0].x, pts[0].y);
+    for (int j = 1; j < MAX_PTS; ++j) {
+      cairo_line_to(cr, pts[j].x, pts[j].y);
     }
+    cairo_line_to(cr, pts[0].x, pts[0].y);
+    cairo_fill(cr);
   }
 
+  cairo_destroy(cr);
 
-  static gint
-  led7_expose_event (GtkWidget *widget,
-                     GdkEvent  *event,
-                     gpointer   user_data)
-  {
-
-
-    Led_7Segments *led;
-    guint max_width;
-    guint max_height;
-
-    g_return_val_if_fail (widget != NULL, TRUE);
-    g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
-
-    // de-reference the user_data into an led object
-    led = (Led_7Segments *)user_data;
-
-    max_width = widget->allocation.width;
-    max_height = widget->allocation.height;
-
-    led->update(widget,max_width,max_height);
-
-    return TRUE;
-  }
-
-  static gint
-  cursor_event (GtkWidget          *widget,
-                GdkEvent           *event,
-                gpointer  *user_data)
-  {
-    if ((event->type == GDK_BUTTON_PRESS) &&
-        ((event->button.button == 1) ||
-         (event->button.button == 3)))
-      {
-        return TRUE;
-      }
-
-    return FALSE;
-  }
+  return TRUE;
+}
 
   //-------------------------------------------------------------------
   // build_segments
@@ -290,8 +205,8 @@ namespace Leds {
   // This routine will calculate what those points should be and
   // store them an arrary.
 
-  void Led_7Segments::build_segments( int w, int h)
-  {
+void Led_7Segments::build_segments( int w, int h)
+{
     XfPoint *pts;
     float spacer, hskip, fslope, bslope, midpt, seg_width, segxw;
     float invcosphi, invsinphi, invcospsi, invsinpsi, slope;
@@ -304,21 +219,21 @@ namespace Leds {
 
     // Hard code the display parameters...
 
-    space_factor = (float)0.13;
-    width_factor = (float)0.13;
-    sxw = (float)0.13;
-    angle = 6;
+    float space_factor = 0.13;
+    float width_factor = 0.13;
+    float sxw = 0.13;
+    float angle = 6;
 
     /* define various useful constants */
 
     segxw = sxw * w;
     slope = angle;
     seg_width = width_factor * w;
-    spacer = (float)w * space_factor;
-    hskip = (float)(seg_width * 0.125);
+    spacer = w * space_factor;
+    hskip = seg_width * 0.125;
     fslope = 1 / (segxw/seg_width + 1/slope);
     bslope = -1 / (segxw/seg_width - 1/slope);
-    midpt = (float)h / 2;
+    midpt = h / 2;
 
     /* define some trigonometric values */
     /*  phi is the forward angle separating two segments;
@@ -351,13 +266,7 @@ namespace Leds {
 
     xfactor = w - 2 * spacer - h / slope - segxw;
 
-    /*
-      cout << "temp_xpts[2] " << temp_xpts[2] << '\n';
-      cout << "dx3 " << dx3 << '\n';
-      cout << "fslope " << fslope << '\n';
-    */
-
-    /* calculate the digit positions */
+    // calculate the digit positions
 
     pts = seg_pts[TOP];
     pts[0].y = pts[1].y = 0;
@@ -447,87 +356,21 @@ namespace Leds {
     pts[3].x = seg_pts[BOTTOM][2].x + 2 * dx5;
     pts[4].x = seg_pts[BOTTOM][1].x + dx4 + dx2;
     pts[5].x = seg_pts[MIDDLE][3].x - 2 * dx1;
+}
 
-    // Convert the floating point points into integers.
-    int i,j;
-    for(i=0; i<NUM_SEGS; i++) {
+void Led_7Segments::build_window()
+{
+  darea = gtk_drawing_area_new();
 
-      for(j=0; j<MAX_PTS; j++) {
+  gtk_widget_set_size_request(darea, 100, 110);
 
-        segments[i].p[j].x = (int)seg_pts[i][j].x;
-        segments[i].p[j].y = (int)seg_pts[i][j].y;
-      }
-    }
+  g_signal_connect(darea, "expose_event", G_CALLBACK(led7_expose_event), this);
+  gtk_widget_set_events(darea, GDK_EXPOSURE_MASK);
 
-  }
+  gtk_widget_show(darea);
 
-  void Led_7Segments::build_window()
-  {
-    GtkWidget *main_vbox;
-    GtkWidget *vbox;
-
-    main_vbox = gtk_vbox_new (FALSE, 5);
-    gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 0);
-
-    vbox =
-      gtk_widget_new (gtk_vbox_get_type (),
-                      "GtkBox::homogeneous", FALSE,
-                      //"GtkBox::spacing", 5,
-                      //"GtkContainer::border_width", 10,
-                      "GtkWidget::parent", main_vbox,
-                      "GtkWidget::visible", TRUE,
-                      NULL);
-    gtk_widget_show(vbox);
-
-
-    darea = gtk_drawing_area_new ();
-
-    gtk_widget_set_usize (darea,
-                          100,
-                          110);
-    gtk_container_add (GTK_CONTAINER (vbox), darea);
-
-    gtk_signal_connect (GTK_OBJECT (darea),
-                        "expose_event",
-                        GTK_SIGNAL_FUNC (led7_expose_event),
-                        this);
-    gtk_widget_set_events (darea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
-    gtk_signal_connect (GTK_OBJECT (darea),
-                        "button_press_event",
-                        GTK_SIGNAL_FUNC (cursor_event),
-                        NULL);
-
-    gtk_widget_show (darea);
-
-    set_widget(main_vbox);
-
-    segment_gc=NULL;
-
-    // The 'on' color is bright red
-    led_segment_on_color.red = 0xc000;
-    led_segment_on_color.green = 0x0000;
-    led_segment_on_color.blue = 0x0000;
-
-    gdk_colormap_alloc_color(gdk_colormap_get_system(), &led_segment_on_color, FALSE, TRUE);
-
-    // The `off' color is dark red
-    led_segment_off_color.red = 0x4000;
-    led_segment_off_color.green = 0x0000;
-    led_segment_off_color.blue = 0x0000;
-
-    gdk_colormap_alloc_color(gdk_colormap_get_system(), &led_segment_off_color, FALSE, TRUE);
-
-    // The background is black like my coffee
-    led_background_color.red = 0x0000;
-    led_background_color.green = 0x0000;
-    led_background_color.blue = 0x0000;
-
-    gdk_colormap_alloc_color(gdk_colormap_get_system(), &led_background_color, FALSE, TRUE);
-
-    //  }
-
-
-  }
+  set_widget(darea);
+}
 
   //--------------------------------------------------------------
 
@@ -541,18 +384,14 @@ namespace Leds {
 
     led_interface = new LED_Interface(this);
     get_interface().add_interface(led_interface);
-    callback();
+
     create_iopin_map();
   }
 
-  Led_7Segments::~Led_7Segments()
-  {
-    /* Deleted elsewhere RRR
-    for (int i=0; i<8; i++)
-      delete m_pins[i];
-    delete [] m_pins;
-   */
-  }
+Led_7Segments::~Led_7Segments()
+{
+  gtk_widget_destroy(darea);
+}
 
   //--------------------------------------------------------------
   // create_iopin_map
@@ -583,44 +422,36 @@ namespace Leds {
     //   the bit positions as LED.seg0, LED.seg1, ..., where LED is the
     //   user-assigned name of the 7-segment LED
 
-    m_pins[0] = new Led_Input( (name() + ".cc").c_str(), this);
+    m_pins[0] = new Led_Input(name() + ".cc", this);
     int i;
     char ch;
-    for (ch='0',i = 1; i<8; i++,ch++)
-      m_pins[i] = new Led_Input((name() + ".seg"+ch).c_str(), this);
+    for (ch = '0', i = 1; i < 8; i++, ch++)
+      m_pins[i] = new Led_Input(name() + ".seg" + ch, this);
 
     for (i=0; i<8; i++)
       assign_pin(i+1,m_pins[i]);
 
-    //initializeAttributes();
-
   }
 
   //--------------------------------------------------------------
-  unsigned int Led_7Segments::getPinState()
-  {
-    unsigned int s=0;
-    double delta_v;
+unsigned int Led_7Segments::getPinState()
+{
+  unsigned int s = 0;
 
-
-    for (int i=1; i<8; i++)
-    {
-      delta_v = m_pins[i]->get_nodeVoltage() -m_pins[0]->get_nodeVoltage();
-      s = (s>>1) | (delta_v > 1.5 ? 0x80 : 0);
-    }
-    return s;
+  for (int i = 1; i < 8; i++) {
+    double delta_v = m_pins[i]->get_nodeVoltage() - m_pins[0]->get_nodeVoltage();
+    s = (s >> 1) | (delta_v > 1.5 ? 0x80 : 0);
   }
 
+  return s;
+}
+
   //--------------------------------------------------------------
   // construct
 
   Module * Led_7Segments::construct(const char *_new_name=0)
   {
-
-    Led_7Segments *l7sP = new Led_7Segments(_new_name) ;
-
-    return l7sP;
-
+    return new Led_7Segments(_new_name);
   }
 
 
@@ -647,7 +478,7 @@ void ColorAttribute::set(Value *v)
   {
      char buff[20];
 
-     v->get((char *)buff, sizeof(buff));
+     v->get(buff, sizeof(buff));
      set(buff);
   }
   else
@@ -676,23 +507,23 @@ void ColorAttribute::get(char *return_str, int len)
     switch(m_led->get_on_color())
     {
     case RED:
-        strncpy(return_str, "red", len);
+        g_strlcpy(return_str, "red", len);
         break;
 
     case ORANGE:
-        strncpy(return_str, "orange", len);
+        g_strlcpy(return_str, "orange", len);
         break;
 
     case GREEN:
-        strncpy(return_str, "green", len);
+        g_strlcpy(return_str, "green", len);
         break;
 
     case YELLOW:
-        strncpy(return_str, "yellow", len);
+        g_strlcpy(return_str, "yellow", len);
         break;
 
     case BLUE:
-        strncpy(return_str, "blue", len);
+        g_strlcpy(return_str, "blue", len);
         break;
 
     }
@@ -701,34 +532,26 @@ void ColorAttribute::get(char *return_str, int len)
 
 bool ColorAttribute::Parse(const char *pValue, Colors &bValue)
 {
+  std::string s(pValue);
 
+  if (s == "red") {
+    bValue = RED;
+    return true;
+  } else if (s == "orange") {
+    bValue = ORANGE;
+    return true;
+  } else if (s == "green") {
+    bValue = GREEN;
+    return true;
+  } else if (s == "yellow") {
+    bValue = YELLOW;
+    return true;
+  } else if (s == "blue") {
+    bValue = BLUE;
+    return true;
+  }
 
-    if(strncmp("red", pValue, sizeof("red")) == 0)
-    {
-        bValue = RED;
-        return true;
-    }
-    else if (strncmp("orange", pValue, sizeof("orange")) == 0)
-    {
-        bValue = ORANGE;
-        return true;
-    }
-    else if (strncmp("green", pValue, sizeof("green")) == 0)
-    {
-        bValue = GREEN;
-        return true;
-    }
-    else if (strncmp("yellow", pValue, sizeof("yellow")) == 0)
-    {
-        bValue = YELLOW;
-        return true;
-    }
-    else if (strncmp("blue", pValue, sizeof("blue")) == 0)
-    {
-        bValue = BLUE;
-        return true;
-    }
-    return false;
+  return false;
 }
 
 
@@ -755,7 +578,7 @@ void ActiveStateAttribute::set(Value *v)
   {
      char buff[20];
 
-     v->get((char *)buff, sizeof(buff));
+     v->get(buff, sizeof(buff));
      set(buff);
   }
   else
@@ -784,11 +607,11 @@ void ActiveStateAttribute::get(char *return_str, int len)
     switch(m_led->get_the_activestate())
     {
     case HIGH:
-        strncpy(return_str, "high", len);
+        g_strlcpy(return_str, "high", len);
         break;
 
     case LOW:
-        strncpy(return_str, "low", len);
+        g_strlcpy(return_str, "low", len);
         break;
     }
   }
@@ -815,90 +638,21 @@ bool ActiveStateAttribute::Parse(const char *pValue, ActiveStates &bValue)
   //-------------------------------------------------------------
   // Led (simple)
   //-------------------------------------------------------------
-  void Led::update()
-  {
-    update(darea, w_width,w_height);
-  }
+void Led::update()
+{
+  if (get_interface().bUsingGUI())
+    gtk_widget_queue_draw(darea);
+}
 
-  void Led::callback()
-  {
-
-    get_cycles().set_break_delta( get_interface().get_update_rate()+1, this);
-    update();
-
-  }
-  void Led::update(  GtkWidget *widget,
-                     guint new_width,
-                     guint new_height)
-  {
-    if(!get_interface().bUsingGUI())
-      return;
-
-    w_width = new_width;
-    w_height = new_height;
-    GdkDrawable *drawable = widget->window;
-
-    if(!GTK_WIDGET_REALIZED(widget))
-      return;
-
-    if(gc==NULL)
-      {
-        gc = gdk_gc_new(darea->window);
-        gdk_gc_set_line_attributes(gc,
-                                   5,
-                                   GDK_LINE_SOLID,
-                                   GDK_CAP_ROUND,
-                                   GDK_JOIN_ROUND);
-        g_assert(gc!=NULL);
-      }
-
-
-    gdk_gc_set_foreground(gc,&led_segment_off_color);
-    gdk_draw_rectangle (drawable, gc,
-                        TRUE,
-                        0,
-                        0,
-                        w_width,
-                        w_height);
-
-    // Led is on when DrivenState=TRUE in current HIGH active state OR
-    // when DrivenState=FALSE in current LOW active state.
-    double delta_v ;
-    if (get_the_activestate() == HIGH)
-	delta_v = m_pin->get_nodeVoltage() - m_pin->get_Vth();
-    else
-	delta_v = m_pin->get_Vth() - m_pin->get_nodeVoltage();
-/*
-    if((m_pin->getDrivenState() && get_the_activestate()==HIGH) ||
-       (!m_pin->getDrivenState() && get_the_activestate()==LOW)) {
-*/
-    if (delta_v > 1.5)
-    {
-        gdk_gc_set_foreground(gc,&led_on_color[on_color]);
-        gdk_draw_arc(drawable, gc,
-                   TRUE,
-                   0,
-                   0,
-                   w_width,
-                   w_height,
-                   0,64*360);
+void Led::set_on_color(Colors color)
+{
+  if (color != on_color) {
+    on_color = color;
+    if (get_interface().bUsingGUI()) {
+      update();
     }
   }
-
-  void Led::set_on_color(Colors color)
-  {
-        if (color != on_color)
-        {
-            on_color = color;
-            if (get_interface().bUsingGUI() )
-            {
-                if (!led_on_color[color].pixel) // color not allocated
-                        gdk_colormap_alloc_color(gdk_colormap_get_system(),
-                                            &led_on_color[color], FALSE, TRUE);
-                update();
-            }
-        }
-  }
+}
 
   void Led::set_the_activestate(ActiveStates activestate)
   {
@@ -917,114 +671,104 @@ bool ActiveStateAttribute::Parse(const char *pValue, ActiveStates &bValue)
   }
 
 
-  static gint
-  led_expose_event (GtkWidget *widget,
-                    GdkEvent  *event,
-                    gpointer   user_data)
-  {
+gboolean Led::led_expose_event(GtkWidget *widget, GdkEvent *event,
+  gpointer user_data)
+{
+  Led *led = static_cast<Led *>(user_data);
 
+  g_return_val_if_fail (widget != NULL, TRUE);
+  g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
 
-    Led *led;
-    guint max_width;
-    guint max_height;
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
 
-    g_return_val_if_fail (widget != NULL, TRUE);
-    g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
+  guint max_width = allocation.width;
+  guint max_height = allocation.height;
 
-    // de-reference the user_data into an led object
-    led = (Led *)user_data;
+  GdkWindow *gdk_win = gtk_widget_get_window(widget);
+  cairo_t *cr = gdk_cairo_create(gdk_win);
 
-    max_width = widget->allocation.width;
-    max_height = widget->allocation.height;
+  // Led is on when DrivenState=TRUE in current HIGH active state OR
+  // when DrivenState=FALSE in current LOW active state.
+  double delta_v ;
+  if (led->get_the_activestate() == HIGH)
+    delta_v = led->m_pin->get_nodeVoltage() - led->m_pin->get_Vth();
+  else
+    delta_v = led->m_pin->get_Vth() - led->m_pin->get_nodeVoltage();
 
-    led->update(widget,max_width,max_height);
-
-    return TRUE;
+  if (delta_v > 1.5) {
+    gdk_cairo_set_source_color(cr, &led->led_on_color[led->on_color]);
+  } else {
+    gdk_cairo_set_source_color(cr, &led->led_segment_off_color);
   }
 
-  void Led::build_window()
-  {
-    GtkWidget *main_vbox;
+  cairo_arc(cr, max_width / 2, max_height / 2, max_width / 2, 0.0, 2 * G_PI);
+  cairo_fill(cr);
 
-    main_vbox = gtk_vbox_new (FALSE, 5);
-    gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 0);
+  cairo_destroy(cr);
 
-    darea = gtk_drawing_area_new ();
+  return FALSE;
+}
 
-    w_height=20;
-    w_width=20;
+void Led::build_window()
+{
+  darea = gtk_drawing_area_new ();
 
-    gtk_widget_set_usize (darea,
-                          w_height,
-                          w_width);
-    gtk_signal_connect (GTK_OBJECT (darea),
-                        "expose_event",
-                        GTK_SIGNAL_FUNC (led_expose_event),
-                        this);
-    gtk_widget_set_events (darea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
-    gtk_widget_show (darea);
+  w_height=20;
+  w_width=20;
 
-    set_widget(darea);
+  gtk_widget_set_size_request(darea, w_height, w_width);
+  g_signal_connect (darea,
+                      "expose_event",
+                      G_CALLBACK(led_expose_event),
+                      this);
+  gtk_widget_set_events (darea, GDK_EXPOSURE_MASK);
+  gtk_widget_show (darea);
 
-    gc=NULL;
+  set_widget(darea);
 
-    // The default 'on' color is bright red
+  // The default 'on' color is bright red
 
-    for(int i = RED; i <= BLUE; i++ )
-        led_on_color[i].pixel = 0;
+  gdk_color_parse("red3", &led_on_color[RED]);
+  gdk_color_parse("orange", &led_on_color[ORANGE]);
+  gdk_color_parse("green", &led_on_color[GREEN]);
+  gdk_color_parse("yellow", &led_on_color[YELLOW]);
+  gdk_color_parse("blue", &led_on_color[BLUE]);
 
-    gdk_color_parse("red3", &led_on_color[RED]);
-    gdk_color_parse("orange", &led_on_color[ORANGE]);
-    gdk_color_parse("green", &led_on_color[GREEN]);
-    gdk_color_parse("yellow", &led_on_color[YELLOW]);
-    gdk_color_parse("blue", &led_on_color[BLUE]);
-
-    GdkColormap *colormap = gdk_colormap_get_system();
-
-    for(int i = RED; i <= BLUE; i++ )
-        gdk_colormap_alloc_color(colormap, &led_on_color[i], FALSE, TRUE);
-
-    // The `off' color is dark red
-    led_segment_off_color.red = 0x4000;
-    led_segment_off_color.green = 0x0000;
-    led_segment_off_color.blue = 0x0000;
-
-    gdk_colormap_alloc_color(colormap, &led_segment_off_color, FALSE, TRUE);
-  }
+  // The `off' color is dark red
+  led_segment_off_color.red = 0x4000;
+  led_segment_off_color.green = 0x0000;
+  led_segment_off_color.blue = 0x0000;
+}
 
   //--------------------------------------------------------------
 
-  Led::Led(const char *name) : Module(name, "Simple LED")
-  {
+Led::Led(const char *name)
+  : Module(name, "Simple LED"), on_color(RED), the_activestate(HIGH)
+{
+  create_iopin_map();
+  // the following will load the driver of the input as would a real
+  // LED
+  m_pin->set_Zth(150.);
+  m_pin->set_Vth(0.);
+  if (get_interface().bUsingGUI())
+    build_window();
 
-    create_iopin_map();
-    // the following will load the driver of the input as would a real
-    // LED
-    m_pin->set_Zth(150.);
-    m_pin->set_Vth(0.);
-    if(get_interface().bUsingGUI())
-      build_window();
+  m_colorAttribute = new ColorAttribute(this);
+  addSymbol(m_colorAttribute);
+  m_activestateAttribute = new ActiveStateAttribute(this);
+  addSymbol(m_activestateAttribute);
+  led_interface = new LED_Interface(this);
+  get_interface().add_interface(led_interface);
+}
 
-
-    on_color = RED;
-    m_colorAttribute = new ColorAttribute(this);
-    addSymbol(m_colorAttribute);
-    the_activestate = HIGH;
-    m_activestateAttribute = new ActiveStateAttribute(this);
-    addSymbol(m_activestateAttribute);
-    led_interface = new LED_Interface(this);
-    get_interface().add_interface(led_interface);
-    callback();
-  }
-
-  Led::~Led()
-  {
-
-    delete m_colorAttribute;
-    /* deleted elsewhere RRR
-    delete m_pin;
-    */
-  }
+Led::~Led()
+{
+  delete m_activestateAttribute;
+  delete led_interface;
+  delete m_colorAttribute;
+  gtk_widget_destroy(darea);
+}
 
   //--------------------------------------------------------------
   // create_iopin_map
@@ -1043,9 +787,9 @@ bool ActiveStateAttribute::Parse(const char *pValue, ActiveStates &bValue)
 
     // Define the LED Cathode. (The anode is implicitly tied to VCC)
 
-    m_pin = new Led_Input((name() + ".in").c_str(), this);
+    m_pin = new Led_Input(name() + ".in", this);
     assign_pin(1, m_pin);
-    //initializeAttributes();
+
   }
 
   //--------------------------------------------------------------
@@ -1053,11 +797,7 @@ bool ActiveStateAttribute::Parse(const char *pValue, ActiveStates &bValue)
 
   Module * Led::construct(const char *_new_name=0)
   {
-
-    Led *ledP = new Led(_new_name);
-
-    return ledP;
-
+    return new Led(_new_name);
   }
 
 }
