@@ -25,14 +25,9 @@ License along with this library; if not, see
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
-#include "../src/stimuli.h"
-#include "../src/ioports.h"
-#include "../src/symbol.h"
-#include "../src/trace.h"
-#include "../src/interface.h"
-
 #include <gtk/gtk.h>
 
+#include "../src/modules.h"
 
 namespace Leds {
 
@@ -57,18 +52,14 @@ namespace Leds {
   //------------------------------------------------------------------------
   // LED base class
 
-  class Led_base //: public Module
+  class Led_base
   {
   public:
-    virtual ~Led_base()
-    {
-    }
+    virtual ~Led_base() {}
 
     virtual void build_window() = 0;
     virtual void update() = 0;
-    virtual void update( GtkWidget *drawable,   
-                         guint max_width,
-                         guint max_height) = 0;
+
     LED_Interface *led_interface;
   };
 
@@ -78,8 +69,8 @@ namespace Leds {
 
   // define a point
   typedef struct {
-    float x;
-    float y;
+    double x;
+    double y;
   } XfPoint;
 
 #define MAX_PTS         6       /* max # of pts per segment polygon */
@@ -99,53 +90,23 @@ namespace Leds {
 
   typedef XfPoint segment_pts[NUM_SEGS][MAX_PTS];
 
-  class Led_7Segments : public Module, public Led_base, public TriggerObject
+  class Led_7Segments : public Module, public Led_base
   {
   public:
-
-    struct {
-      GdkPoint p[7];  // Segments 
-    } segments[7];
-
-    GdkPoint offset;
-    int height,
-      slant,
-      segment_thickness;
-
-    float sxw;
-
-    float               angle;          // rise over run 
-    float               width_factor;   // ratio of digit to segment width 
-    float               small_ratio;    // ratio of small to large digits
-    float               sec_gap;        // gap between normal digits and 
-    // seconds, as ratio to digit width 
-    float               space_factor;   /* ratio of digit width to border sp.*/
-
-
     guint w_width;
     guint w_height;
 
     segment_pts seg_pts;
 
     GtkWidget *darea;
-    GdkGC *segment_gc;
-
-    GdkColor led_segment_on_color;
-    GdkColor led_segment_off_color;
-    GdkColor led_background_color;
 
     Led_7Segments(const char *);
     ~Led_7Segments();  
 
-
     void build_segments( int w, int h);
 
-    virtual void callback();
     virtual void build_window();
     virtual void update();
-    virtual void update( GtkWidget *drawable,   
-                         guint max_width,
-                         guint max_height);
     unsigned int getPinState();
 
     // Inheritances from the Package class
@@ -155,6 +116,9 @@ namespace Leds {
     const virtual char *type() { return ("led_7segments"); };
     static Module *construct(const char *new_name);
   private:
+    static gboolean led7_expose_event(GtkWidget *widget, GdkEvent *event,
+      gpointer user_data);
+
     //unsigned int m_segmentStates;
     Led_Input **m_pins;
     int m_nPins;
@@ -168,27 +132,21 @@ namespace Leds {
 
   class ActiveStateAttribute;
 
-  class Led: public Module, public Led_base, public TriggerObject
+  class Led: public Module, public Led_base
   {
   public:
 
     GtkWidget *darea;
-    GdkGC *gc;
+
     GdkColor led_on_color[BLUE+1];
     GdkColor led_segment_off_color;
     int w_width, w_height;
 
-    gpointer cbp;  // cycle break point pointer (need to delete in destructor)
-
     Led(const char *);
     ~Led();
 
-    virtual void callback();
     virtual void build_window();
     virtual void update();
-    virtual void update( GtkWidget *drawable,   
-                         guint max_width,
-                         guint max_height);
 
     // Inheritances from the Package class
     virtual void create_iopin_map();
@@ -200,9 +158,11 @@ namespace Leds {
     void set_on_color(Colors color);
     ActiveStates get_the_activestate() { return the_activestate; }
     void set_the_activestate(ActiveStates activestate);
-    
 
   private:
+    static gboolean led_expose_event(GtkWidget *widget, GdkEvent *event,
+      gpointer user_data);
+
     Led_Input *m_pin;
     Colors on_color;
     ColorAttribute *m_colorAttribute;
