@@ -28,23 +28,12 @@ License along with this library; if not, see
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
-#include <time.h>
-#include <stdio.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#include <errno.h>
-#include <stdlib.h>
-#include <string>
-#include <iostream>
-
 #include "../config.h"    // get the definition for HAVE_GUI
 #ifdef HAVE_GUI
 
 #include <gtk/gtk.h>
 
+#include "../src/stimuli.h"
 #include "../src/packages.h"
 #include "push_button.h"
 
@@ -135,10 +124,10 @@ void PushButton::create_widget(PushButton *pb)
   button = gtk_button_new_with_label ((char*)pb->name().c_str());
   gtk_container_set_border_width (GTK_CONTAINER (button), 5);
 
-  gtk_signal_connect (GTK_OBJECT (button), "pressed",			    
-		      GTK_SIGNAL_FUNC (press_cb), (gpointer)pb);
-  gtk_signal_connect (GTK_OBJECT (button), "released",			    
-		      GTK_SIGNAL_FUNC (released_cb), (gpointer)pb);
+  g_signal_connect (button, "pressed",			    
+		      G_CALLBACK(press_cb), (gpointer)pb);
+  g_signal_connect (button, "released",			    
+		      G_CALLBACK(released_cb), (gpointer)pb);
 	
 		
   gtk_widget_show(button);
