@@ -27,14 +27,12 @@ License along with this library; if not, see
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
-#include "../src/stimuli.h"
-#include "../src/ioports.h"
-#include "../src/symbol.h"
-#include "../src/trace.h"
 #include "../src/modules.h"
 
 #include <gtk/gtk.h>
 
+class IOPIN;
+
 class PushButton : public Module
 {
     void create_widget(PushButton *pb);
