@@ -73,6 +73,9 @@ License along with this library; if not, see
 
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
+
+#include <cstdio>
+#include <algorithm>
 #include <fstream>
 
 #include "../config.h"
@@ -514,7 +517,16 @@ File Stimulus\n\
     char fname[20] = "";
     m_filename->get(fname, 20);
     if (fname[0])
+    {
       m_fp = new ifstream(fname);
+      if (m_fp->fail())
+      {
+          printf( "Warning can't open Stimulus file %s\n", fname);
+          delete m_fp;
+          m_fp = NULL;
+          return;
+      }
+    }
 
     // Set the first breakpoint
     parseLine(true);
