@@ -30,21 +30,14 @@ License along with this library; if not, see
 /* IN_MODULE should be defined for modules */
 #define IN_MODULE
 
-#include <time.h>
-#include <stdio.h>
+#include <cstdio>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <assert.h>
 #include <typeinfo>
 
-#include <errno.h>
-#include <stdlib.h>
-#include <assert.h>
+#include <cstdlib>
+#include <cassert>
 #include <string>
 #include <iostream>
-#include <math.h>
 
 #include "../config.h"    // get the definition for HAVE_GUI
 
@@ -54,10 +47,7 @@ License along with this library; if not, see
 
 #include "../src/packages.h"
 #include "../src/stimuli.h"
-#include "../src/symbol.h"
 #include "../src/gpsim_interface.h"
-#include "../src/gpsim_time.h"
-
 
 #include "switch.h"
 
@@ -657,8 +647,8 @@ Two port switch\n\
 
     m_button = GTK_TOGGLE_BUTTON(gtk_toggle_button_new_with_label ((char*)sw->name().c_str()));
     gtk_container_set_border_width (GTK_CONTAINER (m_button), 1);
-    gtk_signal_connect (GTK_OBJECT (m_button), "toggled",
-                        GTK_SIGNAL_FUNC (toggle_cb), (gpointer)sw);
+    g_signal_connect (m_button, "toggled",
+                        G_CALLBACK(toggle_cb), (gpointer)sw);
     gtk_widget_show(GTK_WIDGET(m_button));
     gtk_box_pack_start (GTK_BOX (box1), GTK_WIDGET(m_button), FALSE, FALSE, 0);
 
