@@ -32,10 +32,18 @@ License along with this library; if not, see
 
 #include "ttl.h"
 #include "../src/stimuli.h"
-#include "../src/ioports.h"
-#include "../src/symbol.h"
-#include "../src/value.h"
 #include "../src/packages.h"
+#include "../src/gpsim_time.h"
+
+#include <string>
+
+//#define DEBUG
+#if defined(DEBUG)
+#include "../config.h"
+#define Dprintf(arg) {printf("%s:%d %s ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
 
 namespace TTL
 {
@@ -278,7 +286,7 @@ Module *TTL595::construct(const char *_new_name)
 
 
 TTL595::TTL595(const char *_name)
-  : TTLbase(_name, "TTL595 - Octal Shift Register"), sreg(0)
+  : TTLbase(_name, "TTL595 - Octal Shift Register"),  m_bStrobe(false), sreg(0)
 {
 
   m_Q = new IO_bi_directional *[8];
@@ -314,13 +322,15 @@ TTL595::TTL595(const char *_name)
 void TTL595::setClock(bool bNewClock)
 {
   // Clock shifts the shift register on rising edge, if MR pin is high
+  Dprintf(("bNewClock %d m_bClock %d reset %d Ds %d\n", bNewClock, m_bClock, m_reset->getDrivenState(), m_Ds->getDrivenState()));
   if (bNewClock && !m_bClock && m_reset->getDrivenState())
   {
     // Move the shift register left and out.
-    m_Qs->putState ( (sreg & 0x80)!=0 );
     sreg <<= 1;
     if ( m_Ds->getDrivenState() )
       sreg |= 0x01;
+    Dprintf(("%s sreg=%x now=0x%lx bNewClock %d m_bClock %d\n", name().c_str(), sreg, get_cycles().get(), bNewClock,m_bClock));
+    get_cycles().set_break(get_cycles().get() + 1, this);
   }
   m_bClock = bNewClock;
 }
@@ -340,6 +350,8 @@ void TTL595::setReset(bool bNewReset)
 
 void TTL595::setStrobe(bool bNewStrobe)
 {
+
+  Dprintf(("bNewStrobe %d m_bStrobe %d\n", bNewStrobe, m_bStrobe));
   // Strobe copies the contents of the shift register into the outputs
   if (bNewStrobe && !m_bStrobe) {
     update_state();
@@ -351,8 +363,18 @@ void TTL595::update_state()
 {
   for (int i=0, ss=sreg; i<8; i++,ss>>=1)
     m_Q[i]->putState((bool) (ss&1));
+  Dprintf(("sreg=0x%x\n", sreg));
 }
 
+void TTL595::callback_print()
+{
+    cout << "TTL595 " << name() << " CallBack ID 0x" << hex << CallBackID << '\n';
+}
+void TTL595::callback()
+{
+    Dprintf(("%s %s now=0x%lx\n", __FUNCTION__, name().c_str(), get_cycles().get()));
+    m_Qs->putState ( (sreg & 0x80)!=0 );
+}
 void TTL595::create_iopin_map()
 {
   package = new Package(16);
