@@ -39,14 +39,10 @@ License along with this library; if not, see
 #define IN_MODULE
 #define DEFAULT_BAUD    9600
 
-#include <errno.h>
-#include <stdlib.h>
 #include <string>
-#include <math.h>  // for floor()
 
 #include "../config.h"    // get the definition for HAVE_GUI
 #ifdef HAVE_GUI
-#define GTK_ENABLE_BROKEN
 #include <gtk/gtk.h>
 #endif
 
@@ -56,13 +52,7 @@ License along with this library; if not, see
 #include <cstdio>
 #include "../src/value.h"
 #include "../src/modules.h"
-#include "../src/packages.h"
 #include "../src/stimuli.h"
-#include "../src/ioports.h"
-#include "../src/symbol.h"
-#include "../src/interface.h"
-#include "../src/pir.h"
-#include "../src/trace.h"
 #include "../src/uart.h"
 
 
@@ -1077,7 +1067,7 @@ key_press(GtkWidget *widget,
 {
         unsigned int c = key->keyval;
 
-        gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+        g_signal_stop_emission_by_name (widget, "key_press_event");
         if(c == 0xffe3 || c == 0xffe4) // key is left or right ctrl
         {
             ctl = true;
@@ -1193,15 +1183,15 @@ void USARTModule::CreateGraphics()
 
     gtk_widget_add_events(window, GDK_KEY_RELEASE_MASK);
 
-    gtk_signal_connect(GTK_OBJECT(text),"key_press_event",
-                       (GtkSignalFunc) key_press,
+    g_signal_connect(text, "key_press_event",
+                       G_CALLBACK(key_press),
                        this);
-    gtk_signal_connect(GTK_OBJECT(text),"key_release_event",
-                       (GtkSignalFunc) key_release,
+    g_signal_connect(text, "key_release_event",
+                       G_CALLBACK(key_release),
                        this);
 
-    gtk_signal_connect (GTK_OBJECT (window), "destroy",
-                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window);
+    g_signal_connect (window, "destroy",
+                        G_CALLBACK(gtk_widget_destroy), window);
 
     gtk_widget_show_all(window);
 
