@@ -11,12 +11,14 @@ fi
 
 case "$1" in
     *clean)
-        DIRS='breakpoints instructions_12bit instructions_14bit instructions_16bit 
-node_test p16f84 p18f452_ports p16f628 digital_stim analog_stim p12ce518 eeprom_wide 
-interrupts_14bit macro_test logic_test resistor usart_test txisr_test 
-tmr0_16bit tmr1_16bit tmr3_16bit 
-switch_test p18f comparator a2d psp ttl ccp ccp_628 wavegen spi i2c port_stim 
-p12c509 wdt p12f675 p16f676 p16f690 p16f684 p1xf18xx'
+        DIRS='breakpoints instructions_12bit instructions_14bit 
+		instructions_16bit node_test p16f84 p18f452_ports 
+		p16f628 digital_stim analog_stim p12ce518 eeprom_wide epwm
+		interrupts_14bit macro_test logic_test resistor usart_test 
+	        txisr_test tmr0_16bit tmr1_16bit tmr3_16bit switch_test p18f 
+		comparator a2d psp ttl ccp ccp_628 wavegen spi i2c port_stim 
+		p12c509 wdt p12f675 p16f676 p16f690 p16f684 p1xf18xx
+		p18f26k22'
 
         echo ${DIRS}
         for i in ${DIRS} ; do
@@ -114,13 +116,7 @@ ${RT} p18f sim_extended_instructions
 
 ${RT} p18f sim_reset
 
-${RT} comparator sim_628
-
-${RT} comparator sim_877a
-
-${RT} comparator sim_882
-
-${RT} comparator sim_10f204
+${RT} comparator sim
 
 ${RT} a2d sim_p16c71
 
@@ -142,6 +138,8 @@ ${RT} a2d sim_p18f1220
 
 ${RT} a2d sim_p10f222
 
+${RT} a2d sim_p18f26k22
+
 ${RT} psp sim_p18f452
 
 ${RT} psp sim_p18f6520
@@ -152,6 +150,8 @@ ${RT} ttl sim_ttl377
 
 ${RT} ttl sim_ttl165
 
+${RT} ttl sim_ttl595
+
 ${RT} ccp sim_ccp_877a
 
 ${RT} ccp sim_ccp_819
@@ -160,6 +160,10 @@ ${RT} ccp sim_pwm_877a
 
 ${RT} ccp sim_pwm_6520
 
+${RT} ccp sim_pwm_26k22
+
+${RT} epwm sim
+
 ${RT} ccp_628 sim
 
 ${RT} wavegen sim
@@ -190,6 +194,9 @@ ${RT} wdt sim_wdt_10f200
 ${RT} wdt sim_wdt_16f88
 ${RT} wdt sim_nwdt_16f88
 
+${RT} wdt sim_nwdt_16f1823
+${RT} wdt sim_pwdt_16f1823
+
 ${RT} wdt sim_wdt_16f628
 ${RT} wdt sim_nwdt_16f628
 ${RT} wdt sim_nwdt_16f648a
@@ -225,3 +232,5 @@ ${RT}       p1xf18xx sim_p16f1823_comp
 ${RT}       p1xf18xx sim_p16f1823_i2c
 ${RT}       p1xf18xx sim_p16f1823_i2c_v2
 ${RT}       p1xf18xx sim_p16f1823_spi
+
+${RT}	p18f26k22 sim
