@@ -40,7 +40,7 @@ License along with this library; if not, see
 
 //#define DEBUG
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("0x%06X %s() ",cycles.get(),__FUNCTION__); printf arg; }
+#define Dprintf(arg) {printf("0x%06"PRINTF_GINT64_MODIFIER"X %s() ",cycles.get(),__FUNCTION__); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
@@ -165,7 +165,7 @@ void  BSR::put_value(unsigned int new_value)
 void  IOCxF::put(unsigned int new_value)
 {
   unsigned int masked_value = new_value & mValidBits;
-  Dprintf((" %s value %x add %x\n", name().c_str(), new_value, new_value&valid_bits));
+  Dprintf((" %s value %x masked %x\n", name().c_str(), new_value, masked_value));
 
   get_trace().raw(write_trace.get() | value.get());
   value.put(masked_value);
@@ -308,7 +308,7 @@ void  OSCCON::put(unsigned int new_value)
 // Clock is stable
 void OSCCON_2::callback()
 {
-    unsigned int mask = 0;
+    unsigned int add_bits = 0;
     unsigned int val;
 
     if (!oscstat) return;
@@ -317,25 +317,25 @@ void OSCCON_2::callback()
     switch(mode)
     {
     case LF:
-	mask = OSCSTAT::LFIOFR;
+	add_bits = OSCSTAT::LFIOFR;
 	break;
 
     case MF:
-	mask = OSCSTAT::MFIOFR;
+	add_bits = OSCSTAT::MFIOFR;
 	break;
 
     case HF:
-	mask = OSCSTAT::HFIOFL | OSCSTAT::HFIOFR;
+	add_bits = OSCSTAT::HFIOFL | OSCSTAT::HFIOFR  | OSCSTAT::HFIOFS;
 	break;
 
     case PLL:
-	mask = OSCSTAT::PLLR;
+	add_bits = OSCSTAT::PLLR;
 	break;
 
     case T1OSC:
 	break;
     }
-    val |= mask;
+    val |= add_bits;
     oscstat->value.put(val);
 }
 bool OSCCON_2::set_rc_frequency()
@@ -575,7 +575,8 @@ void WDTCON::put(unsigned int new_value)
 
   if (valid_bits > 1)
       cpu_pic->wdt.set_prescale(masked_value >> 1);
-  cpu_pic->wdt.swdten((masked_value & SWDTEN) == SWDTEN);
+  if (cpu_pic->swdten_active())
+      cpu_pic->wdt.swdten((masked_value & SWDTEN) == SWDTEN);
 }
 void WDTCON::reset(RESET_TYPE r)
 {
@@ -1013,7 +1014,7 @@ unsigned int Indirect_Addressing14::get()
     else if (fsr_adj >= 0x2000 && fsr_adj < 0x29b0) // Linear GPR region
     {
 	unsigned int bank = (fsr_adj & 0xfff) / 0x50;
-	unsigned int low_bits = ((fsr_adj & 0xff) % 0x50) + 0x20;
+	unsigned int low_bits = ((fsr_adj & 0xfff) % 0x50) + 0x20;
         return(cpu_pic->registers[bank * 0x80 + low_bits]->get()); 
     }
     else if (fsr_adj >= 0x8000 && fsr_adj <= 0xffff) // program memory
@@ -1194,7 +1195,7 @@ Stack::Stack(Processor *pCpu) : cpu(pCpu)
 {
 
   stack_warnings_flag = 0;   // Do not display over/under flow stack warnings
-  break_on_overflow = 0;     // Do not break if the stack over flows
+  break_on_overflow = true;     // Do not break if the stack over flows
   break_on_underflow = 0;    // Do not break if the stack under flows
   stack_mask = 7;            // Assume a 14 bit core.
   pointer = 0;
@@ -1214,28 +1215,29 @@ Stack::Stack(Processor *pCpu) : cpu(pCpu)
 
 bool Stack::push(unsigned int address)
 {
-  Dprintf(("pointer=%d address0x%x\n",pointer,address));
+  Dprintf(("pointer=%d address 0x%x\n",pointer,address));
   // Write the address at the current point location. Note that the '& stack_mask'
   // implicitly handles the stack wrap around.
 
   
-  contents[pointer & stack_mask] = address;
-
   // If the stack pointer is too big, then the stack has definitely over flowed.
   // However, some pic programs take advantage of this 'feature', so provide a means
   // for them to ignore the warnings.
 
-  if(pointer++ >= (int)stack_mask) {
+  if(pointer > (int)stack_mask) {
 	stack_overflow();
 	return false;
   }
+  contents[pointer++ & stack_mask] = address;
+
   return true;
 
 }
 bool Stack::stack_overflow()
 {
+    Dprintf(("stack_warnings_flag=%d break_on_overflow=%d\n", stack_warnings_flag,break_on_overflow));
     if(stack_warnings_flag || break_on_overflow)
-      cout << "stack overflow ";
+      cout << "stack overflow \n";
     if(break_on_overflow)
       bp.halt();
     return true;
@@ -1256,7 +1258,7 @@ unsigned int Stack::pop()
   }
 
 
-  Dprintf(("pointer=%d address0x%x\n",pointer,contents[pointer & stack_mask]));
+  Dprintf(("pointer=%d address 0x%x\n",pointer,contents[pointer & stack_mask]));
 
   return(contents[pointer & stack_mask]);
 }
@@ -1362,7 +1364,7 @@ void Stack14E::reset(RESET_TYPE r)
     else
 	contents[pointer-1] = contents[stack_mask];
   
-    Dprintf((" pointer %x\n", pointer));
+    Dprintf((" pointer 0x%x\n", pointer));
     stkptr.put(pointer-1);
 }
 
@@ -1756,6 +1758,7 @@ void CPSCON0::calculate_freq()
 	period = 1;
     }
     guint64 fc = get_cycles().get() + period;
+
     if (future_cycle > get_cycles().get())
     {
 	get_cycles().reassign_break(future_cycle, fc, this);
@@ -1766,6 +1769,10 @@ void CPSCON0::calculate_freq()
     future_cycle = fc;
 }
 
+void CPSCON0::callback_print()
+{
+    cout <<  name() << " has callback, ID = " << CallBackID << '\n';
+}
 void CPSCON0::callback()
 {
     Dprintf(("now=0x%"PRINTF_GINT64_MODIFIER"x\n",get_cycles().get()));
@@ -1851,7 +1858,7 @@ CPS_stimulus::CPS_stimulus(CPSCON0 * arg, const char *cPname,double _Vth, double
 void   CPS_stimulus::set_nodeVoltage(double v)
 {
 	Dprintf(("set_nodeVoltage =%.1f\n", v));
- 	nodeVoltage = v;;
+ 	nodeVoltage = v;
 	m_cpscon0->calculate_freq();
 }
 	
@@ -1942,19 +1949,43 @@ private:
   SR_MODULE *sr_module;
 };
 
+class SRnSource : public PeripheralSignalSource
+{
+
+public:
+
+  SRnSource(PinModule *_pin, SR_MODULE *_sr_module, int _index) :
+	PeripheralSignalSource(_pin), sr_module(_sr_module),
+	index(_index)
+  { ;}
+  virtual void release() { sr_module->releasePin(index); }
+private:
+  SR_MODULE *sr_module;
+  int       index;
+};
+
+enum {
+	SRQ = 0,
+	SRNQ
+};
 SR_MODULE::SR_MODULE(Processor *_cpu) :
     srcon0(_cpu, "srcon0", "SR Latch Control 0 Register", this),
     srcon1(_cpu, "srcon1", "SR Latch Control 1 Register", this),
     cpu(_cpu), future_cycle(0), state_set(false), state_reset(false),
     state_Q(false), srclk(0), syncc1out(false), syncc2out(false),
     SRI_pin(0), SRQ_pin(0), SRNQ_pin(0), m_SRinSink(0),
-    m_SRQsource(0), m_SRNQsource(0)
+    m_SRQsource(0), m_SRNQsource(0), m_SRQsource_active(false),
+    m_SRNQsource_active(false)
 {
 }
 SR_MODULE::~SR_MODULE()
 {
+    if (m_SRQsource_active)
+	SRQ_pin->setSource(0);
     if ( m_SRQsource)
 	delete  m_SRQsource;
+    if (m_SRNQsource_active)
+	SRNQ_pin->setSource(0);
     if ( m_SRNQsource)
 	delete  m_SRNQsource;
 }
@@ -2121,10 +2152,11 @@ void SR_MODULE::Qoutput()
 	(SRCON0::SRLEN | SRCON0::SRQEN))
     {
 	if (!m_SRQsource)
-	    m_SRQsource = new PeripheralSignalSource(SRQ_pin);
+	    m_SRQsource = new SRnSource(SRQ_pin, this, SRQ);
 
 	SRQ_pin->setSource(m_SRQsource);
 	SRQ_pin->getPin().newGUIname("SRQ");
+	m_SRQsource_active = true;
     }
     else
     {
@@ -2143,10 +2175,11 @@ void SR_MODULE::NQoutput()
 	(SRCON0::SRLEN | SRCON0::SRNQEN))
     {
 	if (!m_SRNQsource)
-	    m_SRNQsource = new PeripheralSignalSource(SRNQ_pin);
+	    m_SRNQsource = new SRnSource(SRNQ_pin, this, SRNQ);
 
 	SRNQ_pin->setSource(m_SRNQsource);
 	SRNQ_pin->getPin().newGUIname("SRNQ");
+	m_SRNQsource_active = true;
     }
     else
     {
@@ -2157,3 +2190,18 @@ void SR_MODULE::NQoutput()
 	}
     }
 }
+
+void SR_MODULE::releasePin(int index)
+{
+    switch(index)
+    {
+    case SRQ:
+	m_SRQsource_active = false;
+	break;
+
+    case SRNQ:
+	m_SRNQsource_active = false;
+	break;
+    }
+}
+
