@@ -36,6 +36,8 @@ class _14bit_processor;
 #include "breakpoints.h"
 
 #include "rcon.h"
+#include "intcon.h"
+
 class stimulus;  // forward reference
 class IOPIN;
 class source_stimulus;
@@ -650,6 +652,8 @@ class OSCTUNE : public  sfr_register
     TUN3 = 1<<3,
     TUN4 = 1<<4,
     TUN5 = 1<<5,
+    PLLEN= 1<<6,
+    INTSRC=1<<7 
   };
   OSCCON *osccon;
                                                                                 
@@ -696,7 +700,8 @@ class OSCCON : public  sfr_register,  public TriggerObject
     OSTS = 1<<3,
     IRCF0 = 1<<4,
     IRCF1 = 1<<5,
-    IRCF2 = 1<<6
+    IRCF2 = 1<<6,
+    IDLEN = 1<<7
   };
                                                                                 
   OSCCON(Processor *pCpu, const char *pName, const char *pDesc)
@@ -804,13 +809,13 @@ class IOC :  public sfr_register
 {
 public:
 
- IOC(Processor *pCpu, const char *pName, const char *pDesc)
+ IOC(Processor *pCpu, const char *pName, const char *pDesc, unsigned int _valid_bits = 0xff)
     : sfr_register(pCpu,pName,pDesc)
   {
-      mValidBits=0x3f;
+      mValidBits=_valid_bits;
   }
 
-  void put(unsigned int new_value)
+  virtual void put(unsigned int new_value)
   {
     unsigned int masked_value = new_value & mValidBits;
 
@@ -820,6 +825,24 @@ public:
 
 };
 
+// Interrupt-On-Change  Register
+class IOCxF : public IOC
+{
+public:
+
+  IOCxF(Processor *pCpu, const char *pName, const char *pDesc, unsigned int _valid_bits = 0xff)
+    : IOC(pCpu,pName,pDesc, _valid_bits), intcon(0)
+  {
+  }
+
+  void set_intcon(INTCON *new_value) { intcon = new_value; }
+  void put(unsigned int new_value);
+
+protected:
+    INTCON  *intcon;
+
+};
+
 class PicPortRegister;
 // WPU set weak pullups on pin by pin basis
 //
@@ -844,6 +867,7 @@ class CPSCON1;
 class T1CON_G;
 class CPS_stimulus;
 
+// Capacitance Sensing Control Register 0
 class CPSCON0  : public  sfr_register,  public TriggerObject
 {
 
@@ -865,6 +889,7 @@ public:
   void set_DAC_volt(double);
   void set_FVR_volt(double);
   void callback();
+  virtual void callback_print();
 
   CPSCON0(Processor *pCpu, const char *pName, const char *pDesc=0);
   ~CPSCON0();
@@ -882,6 +907,7 @@ private:
   CPS_stimulus  *cps_stimulus;
 
 };
+// Capacitance Sensing Control Register 1
 class CPSCON1  : public  sfr_register
 {
 
@@ -991,6 +1017,7 @@ public:
     void	setState(char);
     void	Qoutput();
     void	NQoutput();
+    void	releasePin(int);
 
 protected:
 
@@ -1011,6 +1038,8 @@ protected:
     SRinSink	*m_SRinSink;
     PeripheralSignalSource *m_SRQsource;
     PeripheralSignalSource *m_SRNQsource;
+    bool                    m_SRQsource_active;
+    bool                    m_SRNQsource_active;
 };
 
 #endif
