@@ -1,6 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2009,2010 Roy R. Rankin
+   Copyright (C) 2009,2010,2013 Roy R. Rankin
 
 This file is part of the libgpsim library of gpsim
 
@@ -27,6 +27,7 @@ License along with this library; if not, see
 #include "breakpoints.h"
 #include "ioports.h"
 #include "ssp.h"
+#include "interface.h"
 
 class TMRL;
 class TMRH;
@@ -37,6 +38,7 @@ class PWM1CON;
 class ADCON0;
 class PIR_SET;
 class InterruptSource;
+class TMR1_Interface;
 
 class _14bit_processor;
 
@@ -113,7 +115,7 @@ public:
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
-  void set_mask(unsigned int bm) { bit_mask = bm; }
+  void setBitMask(unsigned int bm) { mValidBits = bm; }
   void setIOpin(PinModule *p0, PinModule *p1, PinModule *p2);
   void c1_output(int value);
   void c2_output(int value);
@@ -124,7 +126,6 @@ public:
 private:
   PWM1CON 	*pwm1con;
   CCPCON  	*ccp1con;
-  unsigned int 	bit_mask;
   PinModule 	*m_PinModule;
   INT_SignalSink  *m_sink;
   bool		trig_state[3];
@@ -157,10 +158,9 @@ public:
   ~PWM1CON();
 
   void put(unsigned int new_value);
-  void set_mask(unsigned int bm) { bit_mask = bm; }
+  void setBitMask(unsigned int bm) { mValidBits = bm; }
 
 private:
-  unsigned int bit_mask;
 };
 //
 // Enhanced PWM Pulse Steering control register
@@ -231,7 +231,7 @@ public:
     PWM3 = 15
   };
 
-  void setBitMask(unsigned int bv) { bit_mask = bv; }
+  void setBitMask(unsigned int bv) { mValidBits = bv; }
   void new_edge(unsigned int level);
   void compare_match();
   void pwm_match(int new_state);
@@ -241,20 +241,26 @@ public:
   char getState();
   bool test_compare_mode();
   void callback();
+  void releasePins(int);
+  void releaseSink();
 
-  void setCrosslinks(CCPRL *, PIR *, TMR2 *, ECCPAS *_eccpas=0);
+  void setCrosslinks(CCPRL *, PIR *, unsigned int _mask, TMR2 *, ECCPAS *_eccpas=0);
   void setADCON(ADCON0 *);
   CCPCON(Processor *pCpu, const char *pName, const char *pDesc=0);
   ~CCPCON();
   void setIOpin(PinModule *p1, PinModule *p2=0, PinModule *p3=0, PinModule *p4=0);
+  void setIOPin1(PinModule *p1);
+  void setIOPin2(PinModule *p2);
+  void set_tmr2(TMR2 *_tmr2) { tmr2 = _tmr2;}
+
   PSTRCON *pstrcon;
   PWM1CON *pwm1con;
   ECCPAS  *eccpas;
 
 protected:
-  unsigned int 	bit_mask;
   PinModule 	*m_PinModule[4];
   CCPSignalSource *m_source[4];
+  bool		source_active[4];
   CCPSignalSink *m_sink;
   Tristate	*m_tristate;
   bool  	m_bInputEnabled;    // Input mode for capture/compare
@@ -270,7 +276,7 @@ protected:
   PIR     *pir;
   TMR2    *tmr2;
   ADCON0  *adcon0;
-  
+  unsigned int pir_mask;
 
 };
 
@@ -300,6 +306,7 @@ enum
   TMR1_Freq_Attribute *freq_attribute;
 
   T1CON(Processor *pCpu, const char *pName, const char *pDesc=0);
+  ~T1CON();
 
   unsigned int get();
 
@@ -312,27 +319,27 @@ enum
 
   virtual unsigned int get_prescale();
 
-  unsigned int get_tmr1cs()
+  virtual unsigned int get_tmr1cs()
     {
-      return(value.get() & TMR1CS);
+      return((value.get() & TMR1CS)?2:0);
     }
-  unsigned int get_tmr1on()
+  virtual bool get_tmr1on()
     {
       return(value.get() & TMR1ON);
     }
-  unsigned int get_t1oscen()
+  virtual bool  get_t1oscen()
     {
       return(value.get() & T1OSCEN);
     }
-  unsigned int get_tmr1GE()
+  virtual bool get_tmr1GE()
     {
       return(value.get() & TMR1GE);
     }
-  unsigned int get_t1GINV()
+  virtual bool get_t1GINV()
     {
       return(value.get() & T1GINV);
     }
-  unsigned int get_t1sync()
+  virtual bool get_t1sync()
     {
       return(value.get() & T1SYNC);
     }
@@ -341,6 +348,132 @@ enum
 };
 
 
+
+class T1GCon_GateSignalSink;
+class T1CON_G;
+
+// Timer 1 gate control Register
+class T1GCON : public sfr_register
+{
+public:
+
+  enum
+  {
+	T1GSS0	= 1<<0,		// Gate source select bits
+	T1GSS1	= 1<<1,
+	T1GVAL  = 1<<2,		// Current state bit
+	T1GGO	= 1<<3,		// Gate Single-Pulse Acquisition Status bit
+	T1GSPM	= 1<<4, 	// Gate Single-Pulse Mode bit
+	T1GTM	= 1<<5,		// Gate Toggle Mode bit
+	T1GPOL	= 1<<6,		// Gate Polarity bit
+	TMR1GE  = 1<<7,		// Gate Enable bit
+  };
+
+  
+  void put(unsigned int new_value);
+  virtual void setGatepin(PinModule *);
+  virtual void PIN_gate(bool);
+  virtual void T0_gate(bool);
+  virtual void T2_gate(bool);
+  virtual void CM1_gate(bool);
+  virtual void CM2_gate(bool);
+  virtual void new_gate(bool);
+  void set_WRmask(unsigned int mask) { write_mask = mask;}
+  void set_tmrl(TMRL *_tmrl) { tmrl = _tmrl;}
+  virtual void setInterruptSource(InterruptSource * _int) 
+	{ m_Interrupt = _int;}
+  virtual InterruptSource * getInterruptSource() { return m_Interrupt; }
+  virtual bool get_tmr1GE()
+    {
+      return(value.get() & TMR1GE);
+    }
+  bool get_t1GPOL() { return (value.get() & T1GPOL); }
+  virtual bool tmr1_isON();
+  
+
+  T1GCON(Processor *pCpu, const char *pName, const char *pDesc=0, T1CON_G *t1con_g=0);
+  ~T1GCON();
+
+private:
+
+  T1GCon_GateSignalSink *sink;
+  unsigned int 	write_mask;
+  TMRL		*tmrl;
+  T1CON_G	*t1con_g;
+  InterruptSource *m_Interrupt;
+  bool 		PIN_gate_state;
+  bool 		T0_gate_state; // can also be Tx = PRx where x=2,4,6
+  bool 		CM1_gate_state;
+  bool 		CM2_gate_state;
+  bool 		last_t1g_in;
+  PinModule	*gate_pin;
+};
+
+//
+// T1CON_G	combines T1CON and T1GCON into one virtual register
+//
+class T1CON_G : public T1CON
+{
+public:
+
+  enum
+  {
+    TMR1ON  = 1<<0,
+    T1SYNC  = 1<<2,
+    T1OSCEN = 1<<3,
+    T1CKPS0 = 1<<4,
+    T1CKPS1 = 1<<5,
+    TMR1CS0  = 1<<6,
+    TMR1CS1  = 1<<7,
+  };
+
+  TMRL  *tmrl;
+  TMR1_Freq_Attribute *freq_attribute;
+
+  T1CON_G(Processor *pCpu, const char *pName, const char *pDesc=0);
+  ~T1CON_G();
+
+  void t1_cap_increment();
+
+  // RRR unsigned int get();
+
+  // For (at least) the 18f family, there's a 4X PLL that effects the
+  // the relative timing between gpsim's cycle counter (which is equivalent
+  // to the cumulative instruction count) and the external oscillator. In
+  // all parts, the clock source for the timer is fosc, the external oscillator.
+  // However, for the 18f parts, the instructions execute 4 times faster when
+  // the PLL is selected.
+
+  virtual unsigned int get_prescale() 
+	{ return( ((value.get() &(T1CKPS0 | T1CKPS1)) >> 4) );}
+
+  virtual unsigned int get_tmr1cs()
+    {
+      return((value.get() & (TMR1CS1 | TMR1CS0))>>6);
+    }
+  virtual bool get_tmr1on()
+    {
+      return(value.get() & TMR1ON);
+    }
+  virtual bool get_t1oscen()
+    {
+      return(value.get() & T1OSCEN);
+    }
+  virtual bool get_t1sync()
+    {
+      return(value.get() & T1SYNC);
+    }
+  virtual void put(unsigned int new_value);
+  virtual bool get_tmr1GE() 
+  {
+    return t1gcon.get_tmr1GE();
+  }
+  virtual bool get_t1GINV() { return true;}
+
+  T1GCON t1gcon;
+
+};
+
 //---------------------------------------------------------
 // TMRL & TMRH - Timer 1
 class TMRH : public sfr_register
@@ -407,6 +540,7 @@ public:
   virtual void IO_gate(bool);
   virtual void compare_gate(bool);
   virtual void setInterruptSource(InterruptSource *);
+  virtual InterruptSource * getInterruptSource() { return m_Interrupt; }
   virtual void sleep();
   virtual void wake();
 
@@ -414,10 +548,10 @@ public:
   void clear_compare_event ( CCPCON *host );
 
   void set_T1GSS(bool arg);
-
-protected:
   virtual void increment();   // Used when TMR1 is attached to an external clock
+
 private:
+  TMR1_Interface *tmr1_interface;
   char m_cState;
   bool m_GateState;		// Only changes state if setGatepin() has been called
   bool m_compare_GateState;
@@ -509,21 +643,26 @@ enum
 
 };
 
-
+#define MAX_PWM_CHANS   5
 
 //---------------------------------------------------------
 // TMR2 - Timer
 class TMR2 : public sfr_register, public TriggerObject
 {
+protected:
+  CCPCON * ccp[MAX_PWM_CHANS];
+
 public:
   /* Define the way in which the tmr2 callback function may be updated. */
   enum TMR2_UPDATE_TYPES
   {
-    TMR2_PWM1_UPDATE = 1<<0,       // wrt ccp1
-    TMR2_PWM2_UPDATE = 1<<1,       // wrt ccp2
-    TMR2_PR2_UPDATE  = 1<<2,       // update pr2 match
-    TMR2_WRAP        = 1<<3,	   // wrap TMR2
-    TMR2_DONTCARE_UPDATE = 0xf     // whatever comes next
+    TMR2_WRAP        = 1<<0,	   // wrap TMR2
+    TMR2_PR2_UPDATE  = 1<<1,       // update pr2 match
+    TMR2_PWM1_UPDATE = 1<<2,       // wrt ccp1
+//    TMR2_PWM2_UPDATE = 1<<3,       // wrt ccp2
+    // PWM must be last as a variable number of channels follows
+    TMR2_ANY_PWM_UPDATE = 0xfc,    // up to six PWM channels
+    TMR2_DONTCARE_UPDATE = 0xff    // whatever comes next
   };
 
   int pwm_mode;
@@ -534,20 +673,16 @@ public:
     prescale,
     prescale_counter,
     break_value,
-    duty_cycle1,     /* for ccp1 */
-    duty_cycle2;     /* for ccp2 */
-  int
-    post_scale;
-  guint64
-    last_cycle,
-    future_cycle;
+    duty_cycle[MAX_PWM_CHANS];     /* for ccp channels */
+  int 		post_scale;
+  guint64	last_cycle;
+  guint64	future_cycle;
 
-  PR2  *pr2;
-  PIR_SET *pir_set;
-  T2CON *t2con;
-  CCPCON *ccp1con;
-  CCPCON *ccp2con;
-  SSP_MODULE *ssp_module;
+  PR2  		*pr2;
+  PIR_SET 	*pir_set;
+  T2CON 	*t2con;
+  SSP_MODULE	*ssp_module[2];
+  T1GCON	*m_txgcon;
 
   virtual void callback();
   virtual void callback_print();
@@ -564,7 +699,13 @@ public:
   void pwm_dc(unsigned int dc, unsigned int ccp_address);
   void stop_pwm(unsigned int ccp_address);
   virtual unsigned int get_value();
+  virtual void setInterruptSource(InterruptSource * _int) 
+	{ m_Interrupt = _int;}
+  virtual InterruptSource * getInterruptSource() { return m_Interrupt; }
 
+  bool add_ccp ( CCPCON * _ccp );
+  bool rm_ccp(CCPCON *_ccp);
+  InterruptSource *m_Interrupt;
 };
 
 
