@@ -209,6 +209,10 @@ void ADDFSR16::execute()
 {
   if (cpu16->extended_instruction())
   {
+      // Apply pending update.
+      ia->fsr_value += ia->fsr_delta;
+      ia->fsr_delta = 0;
+
       if (opcode & 0x100)
     	ia->put_fsr(ia->get_fsr_value() - m_lit);  //SUBFSR
       else
@@ -1807,7 +1811,8 @@ void RETFIE16::execute()
     cpu16->fast_stack.pop();
   //cout << "retfie: need to enable interrupts\n";
 
-  cpu16->intcon.set_gies();  // re-enable the appropriate interrupt
+  cpu16->intcon.in_interrupt = false;
+  cpu16->intcon.put_value(cpu16->intcon.value.get()); //test for new interrupts
 
 }
 
