@@ -566,6 +566,7 @@ void Program_Counter16::computed_goto(unsigned int new_address)
 
 
   value = ( (new_address | cpu_pic->get_pclath_branching_modpcl() )>>1);
+
   if (value >= memory_size)
 	value -= memory_size;
 
@@ -797,27 +798,29 @@ void T0CON::put(unsigned int new_value)
   //trace.register_write(address,value.get());
   value.put(new_value);
 
-  if( (value.get() ^ old_value) & (T08BIT | TMR0ON)) {
-    cpu16->option_new_bits_6_7(value.get() & (BIT6 | BIT7));
+  if (new_value == old_value)
+	return;
 
-    if(value.get() & TMR0ON) {
-      unsigned int initialTmr0value = (cpu16->tmr0l.value.get() & 0xff) |
-	( ((value.get() & T08BIT)==0) ? ((cpu16->tmr0l.value.get() & 0xff)<<8) : 0);
-      cpu16->tmr0l.start(initialTmr0value);
-    } else
-      cpu16->tmr0l.stop();
+  // new_prescale causes issues in 16 bit mode, so save current tmr0l and
+  // tmr0h values , call new_prescale (if required), and then restart timer
+  // using saved values
+  //
+  unsigned int initialTmr0value = (cpu16->tmr0l.value.get() & 0xff) |
+        (( (value.get() & T08BIT)) ? 0: ((cpu16->tmr0h.value.get() & 0xff)<<8));
 
-  } 
 
+  cpu16->option_new_bits_6_7(value.get() & (BIT6 | BIT7));
 
   // %%%FIX ME%%% - can changing the state of TOSE cause the timer to
   // increment if tmr0 is being clocked by an external clock?
-
-  // Now check the rest of the tmr0 bits.
+  // 
   if( (value.get() ^ old_value) & (T0CS | T0SE | PSA | PS2 | PS1 | PS0))
     cpu16->tmr0l.new_prescale();
 
-  //cout <<"T0CON::put - new val 0x" << hex << value.get() <<'\n';
+  if(value.get() & TMR0ON) {
+      cpu16->tmr0l.start(initialTmr0value);
+  } else
+      cpu16->tmr0l.stop();
 }
 
 //--------------------------------------------------
@@ -889,6 +892,13 @@ unsigned int TMR0_16::get_prescale()
 void TMR0_16::set_t0if()
 {
   intcon->set_t0if();
+  if (m_t1gcon)
+  {
+      m_t1gcon->T0_gate(true);
+      // Spec sheet does not indicate when the overflow signal
+      // is cleared, so I am assuming it is just a pulse. RRR
+      m_t1gcon->T0_gate(false);
+  }
 }
 
 bool TMR0_16::get_t0cs()
@@ -922,6 +932,7 @@ void TMR0_16::put_value(unsigned int new_value)
 {
   value.put(new_value & 0xff);
   value16 = (new_value & 0xff) | (tmr0h ? (tmr0h->get_value()<<8)  : 0);
+
   if(t0con->value.get() & T0CON::TMR0ON) {
     if(t0con->value.get() & T0CON::T08BIT)
       TMR0::put_value(new_value);
@@ -1136,11 +1147,11 @@ void TMR3_MODULE::initialize(T3CON *t3con_, PIR_SET *pir_set_)
 //-------------------------------------------------------------------
 
 TBL_MODULE::TBL_MODULE(_16bit_processor *pCpu)
-  : cpu(pCpu),
+  : EEPROM_EXTND(pCpu, 0), cpu(pCpu),
     tablat(pCpu,"tablat"),
-    tabptrl(pCpu,"tabptrl"),
-    tabptrh(pCpu,"tabptrh"),
-    tabptru(pCpu,"tabptru")
+    tblptrl(pCpu,"tblptrl"),
+    tblptrh(pCpu,"tblptrh"),
+    tblptru(pCpu,"tblptru")
 {
 }
 
@@ -1165,17 +1176,17 @@ TBL_MODULE::TBL_MODULE(_16bit_processor *pCpu)
 void TBL_MODULE::increment()
 {
 
-  if(tabptrl.value.get() >= 0xff) {
-    tabptrl.put(0);
-    if(tabptrh.value.get() >= 0xff) {
-      tabptrh.put(0);
-      tabptru.put(tabptru.value.get() + 1);
+  if(tblptrl.value.get() >= 0xff) {
+    tblptrl.put(0);
+    if(tblptrh.value.get() >= 0xff) {
+      tblptrh.put(0);
+      tblptru.put(tblptru.value.get() + 1);
     } else {
-      tabptrh.put(tabptrh.value.get() + 1);
+      tblptrh.put(tblptrh.value.get() + 1);
     }
   }
   else
-    tabptrl.put(tabptrl.value.get() + 1);
+    tblptrl.put(tblptrl.value.get() + 1);
 
 
 }
@@ -1184,33 +1195,36 @@ void TBL_MODULE::increment()
 void TBL_MODULE::decrement()
 {
 
-  if(tabptrl.value.get() == 0) {
-    tabptrl.put(0xff);
-    if(tabptrh.value.get() == 0) {
-      tabptrh.put(0xff);
-      tabptru.put(tabptru.value.get() - 1);
+  if(tblptrl.value.get() == 0) {
+    tblptrl.put(0xff);
+    if(tblptrh.value.get() == 0) {
+      tblptrh.put(0xff);
+      tblptru.put(tblptru.value.get() - 1);
     } else {
-      tabptrh.put(tabptrh.value.get() - 1);
+      tblptrh.put(tblptrh.value.get() - 1);
     }
   }
   else
-    tabptrl.put(tabptrl.value.get() - 1);
+    tblptrl.put(tblptrl.value.get() - 1);
 
 }
 //-------------------------------------------------------------------
 //-------------------------------------------------------------------
 void TBL_MODULE::read()
 {
-  unsigned int tabptr,opcode;
+  unsigned int tblptr,opcode;
 
-  tabptr = 
-    ( (tabptru.value.get() & 0xff) << 16 ) |
-    ( (tabptrh.value.get() & 0xff) << 8 )  |
-    ( (tabptrl.value.get() & 0xff) << 0 );
+  // tblptr is 12 bit address pointer
+  tblptr = 
+    ( (tblptru.value.get() & 0xff) << 16 ) |
+    ( (tblptrh.value.get() & 0xff) << 8 )  |
+    ( (tblptrl.value.get() & 0xff) << 0 );
 
-  opcode = cpu_pic->pma->get_rom(tabptr & 0xfffffe);
+  // read 16 bits of program memory from even address
+  opcode = cpu_pic->pma->get_rom(tblptr & 0xfffffe);
 
-  if(tabptr & 1)
+  // return high or low byte depending on lsb of address
+  if(tblptr & 1)
     {
       tablat.put((opcode >> 8) & 0xff);
       internal_latch = (internal_latch & 0x00ff) | (opcode & 0xff00);
@@ -1227,28 +1241,61 @@ void TBL_MODULE::read()
 void TBL_MODULE::write()
 {
 
-  unsigned int tabptr;
+  unsigned int tblptr;
+  unsigned int latch_index;
+  unsigned int *pt;
 
-  tabptr = 
-    ( (tabptru.value.get() & 0xff) << 16 ) |
-    ( (tabptrh.value.get() & 0xff) << 8 )  |
-    ( (tabptrl.value.get() & 0xff) << 0 );
+  tblptr = 
+    ( (tblptru.value.get() & 0xff) << 16 ) |
+    ( (tblptrh.value.get() & 0xff) << 8 )  |
+    ( (tblptrl.value.get() & 0xff) << 0 );
 
-  if(tabptr & 1)
+  latch_index = (tblptr >> 1) % num_write_latches;
+  pt = &write_latches[latch_index];
+
+  if(tblptr & 1)
     {
-      // Long write
-      internal_latch = (internal_latch & 0x00ff) | ((tablat.value.get()<<8) & 0xff00);
-      cpu_pic->pma->put_opcode_start(tabptr & 0xfffffe, internal_latch);
+      *pt = (*pt & 0x00ff) | ((tablat.value.get()<<8) & 0xff00);
     }
   else
     {
-      // Short Write
-      internal_latch = (internal_latch & 0xff00) | (tablat.value.get() & 0x00ff);
+      *pt = (*pt & 0xff00) | (tablat.value.get() & 0x00ff);
     }
+}
+
+void TBL_MODULE::start_write()
+{
+  eecon1.value.put( eecon1.value.get()  | eecon1.WRERR);
+
+  if (eecon1.value.get() & (EECON1::EEPGD|EECON1::CFGS))
+  {
+
+      int index;
+      wr_adr  =
+          ( (tblptru.value.get() & 0xff) << 16 ) |
+          ( (tblptrh.value.get() & 0xff) << 8 )  |
+          ( (tblptrl.value.get() & 0xff) << 0 );
+      wr_adr = cpu->map_pm_address2index(wr_adr);
+      index = wr_adr % num_write_latches;
+      wr_data = write_latches[index];
+
+      eecon2.start_write();
+
+      // stop execution fo 2 ms
+      get_cycles().set_break(get_cycles().get() + (guint64)(.002*get_cycles().instruction_cps()), this);
+      bp.set_pm_write();
+      cpu_pic->pm_write();
+  }
+  else
+  {
+      get_cycles().set_break(get_cycles().get() + EPROM_WRITE_TIME, this);
+      wr_adr = eeadr.value.get() + (eeadrh.value.get() << 8);
+      wr_data = eedata.value.get() + (eedatah.value.get() << 8);
+      eecon2.start_write();
+  }
 
 
 }
-
 //-------------------------------------------------------------------
 //-------------------------------------------------------------------
 LVDCON::LVDCON(Processor *pCpu, const char *pName, const char *pDesc)
@@ -1258,151 +1305,211 @@ LVDCON::LVDCON(Processor *pCpu, const char *pName, const char *pDesc)
 }
 
 
-#if 0
-//-------------------------------------------------------------------
-//
-// PORTC16
-//-------------------------------------------------------------------
-
-
-PORTC16::PORTC16()
+bool OSCCON_HS::set_rc_frequency()
 {
-  usart = 0;
-  ccp1con = 0;
-}
+  double base_frequency = 31.e3;
+  bool config_pplx4 = cpu_pic->get_pplx4_osc();
+  bool osccon_pplx4 = (osctune)?osctune->value.get() & OSCTUNE::PLLEN:0;
 
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-unsigned int PORTC16::get()
-{
-  unsigned int old_value;
+  if (!cpu_pic->get_int_osc())
+     return false;
 
-  old_value = value.get();
-
-  IOPORT::get();
-
-  int diff = old_value ^ value.get(); // The difference between old and new
-
-  // 
-  if( ccp1con && (diff & CCP1) ) {
-    ccp1con->new_edge(value.get() & CCP1);
-  }
-  // if this cpu has a usart and there's been a change detected on
-  // the RX pin, then we need to notify the usart
-  if( usart && (diff & RX))
-    usart->new_rx_edge(value.get() & RX);
-
-  return(value.get());
-}
-
-//-------------------------------------------------------------------
-//-------------------------------------------------------------------
-void PORTC16::setbit(unsigned int bit_number, bool new_value)
-{
-
-  unsigned int old_value = value.get();
-
-  //cout << "PORTC16::setbit() bit " << bit_number << " to " << new_value << '\n';
-
-  IOPORT::setbit( bit_number,  new_value);
-
-  int diff = old_value ^ value.get(); // The difference between old and new
-
-  if(ccp1con && ( diff & CCP1) ) {
-    ccp1con->new_edge(value.get() & CCP1);
-  }
-
-  if( usart && (diff & RX))
-    usart->new_rx_edge(value.get() & RX);
-
-}
-
-//-------------------------------------------------------------------
-//  PORTC16::put(unsigned int new_value)
-//
-//  inputs:  new_value - here's where the I/O port is written (e.g.
-//                       gpsim is executing a MOVWF IOPORT,F instruction.)
-//  returns: none
-//
-//  The I/O Port is updated with the new value. If there are any stimuli
-// attached to the I/O pins then they will be updated as well.
-// This is the same as PIC_IOPORT::put(), but check for the transmit
-// enable bit of the usart in addition to the TRISC register.
-//
-//-------------------------------------------------------------------
-
-void PORTC16::put(unsigned int new_value)
-{
-  if(new_value > 255)
-    cout << "PORTC16::put value >255\n";
-  // The I/O Ports have an internal latch that holds the state of the last
-  // write, even if the I/O pins are configured as inputs. If the tris port
-  // changes an I/O pin from an input to an output, then the contents of this
-  // internal latch will be placed onto the external I/O pin.
-
-  internal_latch = new_value;
-
-  // update only those bits that are really outputs
-  int mack_value = tris->value.get();
-  if (usart && usart->txsta.get() & _TXSTA::TXEN)
-      mack_value |= 1 << 6;
-
-  trace.raw(write_trace.get() | value.get());
-  //trace.register_write(address,value.get());
-  value.put((new_value & ~mack_value) | (value.get() & mack_value));
-
-  // Update the stimuli - if there are any
-  if(stimulus_mask)
-    update_stimuli();
-
-}
-
-//-------------------------------------------------------------------
-// PORTC16::update_pin_directions(unsigned int new_tris)
-//
-//  Whenever a new value is written to a tris register, then we need
-// to update the stimuli associated with the I/O pins. This is true
-// even if the I/O pin are not connected to external stimuli (like a
-// square wave). An example scenario would be like changing a port b
-// pin from an output that's driving low to an input. If there's no
-// stimulus attached to the port b I/O pin then the pull up (if enabled)
-// will pull the I/O pin high.
-// This is the same as PIC_IOPORT::update_pin_directions(), but check for the
-// transmit enable bit of the usart in addition to the TRISC register.
-//
-//-------------------------------------------------------------------
-void PORTC16::update_pin_directions(unsigned int new_tris)
-{
-
-  if(!tris)
-    return;
-
-  unsigned int diff = tris->value.get() ^ new_tris;
-
-  if (usart && usart->txsta.get() & _TXSTA::TXEN)
-      diff &= ~(1 << 6); // don't update TX pin if TXEN
-
-  if(diff)
+    unsigned int new_IRCF = (value.get() & ( IRCF0 | IRCF1 | IRCF2)) >> 4;
+    switch (new_IRCF)
     {
-      // Update the I/O port value to that of the internal latch
-      value.put((value.get() & ~diff) | (internal_latch & diff));
+    case 0:
+	base_frequency = 31.e3;
+	break;
+	
+    case 1:
+	base_frequency = 125e3;
+	break;
+	
+    case 2:
+	base_frequency = 250e3;
+	break;
+	
+    case 3:
+	base_frequency = 1e6;
+	break;
+	
+    case 4:
+	base_frequency = 2e6;
+	break;
+	
+    case 5:
+	if (osccon_pplx4 || config_pplx4)
+	{
+	    base_frequency = 16e6;
+	}
+	else
+	{
+	    base_frequency = 4e6;
+	}
+	break;
+	
+    case 6:
+	if (osccon_pplx4 || config_pplx4)
+	{
+	    base_frequency = 32e6;
+	}
+	else
+	{
+	    base_frequency = 8e6;
+	}
+	break;
+	
+    case 7:
+	if (osccon_pplx4 || config_pplx4)
+	{
+	    base_frequency = 64e6;
+	}
+	else
+	{
+	    base_frequency = 16e6;
+	}
+	break;
+   }
+   if (osctune)
+   {
+       int tune;
+       unsigned int osctune_value = osctune->value.get();
+       tune = osctune_value & (OSCTUNE::TUN5-1);
+       tune = (OSCTUNE::TUN5 & osctune_value) ? -tune : tune;
+       base_frequency *= 1. + 0.125 * tune / 31.;
+   }
+   cpu_pic->set_frequency(base_frequency);
+   if ((bool)verbose)
+   {
+	cout << "set_rc_frequency() : osccon=" << hex << value.get();
+	if (osctune)
+	    cout << " osctune=" << osctune->value.get();
+	cout << " new frequency=" << base_frequency << endl;
+   }
+   return true;
+}
+/*******************************************************************
+	HLVDCON - High/Low-Voltage Detect Module
+*/
 
-      // Go through and update the direction of the I/O pins
-      int i,m;
-      for(i = 0, m=1; i<num_iopins; i++, m <<= 1)
-	if(m & diff & valid_iopins)
-	  {
-	  pins[i]->update_direction(m & (~new_tris));
-	  //cout << __FUNCTION__ << " name " << pins[i]->name() << " pin number " << i << '\n';
-	  }
-      // Now, update the nodes to which the(se) pin(s) may be attached
+HLVD_stimulus::HLVD_stimulus(HLVDCON *_hlvd, const char *cPname):
+  stimulus(cPname, 2.5, 1e12), hlvd(_hlvd)
+{
+}
+HLVD_stimulus::~HLVD_stimulus() 
+{
+}
+void HLVD_stimulus::set_nodeVoltage(double v)
+{
+    nodeVoltage = v;
+    hlvd->check_hlvd();
+}
 
-      guint64 time = get_cycles().get();
-      for(i = 0, m=1; i<num_iopins; i++, m <<= 1)
-	if(stimulus_mask & m & diff)
-          if(pins[i]->snode!=0)
-            pins[i]->snode->update(time);
+HLVDCON::HLVDCON(Processor *pCpu, const char *pName, const char *pDesc) :
+	sfr_register(pCpu, pName, pDesc), hlvdin(0), hlvdin_stimulus(0),
+	stimulus_active(false),write_mask(0x9f), IntSrc(0)
+  {}
+HLVDCON::~HLVDCON()
+{
+    if (IntSrc)
+	delete IntSrc;
+    if (stimulus_active)
+    {
+        hlvdin->getPin().snode->detach_stimulus(hlvdin_stimulus);
+        stimulus_active = false;
+     }
+     if (hlvdin_stimulus)
+	delete hlvdin_stimulus;
+}
+
+void HLVDCON::put(unsigned int new_value)
+{
+    double tivrst = 20e-6;	// typical time for IVR stable
+    unsigned int diff = value.get() ^ new_value;
+    trace.raw(write_trace.get() | value.get());
+
+    if (!diff) return;
+
+    if (diff & HLVDEN) 
+    {
+        if (new_value & HLVDEN)	// Turning on
+	{
+	    // wait tivrst before doing anything
+	    value.put(new_value & write_mask);
+	    get_cycles().set_break(
+		get_cycles().get() + tivrst * get_cycles().instruction_cps(),
+		this);
+	    return;
+	}
+	else	// Turning off
+	{
+	    value.put(new_value & write_mask);
+	    if (stimulus_active)
+	    {
+	        hlvdin->getPin().snode->detach_stimulus(hlvdin_stimulus);
+	        stimulus_active = false;
+            }
+	    return;
+        }
+    }
+    value.put((new_value & write_mask) | (value.get() & ~write_mask));
+    if (!(value.get() & IRVST))		// Just return if voltage not stable
+        return;
+    check_hlvd();
+
+}
+void HLVDCON::callback()
+{
+	unsigned int reg = value.get();
+
+	reg |= (BGVST | IRVST);
+	value.put(reg);
+	check_hlvd();
+}
+
+double hldv_volts[] = { 1.84, 2.07, 2.28, 2.44, 2.54, 2.74, 2.87, 3.01,
+			3.30, 3.48, 3.69, 3.91, 4.15, 4.41, 4.74};
+void HLVDCON::check_hlvd()
+{
+    unsigned int reg = value.get();
+
+    assert(IntSrc);
+    assert(hlvdin);
+    if (!(reg & IRVST))
+	return;
+    if ((reg & HLVDL_MASK) == HLVDL_MASK)	// using HLVDIN pin
+    {
+	if (!hlvdin_stimulus)
+	    hlvdin_stimulus = new HLVD_stimulus(this, "hlvd_stim");
+
+        if (!stimulus_active && hlvdin->getPin().snode)
+	{
+	    hlvdin->getPin().snode->attach_stimulus(hlvdin_stimulus);
+	    stimulus_active = true;
+	    hlvdin->getPin().snode->update();
+	}
+	double voltage = hlvdin->getPin().get_nodeVoltage();
+	// High voltage trip ?
+	if ((reg & VDIRMAG) && (voltage >= 1.024))
+	{
+	       IntSrc->Trigger();
+	}
+
+	// Low voltage trip ?
+	else if (!(reg & VDIRMAG) && (voltage <= 1.024))
+	       IntSrc->Trigger();
+    }
+    else	// Voltage divider on Vdd
+    {
+	double voltage = hldv_volts[reg & HLVDL_MASK];
+	Processor *Cpu = (Processor *)cpu;
+	if ((reg & VDIRMAG) && (Cpu->get_Vdd() >= voltage))
+	       IntSrc->Trigger();
+	else if (!(reg & VDIRMAG) && (Cpu->get_Vdd() <= voltage))
+	       IntSrc->Trigger();
     }
 }
 
-#endif // 0
+
+
+    
