@@ -1058,8 +1058,8 @@ void P16F877::create()
   delete_file_registers(0xa0, 0xff); //		""
   add_file_registers(0x20, 0x7f, 0);
   add_file_registers(0xa0, 0xef, 0);
-  add_file_registers(0x110, 0x11f, 0);
-  add_file_registers(0x190, 0x19f, 0);
+  add_file_registers(0x110, 0x16f, 0);
+  add_file_registers(0x190, 0x1ef, 0);
   alias_file_registers(0x70,0x7f,0x80);
   alias_file_registers(0x70,0x7f,0x100);
   alias_file_registers(0x70,0x7f,0x180);
@@ -1104,8 +1104,8 @@ P16F877::P16F877(const char *_name, const char *desc)
 
 P16F877::~P16F877()
 {
-  delete_file_registers(0x110, 0x11f);
-  delete_file_registers(0x190, 0x19f);
+  delete_file_registers(0x110, 0x16f);
+  delete_file_registers(0x190, 0x1ef);
 }
 
 void P16F877A::create_sfr_map()
