@@ -2,22 +2,22 @@
    Copyright (C) 1998-2002 T. Scott Dattalo
    Copyright (C) 2006	   Roy Rankin
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #ifndef __P16F8X_H__
 #define __P16F8X_H__
@@ -84,7 +84,6 @@ public:
   }
   virtual EEPROM_WIDE *get_eeprom() { return ((EEPROM_WIDE *)eeprom); }
 
-  virtual bool hasSSP() { return true;}
 
   virtual PIR *get_pir1() { return (pir1); }
   virtual PIR *get_pir2() { return (pir2); }
@@ -122,6 +121,7 @@ public:
 
   // The f628 (at least) I/O pins depend on the Fosc Configuration bits.
   virtual bool set_config_word(unsigned int address, unsigned int cfg_word);
+  virtual void create_config_memory();
 
 
   virtual void create(int eesize);
@@ -137,7 +137,6 @@ public:
   }
   virtual EEPROM_WIDE *get_eeprom() { return ((EEPROM_WIDE *)eeprom); }
 
-  virtual bool hasSSP() { return true;}
 
   virtual PIR *get_pir1() { return (pir1); }
   virtual PIR *get_pir2() { return (pir2); }
@@ -177,6 +176,7 @@ public:
 
 
   P16F88(const char *_name=0, const char *desc=0);
+  ~P16F88();
   static Processor *construct(const char *name);
 };
 
@@ -212,6 +212,7 @@ public:
 
 
   P16F819(const char *_name=0, const char *desc=0);
+  ~P16F819();
   static Processor *construct(const char *name);
 };
 
