@@ -325,7 +325,7 @@ P12F1822::P12F1822(const char *_name, const char *desc)
   m_daccon1 = new DACCON1(this, "daccon1", "DAC Voltage reference register 1", 0x1f, m_daccon0);
   m_cpu_temp = new CPU_Temp("cpu_temperature", 30., "CPU die temperature");
 
-  tmr0.set_cpu(this, m_porta, 4, option_reg);
+  tmr0.set_cpu(this, m_porta, 4, &option_reg);
   tmr0.start(0);
   tmr0.set_t1gcon(&t1con_g.t1gcon);
   cpscon1.m_cpscon0 = &cpscon0;
@@ -422,6 +422,7 @@ P12F1822::~P12F1822()
     remove_sfr_register(&ccp1as);
     remove_sfr_register(&pstr1con);
     remove_sfr_register(&osctune);
+    remove_sfr_register(&option_reg);
     remove_sfr_register(&osccon);
     remove_sfr_register(&oscstat);
 
@@ -477,14 +478,14 @@ void P12F1822::create_sfr_map()
   add_sfr_register(&tmr2,   0x1a, RegisterValue(0,0));
   add_sfr_register(&pr2,    0x1b, RegisterValue(0,0));
   add_sfr_register(&t2con,  0x1c, RegisterValue(0,0));
-  add_sfr_register(&cpscon0,  0x1e, RegisterValue(0,0));
+  add_sfr_register(&cpscon0,  0x1e, RegisterValue(0,0), "cpscon0");
   add_sfr_register(&cpscon1,  0x1f, RegisterValue(0,0));
 
 
   add_sfr_register(m_trisa, 0x8c, RegisterValue(0x3f,0));
 
   pcon.valid_bits = 0xcf;
-  add_sfr_register(option_reg,  0x95, RegisterValue(0xff,0));
+  add_sfr_register(&option_reg, 0x95, RegisterValue(0xff,0));
   add_sfr_register(&osctune,    0x98, RegisterValue(0,0));
   add_sfr_register(&osccon,     0x99, RegisterValue(0x38,0));
   add_sfr_register(&oscstat,    0x9a, RegisterValue(0,0));
@@ -499,7 +500,9 @@ void P12F1822::create_sfr_map()
   tmr1h.tmrl  = &tmr1l;
   t1con_g.tmrl  = &tmr1l;
   t1con_g.t1gcon.set_tmrl(&tmr1l);
-  t1con_g.t1gcon.set_pir_set(get_pir_set());
+  t1con_g.t1gcon.setInterruptSource(new InterruptSource(pir1, PIR1v1822::TMR1IF));
+
+  
 
   tmr1l.setIOpin(&(*m_porta)[5]);
   t1con_g.t1gcon.setGatepin(&(*m_porta)[3]);
@@ -566,7 +569,7 @@ void P12F1822::create_sfr_map()
   add_sfr_register(m_iocaf, 0x393, RegisterValue(0,0),"iocaf");
   m_iocaf->set_intcon(intcon);
 
-  tmr2.ssp_module = &ssp;
+  tmr2.ssp_module[0] = &ssp;
 
     ssp.initialize(
 	get_pir_set(),    // PIR
@@ -641,6 +644,7 @@ void P12F1822::create_sfr_map()
     comparator.cmxcon1[0]->set_INpinNeg(&(*m_porta)[1], &(*m_porta)[4]);
     comparator.cmxcon1[0]->set_INpinPos(&(*m_porta)[0]);
     comparator.cmxcon0[0]->setBitMask(0xf7);
+    comparator.cmxcon0[0]->setIntSrc(new InterruptSource(pir2, (1<<5)));
     comparator.cmxcon1[0]->setBitMask(0xf1);
     comparator.assign_pir_set(get_pir_set());
     comparator.assign_t1gcon(&t1con_g.t1gcon);
@@ -729,9 +733,8 @@ void  P12F1822::create(int ram_top, int eeprom_size)
 //-------------------------------------------------------------------
 void P12F1822::enter_sleep()
 {
-    tmr0.sleep();
     tmr1l.sleep();
-    pic_processor::enter_sleep();
+    _14bit_e_processor::enter_sleep();
 }
 
 //-------------------------------------------------------------------
@@ -739,13 +742,11 @@ void P12F1822::exit_sleep()
 {
     if (m_ActivityState == ePASleeping)
     {
-	tmr0.wake();
 	tmr1l.wake();
         osccon.wake();
-	pic_processor::exit_sleep();
+	_14bit_e_processor::exit_sleep();
     }
 }
-
 //-------------------------------------------------------------------
 void P12F1822::option_new_bits_6_7(unsigned int bits)
 {
@@ -931,7 +932,7 @@ P16F178x::P16F178x(const char *_name, const char *desc)
   m_cpu_temp = new CPU_Temp("cpu_temperature", 30., "CPU die temperature");
 
 
-  tmr0.set_cpu(this, m_porta, 4, option_reg);
+  tmr0.set_cpu(this, m_porta, 4, &option_reg);
   tmr0.start(0);
   tmr0.set_t1gcon(&t1con_g.t1gcon);
   set_mclr_pin(1);
@@ -1070,6 +1071,7 @@ P16F178x::~P16F178x()
     remove_sfr_register(&ccp1as);
     remove_sfr_register(&pstr1con);
     remove_sfr_register(&osctune);
+    remove_sfr_register(&option_reg);
     remove_sfr_register(&osccon);
     remove_sfr_register(&oscstat);
 
@@ -1141,7 +1143,7 @@ void P16F178x::create_sfr_map()
     add_sfr_register(m_trise, 0x90, RegisterValue(0x08,0));
 
   pcon.valid_bits = 0xcf;
-  add_sfr_register(option_reg,  0x95, RegisterValue(0xff,0));
+  add_sfr_register(&option_reg, 0x95, RegisterValue(0xff,0));
   add_sfr_register(&osctune,    0x98, RegisterValue(0,0));
   add_sfr_register(&osccon,     0x99, RegisterValue(0x38,0));
   add_sfr_register(&oscstat,    0x9a, RegisterValue(0,0));
@@ -1156,7 +1158,7 @@ void P16F178x::create_sfr_map()
   tmr1h.tmrl  = &tmr1l;
   t1con_g.tmrl  = &tmr1l;
   t1con_g.t1gcon.set_tmrl(&tmr1l);
-  t1con_g.t1gcon.set_pir_set(get_pir_set());
+  t1con_g.t1gcon.setInterruptSource(new InterruptSource(pir1, PIR1v1822::TMR1IF));
 
   tmr1l.setIOpin(&(*m_porta)[5]);
   t1con_g.t1gcon.setGatepin(&(*m_porta)[3]);
@@ -1255,7 +1257,7 @@ void P16F178x::create_sfr_map()
     add_sfr_register(m_dac4con0, 0x595, RegisterValue(0x00,0));
     add_sfr_register(m_dac4con1, 0x596, RegisterValue(0x00,0));
 
-  tmr2.ssp_module = &ssp;
+  tmr2.ssp_module[0] = &ssp;
 
     ssp.initialize(
 	get_pir_set(),    // PIR
@@ -1347,8 +1349,11 @@ void P16F178x::create_sfr_map()
     comparator.cmxcon1[1]->set_OUTpin(&(*m_porta)[5]);
     comparator.cmxcon1[2]->set_OUTpin(&(*m_portb)[5]);
     comparator.cmxcon0[0]->setBitMask(0xbf);
+    comparator.cmxcon0[0]->setIntSrc(new InterruptSource(pir2, (1<<5)));
     comparator.cmxcon0[1]->setBitMask(0xbf);
+    comparator.cmxcon0[1]->setIntSrc(new InterruptSource(pir2, (1<<6)));
     comparator.cmxcon0[2]->setBitMask(0xbf);
+    comparator.cmxcon0[2]->setIntSrc(new InterruptSource(pir2, (1<<1)));
     comparator.cmxcon1[0]->setBitMask(0xff);
     comparator.cmxcon1[1]->setBitMask(0xff);
     comparator.cmxcon1[2]->setBitMask(0xff);
@@ -1419,9 +1424,8 @@ void  P16F178x::create(int ram_top, int eeprom_size)
 //-------------------------------------------------------------------
 void P16F178x::enter_sleep()
 {
-    tmr0.sleep();
     tmr1l.sleep();
-    pic_processor::enter_sleep();
+    _14bit_e_processor::enter_sleep();
 }
 
 //-------------------------------------------------------------------
@@ -1429,10 +1433,9 @@ void P16F178x::exit_sleep()
 {
     if (m_ActivityState == ePASleeping)
     {
-	tmr0.wake();
 	tmr1l.wake();
         osccon.wake();
-	pic_processor::exit_sleep();
+	_14bit_e_processor::exit_sleep();
     }
 }
 
@@ -1691,6 +1694,7 @@ void P16F1788::create_sfr_map()
     comparator.cmxcon1[3]->set_INpinPos(&(*m_porta)[2], &(*m_portb)[6]);
     comparator.cmxcon1[3]->set_OUTpin(&(*m_portc)[7]);
     comparator.cmxcon0[3]->setBitMask(0xbf);
+    comparator.cmxcon0[3]->setIntSrc(new InterruptSource(pir2, (1<<2)));
     comparator.cmxcon1[3]->setBitMask(0xff);
 
 }
@@ -1825,13 +1829,17 @@ void P16F1823::create_sfr_map()
     apfcon.set_pins(5, &(*m_portc)[3], &(*m_porta)[3]); //SSP SS
     apfcon.set_pins(6, &(*m_portc)[2], &(*m_porta)[4]); //SSP SDO
     apfcon.set_pins(7, &(*m_portc)[5], &(*m_porta)[1]); //USART RX Pin
-    comparator.cmxcon1[0]->set_INpinNeg(&(*m_porta)[1], &(*m_portc)[1],  &(*m_portc)[2],  &(*m_portc)[3]);
-    comparator.cmxcon1[1]->set_INpinNeg(&(*m_porta)[1], &(*m_portc)[1],  &(*m_portc)[2],  &(*m_portc)[3]);
+    comparator.cmxcon1[0]->set_INpinNeg(&(*m_porta)[1], &(*m_portc)[1],
+					  &(*m_portc)[2],  &(*m_portc)[3]);
+    comparator.cmxcon1[1]->set_INpinNeg(&(*m_porta)[1], &(*m_portc)[1],
+					  &(*m_portc)[2],  &(*m_portc)[3]);
     comparator.cmxcon1[1]->set_INpinPos(&(*m_portc)[0]);
     comparator.cmxcon1[0]->set_OUTpin(&(*m_porta)[2]);
     comparator.cmxcon1[1]->set_OUTpin(&(*m_portc)[4]);
     comparator.cmxcon0[0]->setBitMask(0xf7);
+    comparator.cmxcon0[0]->setIntSrc(new InterruptSource(pir2, (1<<5)));
     comparator.cmxcon0[1]->setBitMask(0xf7);
+    comparator.cmxcon0[1]->setIntSrc(new InterruptSource(pir2, (1<<6)));
     comparator.cmxcon1[0]->setBitMask(0xf3);
     comparator.cmxcon1[1]->setBitMask(0xf3);
 
