@@ -364,6 +364,7 @@ void Bit_op::decode(Processor *new_cpu, unsigned int new_opcode)
       break;
 
     case _14BIT_PROCESSOR_:
+    case _14BIT_E_PROCESSOR_:
       mask = 1 << ((opcode >> 7) & 7);
       register_address = opcode & REG_MASK_14BIT;
       access = 1;
@@ -405,7 +406,10 @@ char * Bit_op::name(char *return_str,int len)
       return(return_str);
       break;
 
+    case _14BIT_E_PROCESSOR_:
     case _14BIT_PROCESSOR_:
+	if(access)
+  	    reg = get_cpu()->register_bank[register_address];
       bit = ((opcode >> 7) & 7);
       break;
 
@@ -448,11 +452,23 @@ char * Register_op::name(char *return_str,int len)
 
   source = get_cpu()->registers[register_address];
 
-  if ( cpu_pic->base_isa() != _PIC18_PROCESSOR_ )
+  if ( cpu_pic->base_isa() == _14BIT_E_PROCESSOR_ ||
+ 	cpu_pic->base_isa() == _14BIT_PROCESSOR_ )
+  {
+    if (access)
+	source = cpu_pic->register_bank[register_address];
     snprintf(return_str,len,"%s\t%s,%c",
              gpsimObject::name().c_str(),
              source->name().c_str(),
              destination ? 'f' : 'w');
+  }
+  else if ( cpu_pic->base_isa() != _PIC18_PROCESSOR_ )
+  {
+    snprintf(return_str,len,"%s\t%s,%c",
+             gpsimObject::name().c_str(),
+             source->name().c_str(),
+             destination ? 'f' : 'w');
+  }
   else
     snprintf(return_str,len,"%s\t%s,%c,%c",
              gpsimObject::name().c_str(),
@@ -491,6 +507,7 @@ void  Register_op::decode(Processor *new_cpu, unsigned int new_opcode)
       break;
 
     case _14BIT_PROCESSOR_:
+    case _14BIT_E_PROCESSOR_:
       register_address = opcode & REG_MASK_14BIT;
       destination = (opcode & DESTINATION_MASK_14BIT) ? true : false;
       access = 1;
@@ -532,14 +549,14 @@ void ADDWF::execute()
   else
     source = cpu_pic->register_bank[register_address];
 
-  new_value = (src_value = source->get()) + (w_value = cpu_pic->W->value.get());
+  new_value = (src_value = source->get()) + (w_value = cpu_pic->Wget());
 
   // Store the result
 
   if(destination)
     source->put(new_value & 0xff);      // Result goes to source
   else
-    cpu_pic->W->put(new_value & 0xff);
+    cpu_pic->Wput(new_value & 0xff);
 
   cpu_pic->status->put_Z_C_DC(new_value, src_value, w_value);
 
@@ -547,6 +564,41 @@ void ADDWF::execute()
 
 }
 
+
+//--------------------------------------------------
+
+ADDWFC::ADDWFC (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("addwfc");
+}
+
+void ADDWFC::execute()
+{
+  unsigned int new_value,src_value,w_value;
+
+  source = ((!access) ?
+	    cpu_pic->registers[register_address] 
+	    :
+	    cpu_pic->register_bank[register_address] );
+
+  new_value = (src_value = source->get()) + 
+    (w_value = cpu_pic->Wget()) +
+    ((cpu_pic->status->value.get() & STATUS_C) ? 1 : 0);
+
+  // Store the result
+
+  if(destination)
+    source->put(new_value & 0xff);      // Result goes to source
+  else
+    cpu_pic->Wput(new_value & 0xff);
+
+  cpu_pic->status->put_Z_C_DC(new_value, src_value, w_value);
+
+  cpu_pic->pc->increment();
+
+}
 //--------------------------------------------------
 
 ANDLW::ANDLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
@@ -560,9 +612,9 @@ void ANDLW::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu_pic->W->value.get() & L;
+  new_value = cpu_pic->Wget() & L;
 
-  cpu_pic->W->put(new_value);
+  cpu_pic->Wput(new_value);
   cpu_pic->status->put_Z(0==new_value);
 
   cpu_pic->pc->increment();
@@ -589,12 +641,12 @@ void ANDWF::execute()
   else
     source = cpu_pic->register_bank[register_address];
 
-  new_value = source->get() & cpu_pic->W->value.get();
+  new_value = source->get() & cpu_pic->Wget();
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   cpu_pic->status->put_Z(0==new_value);
 
@@ -605,6 +657,41 @@ void ANDWF::execute()
 
 //--------------------------------------------------
 
+ASRF::ASRF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("asrf");
+}
+
+void ASRF::execute()
+{
+  unsigned int new_value,src_value, carry, msb;
+
+  source = ((!access) ?
+	    cpu_pic->registers[register_address] 
+	    :
+	    cpu_pic->register_bank[register_address] );
+
+  carry = (src_value = source->get()) & 1;
+  msb = src_value & 0x80;
+  new_value = ((src_value & 0xff) >> 1) | msb;
+  
+  // Store the result
+
+  if(destination)
+    source->put(new_value);      // Result goes to source
+  else
+    cpu_pic->Wput(new_value);
+
+  cpu_pic->status->put_Z(new_value==0);
+  cpu_pic->status->put_C(carry);
+
+  cpu_pic->pc->increment();
+
+}
+//--------------------------------------------------
+
 BCF::BCF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : Bit_op(new_cpu, new_opcode, address)
 {
@@ -627,6 +714,70 @@ void BCF::execute()
 
 }
 
+//--------------------------------------------------
+BRA::BRA (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
+{
+  destination_index = (new_opcode & 0x1ff)+1;
+  absolute_destination_index = (address + destination_index) & 0xfffff;
+
+  if(new_opcode & 0x100)
+    {
+      absolute_destination_index -= 0x200;
+      destination_index = 0x200 - destination_index;
+    }
+
+  new_name("bra");
+}
+
+void BRA::execute()
+{
+  cpu_pic->pc->jump(absolute_destination_index);
+
+}
+
+char * BRA::name(char *return_str,int len)
+{
+
+
+  sprintf(return_str,"%s\t$%c0x%x\t;(0x%05x)",
+	  gpsimObject::name().c_str(),
+	  (opcode & 0x100) ? '-' : '+', 
+	  (destination_index & 0x1ff)<<1,
+	  absolute_destination_index<<1);
+
+  return(return_str);
+}
+
+//--------------------------------------------------
+BRW::BRW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
+{
+  destination_index = cpu_pic->Wget();
+  absolute_destination_index = address + destination_index;
+
+  new_name("brw");
+}
+
+void BRW::execute()
+{
+  cpu_pic->pc->jump(absolute_destination_index);
+
+}
+
+char * BRW::name(char *return_str,int len)
+{
+
+
+  sprintf(return_str,"%s\t$%c0x%x\t;(0x%05x)",
+	  gpsimObject::name().c_str(),
+	  (opcode & 0x100) ? '-' : '+', 
+	  (destination_index & 0x1ff)<<1,
+	  absolute_destination_index<<1);
+
+  return(return_str);
+}
+
 //--------------------------------------------------
 
 BSF::BSF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
@@ -711,6 +862,7 @@ CALL::CALL (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   switch(cpu_pic->base_isa())
     {
     case _14BIT_PROCESSOR_:
+    case _14BIT_E_PROCESSOR_:
       destination = opcode&0x7ff;
       break;
 
@@ -727,9 +879,9 @@ CALL::CALL (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 void CALL::execute()
 {
 
-  cpu_pic->stack->push(cpu_pic->pc->get_next());
-
-  cpu_pic->pc->jump(cpu_pic->get_pclath_branching_jump() | destination);
+  // do not jump if the push fails
+  if (cpu_pic->stack->push(cpu_pic->pc->get_next()))
+        cpu_pic->pc->jump(cpu_pic->get_pclath_branching_jump() | destination);
 
 }
 
@@ -744,6 +896,28 @@ char * CALL::name(char *return_str,int len)
 }
 
 
+//--------------------------------------------------
+CALLW::CALLW(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  :instruction (new_cpu, new_opcode, address)
+{
+  new_name("callw");
+}
+char *CALLW::name(char *return_str,int len)
+{
+
+  snprintf(return_str,len,"%s",
+	   gpsimObject::name().c_str());
+  return(return_str);
+}
+void CALLW::execute()
+{
+  if (cpu_pic->stack->push(cpu_pic->pc->get_next()))
+  {
+      cpu_pic->pcl->put(cpu_pic->Wget());
+      cpu_pic->pc->increment();
+  }
+}
+
 //--------------------------------------------------
 CLRF::CLRF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : Register_op(new_cpu, new_opcode, address)
@@ -769,9 +943,12 @@ void CLRF::execute()
 char * CLRF::name(char *return_str,int len)
 {
 
+  source = get_cpu()->registers[register_address];
+  if (access)
+	source = cpu_pic->register_bank[register_address];
   snprintf(return_str,len,"%s\t%s",
            gpsimObject::name().c_str(),
-           get_cpu()->registers[register_address]->name().c_str());
+           source->name().c_str());
 
   return(return_str);
 }
@@ -789,7 +966,7 @@ CLRW::CLRW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 void CLRW::execute()
 {
 
-  cpu_pic->W->put(0);
+  cpu_pic->Wput(0);
 
   cpu_pic->status->put_Z(1);
   cpu_pic->pc->increment();
@@ -840,7 +1017,7 @@ void COMF::execute()
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   cpu_pic->status->put_Z(0==new_value);
 
@@ -871,7 +1048,7 @@ void DECF::execute()
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   cpu_pic->status->put_Z(0==new_value);
 
@@ -902,7 +1079,7 @@ void DECFSZ::execute()
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   if(0==new_value)
     cpu_pic->pc->skip();                  // Skip next instruction
@@ -919,6 +1096,7 @@ GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   switch(cpu_pic->base_isa())
     {
     case _14BIT_PROCESSOR_:
+    case _14BIT_E_PROCESSOR_:
       destination = opcode&0x7ff;
       break;
 
@@ -974,7 +1152,7 @@ void INCF::execute()
   if(destination)
     source->put(new_value);
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
 
   cpu_pic->status->put_Z(0==new_value);
@@ -1006,7 +1184,7 @@ void INCFSZ::execute()
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   if(0==new_value)
     cpu_pic->pc->skip();                  // Skip next instruction
@@ -1028,9 +1206,9 @@ void IORLW::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu_pic->W->value.get() | L;
+  new_value = cpu_pic->Wget() | L;
 
-  cpu_pic->W->put(new_value);
+  cpu_pic->Wput(new_value);
   cpu_pic->status->put_Z(0==new_value);
 
   cpu_pic->pc->increment();
@@ -1055,12 +1233,12 @@ void IORWF::execute()
   else
     source = cpu_pic->register_bank[register_address];
 
-  new_value = source->get() | cpu_pic->W->value.get();
+  new_value = source->get() | cpu_pic->Wget();
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   cpu_pic->status->put_Z(0==new_value);
 
@@ -1070,6 +1248,109 @@ void IORWF::execute()
 
 //--------------------------------------------------
 
+LSLF::LSLF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("lslf");
+}
+
+void LSLF::execute()
+{
+  unsigned int new_value,src_value, carry;
+
+  source = ((!access) ?
+	    cpu_pic->registers[register_address] 
+	    :
+	    cpu_pic->register_bank[register_address] );
+
+  carry = (src_value = source->get()) & 0x80;
+  new_value = (src_value << 1) & 0xff;
+  
+  // Store the result
+
+  if(destination)
+    source->put(new_value);      // Result goes to source
+  else
+    cpu_pic->Wput(new_value);
+
+  cpu_pic->status->put_Z(new_value==0);
+  cpu_pic->status->put_C(carry);
+
+  cpu_pic->pc->increment();
+
+}
+
+
+//--------------------------------------------------
+
+LSRF::LSRF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("lsrf");
+}
+
+void LSRF::execute()
+{
+  unsigned int new_value,src_value, carry;
+
+  source = ((!access) ?
+	    cpu_pic->registers[register_address] 
+	    :
+	    cpu_pic->register_bank[register_address] );
+
+  carry = (src_value = source->get()) & 1;
+  new_value = (src_value & 0xff) >> 1;
+  
+  // Store the result
+
+  if(destination)
+    source->put(new_value);      // Result goes to source
+  else
+    cpu_pic->Wput(new_value);
+
+  cpu_pic->status->put_Z(new_value==0);
+  cpu_pic->status->put_C(carry);
+
+  cpu_pic->pc->increment();
+
+}
+
+
+
+//--------------------------------------------------
+
+MOVLP::MOVLP (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Literal_op(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("movlp");
+}
+
+void MOVLP::execute()
+{
+  if (cpu_pic->pclath->address)
+      cpu_pic->registers[cpu_pic->pclath->address]->put(L);
+  else
+      cpu_pic->pclath->put(L);
+
+  cpu_pic->pc->increment();
+
+}
+
+char * MOVLP::name(char *return_str, int len)
+{
+
+  
+  snprintf(return_str,len,"%s\t%d",
+           gpsimObject::name().c_str(),
+           L&0x7f);
+
+  return(return_str);
+}
+//--------------------------------------------------
+
 MOVLW::MOVLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : Literal_op(new_cpu, new_opcode, address)
 {
@@ -1080,7 +1361,7 @@ MOVLW::MOVLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 void MOVLW::execute()
 {
 
-  cpu_pic->W->put(L);
+  cpu_pic->Wput(L);
 
   cpu_pic->pc->increment();
 
@@ -1111,7 +1392,7 @@ void MOVF::execute()
   if(destination)
     source->put(source_value);
   else
-    cpu_pic->W->put(source_value);
+    cpu_pic->Wput(source_value);
 
 
   cpu_pic->status->put_Z(0==source_value);
@@ -1140,9 +1421,9 @@ void MOVWF::execute()
 {
 
   if(!access)
-    cpu_pic->registers[register_address]->put(cpu_pic->W->get());
+    cpu_pic->registers[register_address]->put(cpu_pic->Wget());
   else
-    cpu_pic->register_bank[register_address]->put(cpu_pic->W->get());
+    cpu_pic->register_bank[register_address]->put(cpu_pic->Wget());
 
   cpu_pic->pc->increment();
 }
@@ -1150,9 +1431,13 @@ void MOVWF::execute()
 char * MOVWF::name(char *return_str, int len)
 {
 
+  
+  source = get_cpu()->registers[register_address];
+  if (access)
+	source = cpu_pic->register_bank[register_address];
   snprintf(return_str,len,"%s\t%s",
            gpsimObject::name().c_str(),
-           get_cpu()->registers[register_address]->name().c_str());
+           source->name().c_str());
 
   return(return_str);
 }
@@ -1197,7 +1482,7 @@ OPTION::OPTION (Processor *new_cpu, unsigned int new_opcode, unsigned int addres
 void OPTION::execute()
 {
 
-  cpu_pic->put_option_reg(cpu_pic->W->get());
+  cpu_pic->put_option_reg(cpu_pic->Wget());
 
   cpu_pic->pc->increment();
 
@@ -1205,6 +1490,20 @@ void OPTION::execute()
 
 //--------------------------------------------------
 
+RESET::RESET (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("reset");
+}
+
+void RESET::execute()
+{
+  cpu_pic->reset(SOFT_RESET);
+}
+
+//--------------------------------------------------
+
 RETLW::RETLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   : Literal_op(new_cpu, new_opcode, address)
 {
@@ -1215,7 +1514,7 @@ RETLW::RETLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 void RETLW::execute()
 {
 
-  cpu_pic->W->put(L);
+  cpu_pic->Wput(L);
 
   cpu_pic->pc->new_address(cpu_pic->stack->pop());
 
@@ -1244,7 +1543,7 @@ void RLF::execute()
   if(destination)
     source->put(new_value&0xff);      // Result goes to source
   else
-    cpu_pic->W->put(new_value&0xff);
+    cpu_pic->Wput(new_value&0xff);
 
   cpu_pic->status->put_C(new_value>0xff);
 
@@ -1276,7 +1575,7 @@ void RRF::execute()
   if(destination)
     source->put(new_value&0xff);      // Result goes to source
   else
-    cpu_pic->W->put(new_value&0xff);
+    cpu_pic->Wput(new_value&0xff);
 
   cpu_pic->status->put_C(old_value&0x01);
 
@@ -1316,14 +1615,14 @@ void SUBWF::execute()
   else
     source = cpu_pic->register_bank[register_address];
 
-  new_value = (src_value = source->get()) - (w_value = cpu_pic->W->value.get());
+  new_value = (src_value = source->get()) - (w_value = cpu_pic->Wget());
 
   // Store the result
 
   if(destination)
     source->put(new_value & 0xff);      // Result goes to source
   else
-    cpu_pic->W->put(new_value & 0xff);
+    cpu_pic->Wput(new_value & 0xff);
 
   cpu_pic->status->put_Z_C_DC_for_sub(new_value, src_value, w_value);
 
@@ -1331,6 +1630,37 @@ void SUBWF::execute()
 
 }
 
+//--------------------------------------------------
+
+SUBWFB::SUBWFB (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
+{
+  decode(new_cpu, new_opcode);
+  new_name("subwfb");
+}
+
+void SUBWFB::execute()
+{
+  unsigned int new_value,src_value,w_value;
+
+  source = ((!access) ?
+	    cpu_pic->registers[register_address] 
+	    :
+	    cpu_pic->register_bank[register_address] );
+
+  new_value = (src_value = source->get()) - (w_value = cpu_pic->Wget()) -
+    (1 - cpu_pic->status->get_C());
+
+  if(destination)
+    source->put(new_value & 0xff);
+  else
+    cpu_pic->Wput(new_value & 0xff);
+
+  cpu_pic->status->put_Z_C_DC_for_sub(new_value, src_value, w_value);
+
+  cpu_pic->pc->increment();
+
+}
 
 //--------------------------------------------------
 
@@ -1355,7 +1685,7 @@ void SWAPF::execute()
   if(destination)
     source->put( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );
   else
-    cpu_pic->W->put( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );
+    cpu_pic->Wput( ((src_value >> 4) & 0x0f) | ( (src_value << 4) & 0xf0) );
 
   cpu_pic->pc->increment();
 
@@ -1385,7 +1715,8 @@ TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
 
     }
   else {
-     if(cpu_pic->base_isa() == _14BIT_PROCESSOR_)
+     if(cpu_pic->base_isa() == _14BIT_PROCESSOR_ ||
+          cpu_pic->base_isa() == _14BIT_PROCESSOR_)
        register_address |= 0x80;  // The destination register is the TRIS
   }
   new_name("tris");
@@ -1396,8 +1727,9 @@ void TRIS::execute()
   if(register_address)
     {
       // Execute the instruction only if the register is valid.
-      if(cpu_pic->base_isa() == _14BIT_PROCESSOR_)
-        cpu_pic->registers[register_address]->put(cpu_pic->W->get());
+      if(cpu_pic->base_isa() == _14BIT_PROCESSOR_ ||
+          cpu_pic->base_isa() == _14BIT_PROCESSOR_)
+        cpu_pic->registers[register_address]->put(cpu_pic->Wget());
       else
         cpu_pic->tris_instruction(register_address);
     }
@@ -1408,9 +1740,10 @@ void TRIS::execute()
 char * TRIS::name(char *return_str,int len)
 {
 
+  source = get_cpu()->registers[register_address];
   snprintf(return_str,len,"%s\t%s",
            gpsimObject::name().c_str(),
-           cpu_pic->registers[register_address]->name().c_str());
+           source->name().c_str());
 
   return(return_str);
 }
@@ -1429,9 +1762,9 @@ void XORLW::execute()
 {
   unsigned int new_value;
 
-  new_value = cpu_pic->W->value.get() ^ L;
+  new_value = cpu_pic->Wget() ^ L;
 
-  cpu_pic->W->put(new_value);
+  cpu_pic->Wput(new_value);
   cpu_pic->status->put_Z(0==new_value);
 
   cpu_pic->pc->increment();
@@ -1456,12 +1789,12 @@ void XORWF::execute()
   else
     source = cpu_pic->register_bank[register_address];
 
-  new_value = source->get() ^ cpu_pic->W->value.get();
+  new_value = source->get() ^ cpu_pic->Wget();
 
   if(destination)
     source->put(new_value);      // Result goes to source
   else
-    cpu_pic->W->put(new_value);
+    cpu_pic->Wput(new_value);
 
   cpu_pic->status->put_Z(0==new_value);
 
