@@ -378,6 +378,10 @@ ProcessorConstructor pP18F2520(P18F2520::construct,
                               "__18F2520",  "pic18f2520",   "p18f2520", "18f2520");
 ProcessorConstructor pP18F2550(P18F2550::construct,
                               "__18F2550",  "pic18f2550",   "p18f2550", "18f2550");
+ProcessorConstructor pP18F2620(P18F2620::construct,
+                              "__18F2620",  "pic18f2620",   "p18f2620", "18f2620");
+ProcessorConstructor pP18F26K22(P18F26K22::construct,
+                              "__18F26K22",  "pic18f26k22",   "p18f26k22", "18f26k22");
 ProcessorConstructor pP18F4221(P18F4221::construct,
                               "__18F4221",  "pic18f4221",   "p18f4221", "18f4221");
 ProcessorConstructor pP18F4321(P18F4321::construct,
@@ -1097,7 +1101,8 @@ pic_processor::pic_processor(const char *_name, const char *_desc)
     Wreg(0), pcl(0), pclath(0),m_PCHelper(0),
     tmr0(this,"tmr0","Timer 0"),
     m_configMemory(0),
-    m_MCLR(0), m_MCLR_Save(0), m_MCLRMonitor(0)
+    m_MCLR(0), m_MCLR_Save(0), m_MCLRMonitor(0),
+    PPLx4(false), clksource(0), clkcontrol(0)
 {
 
 
@@ -1123,7 +1128,8 @@ pic_processor::pic_processor(const char *_name, const char *_desc)
   GetTraceLog().switch_cpus(this);
   m_pResetTT = new ResetTraceType(this);
   m_pInterruptTT = new InterruptTraceType(this);
-
+  for(int i = 0; i < 4; i++)
+	osc_pin_Number[i] = 254;
 }
 //-------------------------------------------------------------------
 pic_processor::~pic_processor()
@@ -1163,6 +1169,9 @@ pic_processor::~pic_processor()
   if (m_MCLRMonitor)
     delete m_MCLRMonitor;
 
+  if (clksource) delete clksource;
+  if (clkcontrol) delete clkcontrol;
+
 }
 //-------------------------------------------------------------------
 //
@@ -1192,7 +1201,6 @@ void pic_processor::create ()
 
   register_bank = &registers[0];  // Define the active register bank
 
-  Vdd = 5.0;                      // Assume 5.0 volt power supply
 
   if(pma) {
     m_PCHelper = new PCHelper(this,pma);
@@ -1246,6 +1254,7 @@ void pic_processor::add_sfr_register(Register *reg, unsigned int addr,
       registers[addr] = reg;
       registers[addr]->address = addr;
       registers[addr]->alias_mask = 0;
+
       if(new_name)
         registers[addr]->new_name(new_name);
 
@@ -1707,7 +1716,9 @@ void WDT::clear()
 void WDT::callback_print()
 {
 
-  cout << "WDT\n";
+  cout <<  name() << " has callback, ID = " << CallBackID << '\n';
+
+ // cout << "WDT\n";
 }
 
 
@@ -1894,7 +1905,7 @@ public:
   IO_SignalControl(char _dir){ direction = _dir; }
   ~IO_SignalControl(){}
   virtual char getState() { return direction; }
-  virtual void release() {delete this;}
+  virtual void release() {}
   void setState(char _dir) { direction = _dir; }
 private:
   char direction;
@@ -1911,8 +1922,6 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
                 PicTrisRegister *m_tris,
                 PicLatchRegister *m_lat)
 {
-  IO_SignalControl *clksource;
-  IO_SignalControl *clkcontrol;
 
   IOPIN *m_pin = package->get_pin(pkg_Pin_Number);
   if (name)
@@ -1931,8 +1940,11 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
 	    if (m_lat)
 		m_lat->setEnableMask(mask);
 	}
-	clksource = new IO_SignalControl('0');
-	clkcontrol = new IO_SignalControl(in ? '1' : '0');
+	if (!clksource)
+	{
+	    clksource = new  PeripheralSignalSource(PinMod);
+	    clkcontrol = new IO_SignalControl(in ? '1' : '0');
+	}
     	PinMod->setSource(clksource);
     	PinMod->setControl(clkcontrol);
  	PinMod->updatePinModule();
@@ -1959,11 +1971,7 @@ void pic_processor::clr_clk_pin(unsigned int pkg_Pin_Number,
 	    if (m_lat)
 		m_lat->setEnableMask(mask);
 	}
-	if (PinMod->getActiveSource())
-	    delete (PinMod->getActiveSource());
     	PinMod->setSource(0);
-	if (PinMod->getActiveControl())
-	    delete (PinMod->getActiveControl());
     	PinMod->setControl(0);
  	PinMod->updatePinModule();
   }
