@@ -99,7 +99,7 @@ void Program_Counter::increment()
   if (value == memory_size) // Some processors start at highest memory and roll over
   {
 	
-        printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, value, memory_size);
+        printf("%s PC=0x%x == memory size 0x%x\n", __FUNCTION__, value, memory_size);
 	value = 0;
   }
   else if (value > memory_size) // assume this is a mistake
@@ -432,3 +432,38 @@ void OPTION_REG::reset(RESET_TYPE r)
 {
   putRV(por_value);
 }
+
+// On 14bit enhanced cores the prescaler does not affect the watchdog
+OPTION_REG_2::OPTION_REG_2(Processor *pCpu, const char *pName, const char *pDesc)
+  : OPTION_REG(pCpu, pName, pDesc)
+{
+}
+
+void OPTION_REG_2::initialize()
+{
+    cpu_pic->tmr0.new_prescale();
+    cpu_pic->option_new_bits_6_7(value.get() & (T0CS | BIT6 | BIT7));
+}
+
+void OPTION_REG_2::put(unsigned int new_value)
+{
+  trace.raw(write_trace.get() | value.get());
+
+  unsigned int old_value = value.get();
+  value.put(new_value);
+
+  // First, check the tmr0 clock source bit to see if we are  changing from
+  // internal to external (or vice versa) clocks.
+  //if( (value ^ old_value) & T0CS)
+  //    cpu_pic->tmr0.new_clock_source();
+
+  // %%%FIX ME%%% - can changing the state of TOSE cause the timer to
+  // increment if tmr0 is being clocked by an external clock?
+
+  // Now check the rest of the tmr0 bits.
+  if( (value.get() ^ old_value) & (T0CS | T0SE | PSA | PS2 | PS1 | PS0))
+    cpu_pic->tmr0.new_prescale();
+
+  if( (value.get() ^ old_value) & (T0CS | BIT6 | BIT7))
+    cpu_pic->option_new_bits_6_7(value.get() & (T0CS | BIT6 | BIT7));
+}
