@@ -18,11 +18,13 @@ void PIE::setPir(PIR *pPir)
 
 void PIE::put(unsigned int new_value)
 {
-  trace.raw(write_trace.get() | value.get());
-  value.put(new_value);
 
   assert(pir);
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value & pir->valid_bits);
+
 
   if(pir->interrupt_status())
     pir->setPeripheralInterrupt();
 }
+
