@@ -208,6 +208,7 @@ int CSimulationContext::LoadProgram(const char *filename,
 				    const char *pProcessorName)
 {
   bool bReturn = false;
+
   Processor *pProcessor;
   FILE * pFile = fopen_path (filename, "rb");
   if(pFile == NULL) {
@@ -315,7 +316,8 @@ void CSimulationContext::NotifyUserCanceled() {
     m_pbUserCanceled = NULL;
     return;
   }
-  if(CSimulationContext::GetContext()->GetActiveCPU()->simulation_mode
+  if(CSimulationContext::GetContext()->GetActiveCPU() && 
+      CSimulationContext::GetContext()->GetActiveCPU()->simulation_mode
     == eSM_RUNNING) {
     // If we get a CTRL->C while processing a command file
     // we should probably stop the command file processing.
