@@ -706,6 +706,8 @@ SSP_MODULE::SSP_MODULE(Processor *pCpu)
     sspstat(pCpu,this),
     sspcon2(pCpu,this),
     sspadd(pCpu,this),
+    m_ssp_if(0),
+    m_bcl_if(0),
     m_pirset(0),
     m_spi(0),
     m_i2c(0),
@@ -756,6 +758,8 @@ SSP_MODULE::~SSP_MODULE()
 	delete m_spi;
 	delete m_i2c;
     }
+    if (m_ssp_if) delete m_ssp_if;
+    if (m_bcl_if) delete m_bcl_if;
 }
 
 SSP1_MODULE::SSP1_MODULE(Processor *pCpu) : 
@@ -2049,6 +2053,18 @@ void SSP_MODULE::initialize(
   }
 
 
+}
+// this allows backward compatibility
+void SSP_MODULE::set_sspif()
+{
+    if (m_ssp_if) m_ssp_if->Trigger();
+    else m_pirset->set_sspif();
+}
+// this allows backward compatibility
+void SSP_MODULE::set_bclif()
+{
+    if (m_bcl_if) m_bcl_if->Trigger();
+    else m_pirset->set_bclif();
 }
 void SSP_MODULE::ckpSPI(unsigned int value)
 {
@@ -2108,8 +2124,6 @@ void SSP_MODULE::stopSSP(unsigned int old_value)
         m_spi->stop_transfer();
         m_sck->setSource(0);
         m_sdo->setSource(0);
-	m_sdo_active = false;
-	m_sck_active = false;
 	m_ss->getPin().newGUIname(m_ss->getPin().name().c_str());
 	m_sdo->getPin().newGUIname(m_sdo->getPin().name().c_str());
 	m_sdi->getPin().newGUIname(m_sdi->getPin().name().c_str());
@@ -2273,32 +2287,16 @@ void SSP_MODULE::changeSSP(unsigned int new_value, unsigned int old_value)
 
 void SSP_MODULE::releaseSDIpin()
 {
-    if (m_sdi)
-    {
-	m_sdi_active = false;
-	delete m_SdiSource;
-	m_SdiSource = 0;
-    }
-    m_sdi = 0;
+    m_sdi_active = false;
 }
 
 void SSP_MODULE::releaseSDOpin()
 {
-    if (m_sdo)
-    {
-	delete m_SdoSource;
-	m_SdoSource = 0;
-    }
-    m_sdo = 0;
+    m_sdo_active = false;
 }
 void SSP_MODULE::releaseSCKpin()
 {
-    if (m_sck)
-    {
-	delete m_SckSource;
-	m_SckSource = 0;
-    }
-    m_sck = 0;
+    m_sck_active = false;
 }
 
 
@@ -2310,8 +2308,6 @@ void SSP_MODULE::releaseSCLpin()
     {
 	m_sck->setSource(0);
 	m_sck_active = false;
-	delete m_SckSource;
-	m_SckSource = 0;
     }
     m_sck = 0;
 }
